
/*****************************************************************/
/** 	Art1.mod						**/
/*****************************************************************/
// Modifications :
//	
//	1) Remove parameters (NslCharString name, NslModule parent) 
//	   from nslModule definition
//	2) Remove parameters (str,parent) from extends Nslmodule
//	3) Art module definition at EOF
//	4) Add ';' at end of import statement
//	5) Add "import nsl_include;" statement 


import nsl_include;
import art;

  
nslModule Comparison(int sizeF1,int sizeF2) extends NslModule ()
{
	public NslDinInt1 in(sizeF1);		// input pattern 
	public NslDoutInt1 x(sizeF1);		// activity and output pattern 
	public NslDinInt1 s(sizeF2);		// input pattern 
	public NslInt0 v(sizeF1);			// top-down template or learned expectation 
	public NslInt2 w(sizeF1,sizeF2);	// LTM weights 
	public NslInt0 rho();				// vigilance parameter 

public void initModule()
{
    
	// initialization of all patterns
	
    x = 0.0;
    s = 0.0;
    v = 0.0;
    
    // initialization of all LTM weights
	
    w = 1.0;
}

public void initTrain()
{
    resetActive = 1;
    active = -1; 
}

public void simTrain()
{
		
	// activation of X from I or from V & I
	   
	// 	I don't use STM equation. I directly calculate activation of X 
	//    if (in.sum() <= 0) // < 1
	
    	if (resetActive == 1) {
    	    	resetActive = 0;
    		active = -1;
		x = in;
    	}
    	else {
		if (s.max() > 0)
    			s.max(active);
        	    		
        	v = NSLprod(w,s);
   
		for (int i = 0;i < in.get_size();i++) {
		    if (in[i] + v[i] >= 1.99)
	    		x[i] = 1.0;
		    else
	    		x[i] = 0.0;
      	}
    	}
}
public void endTrain()
{
	
    // learning of bottom-up and top_down LTM traces 
    
    s.max(active);
    
    for (int i = 0;i < w.get_imax();i++) {
	if (x[i] == 1.0) 
	    w[i][active] = 1.0;
	else
	    w[i][active] = 0.0;
    }   
}
}


// F2: Recognition


nslModule Recognition(int sizeF1,int sizeF2) extends NslModule()
{	
	public NslDinInt1 in(sizeF1);		// input pattern 
	public NslDinInt1 s(sizeF1);		// input pattern to F2 
	public NslDoutInt1 x(sizeF2);		// activity and topdown output pattern 
	public NslInt1 v(sizeF2);			// top-down template, or learned expectation 
	public NslInt2 w(sizeF2,sizeF1);	// LTM weights from F1 to F2 
		
	public NslInt0 resetY(sizeF2);		// list of reset units in F2 
	public NslInt0 rho();				// vigilance parameter 
	public NslInt0 l();					// constant on initializing bottom-up LTM traces 

public void initModule()
{
	
    // initialization of all patterns 
	
    x = 0.0;
    s = 0.0;
    v = 0.0;
    
    // temp
	
    l = 2.0;
	
	
    // initialization of all LTM weights
	
    float max_value = l.elem() / (l.elem() - 1.0 + in.get_size());
    for (int xi = 0; xi < w.get_imax(); xi++) {
	for (int yi = 0; yi < w.get_jmax(); yi++) {
	    w[xi][yi] = uniform_random(float(0.0),max_value);
	}
    }
}
public void initTrain()
{
    resetY = 0.0;
    active = -1;
}

public void simTrain()
{
    if (s.sum() / in.sum() < rho.get_data()) {
	resetY[active] = -1.0;
	active = -1;
    }

    if (active >= 0) {
	nslPrintln("Matching is passed");
	nslBreakSim();
	return;
    }
    	   	
    v = NSLprod(w,s);
    
	num_type	maxvalue;
    int		i;

    active = -1;
    x = 0.0;
	
	// the smallest value in this program 
	
    const float	BIG_MINUS = -1.0; 
	
	
    // To exclude units which have been already reset
	
    for (i = 0;i < resetY.get_size();i++) {
	if (resetY[i] == -1.0) {
	    v[i] = BIG_MINUS;
	}
    }
	
	
    // search for the unit which receives maximum input
	
    maxvalue = v.max();

		
    // In the case that there is no available unit
	
    if (maxvalue == BIG_MINUS) {
	active = -1;
	nslPrint("An error has occured");
	nslBreakSim();
	return;
    }
	
	
    // To find the maximum input
	
    for (i = 0;i < v.get_size();i++) {
	if (v[i] == maxvalue) {
	    x[i] = 1.0;
	    active = i;
	    break;
	}
    }
	
	
    // For the error
	
    if (i >= v.get_size()) {
	nslPrintln("An error has occured");
	nslBreakSim();
	return;
    }
    if (active < 0) {
	    nslPrintln("There are no available units");
	    nslBreakSim();
	    return;
    }
}
public void endTrain()
{
    nslPrintln("Top-Down Template Unit: ",active);

    if (active < 0) {
	nslPrintln("There are no units for this input");
	nslBreakSim();
	return;
    }

	float val = l.get_data() / (l.get_data() - 1.0 + s.sum());
    
    for (int i = 0; i < w.get_jmax(); i++) {
	if (s[i] == 1.0) {
	    w[active][i] = val;
	}
	else {
	    w[active][i] = 0.0;
	}
    }
}
}


nslModule Art(int sizeF1,int sizeF2,int sizeX,int sizeY) extends NslModule () 
{

	public Comparison f1(sizeF1, sizeF2);  	
	public Recognition f2(sizeF1, sizeF2);  	

	public NslDoutInt1 in(sizeF1);
	public NslDinInt1 x(sizeF1);

	private NslInt2 matI(sizeX,sizeY); // to F1 and F2 in 
	private NslInt2 matX(sizeX,sizeY); // from F1 x 

public void makeConn() {
	nslConnect(f1.x,f2.s);
	nslConnect(f2.x,f1.s);
	nslConnect(in,f1.in); 
	nslConnect(in,f2.in); 
	nslConnect(f1.x,x); 
}
public void initTrain() {
    	matrix_vector(matI,in);
}
public void endTrain() {
	vector_matrix(x,matX);
}
}
