// Modifications :
//	
//	1) input -> input1				
//	2) output -> output1
//	3) BackPropAllLayers nslmodule must be declared or defined here

extern nslModule BackPropAllLayers;	//this line is a patch for 3)

nslModule BPForward ()
{
	public NslDinFloat1 fInput();
	public NslDinFloat1 dh();     	
	public NslDinFloat2 dw();
	public NslDoutFloat1 mf();
	public NslDoutFloat2 w();   
		
	protected NslFloat1 mp();
	protected NslFloat1 h();      	
	
public void memAlloc(int inSize,int hidSize)
{
	fInput.memAlloc(inSize);		
	mf.memAlloc(hidSize);		
	mp.memAlloc(hidSize);			      
	h.memAlloc(hidSize);      	  
	dh.memAlloc(hidSize);    		
	w.memAlloc(inSize, hidSize);                         
	dw.memAlloc(inSize, hidSize); 	
}
public void initModule()
{
  	nslRandom(w,-0.5,0.5);
	nslRandom(h,-0.5,0.5);
		
 	dw = 0.0;
  	dh = 0.0;
}	
public void forwardPass()
{
	mp = mat_mult_col_vec(w, fInput);    	
  	mf = sigmoid(mp + h);
}
public void simRun()
{
	forwardPass();
}
public void simTrain()
{
  	w = w + dw;
  	h = h + dh;
	forwardPass();
}
}
nslModule BPBackward ()
{
	public NslDinFloat1 bInput();
	public NslDinFloat1 fInput();
	public NslDinFloat1 mf();
	public NslDoutFloat1 dh();     	
	public NslDoutFloat2 dw();
		
	protected NslFloat1 delta();
	protected NslFloat0 lrate();
	protected NslFloat0 momentum();

public void memAlloc(int inSize,int hidSize)
{
	fInput.memAlloc(inSize);		
	bInput.memAlloc(hidSize);		
	mf.memAlloc(hidSize);		  
	dh.memAlloc(hidSize);    		              
	dw.memAlloc(inSize, hidSize); 	
	delta.memAlloc(hidSize);      	    
}
	
public void backwardPass()
{
 	delta = (mf ^ (1.0 - mf)) ^ bInput;
	dw = lrate * vec_mult_vec_trans(delta, fInput); // + momentum * olddw;
 	dh = lrate * delta; // + momentum * olddh;
}
public void simTrain()
{
	backwardPass();
}

}

nslModule BPBackwardProp () extends BPBackward () 
{
	public NslDoutFloat1 bOutput();
	public NslDoutFloat2 w();   
		
public void memAlloc(int hidSize, int outSize)
{
	super.memAlloc(hidSize,outSize);		
	bOutput.memAlloc(hidSize);
	w.memAlloc(hidSize, outSize);      
}
public void simTrain()
{
	backwardPass();

  	bOutput = mat_trans_mult_col_vec(w, delta);
}
}

nslModule BPBackwardError () 
{
	public NslDinFloat1 mf();      
	public NslDinFloat1 desired();      	
	public NslDoutFloat1 eOutput();      
	
	private NslFloat1 stopError();      
	private NslFloat0 pss();	
	private NslFloat0 tss();

public void memAlloc(int inSize,int hidSize, int outSize)
{
	mf.memAlloc(outSize);			
	desired.memAlloc(outSize);		
	eOutput.memAlloc(outSize);
}	
public void initSys()
{
  	tss = 0.0;
}
public void initTrain()
{
  	pss = 0.0;
}
public void simTrain()
{   	
	eOutput = desired - mf;
  	pss = pss + (eOutput ^ eOutput).sum();
}
public void endTrain()
{
	tss = pss;
	if (tss < stopError) {
		nslPrintln("Convergence");
		nslBreakEpoch();
		return; // to get out from epoch loop
	}
}
}

nslModule BackPropLayers () 
{
	public BPForward fh();
	public BPForward fo();
	public BPBackwardError be();
	public BPBackwardProp bo();
	public BPBackward bh();

	public NslDinFloat1 input1();      
	public NslDinFloat1 desired();      	
	public NslDoutFloat1 output1();      

public void memAlloc(int inSize,int hidSize, int outSize)
{
	input1.memAlloc(inSize);			
	desired.memAlloc(outSize);		
 	output1.memAlloc(outSize);
	
	fh.memAlloc(inSize,hidSize);
	fo.memAlloc(hidSize,outSize);
	bh.memAlloc(inSize,hidSize);
	bo.memAlloc(hidSize,outSize);  
	be.memAlloc(outSize);     
}
public void makeConn()
{
	nslRelabel(input1,fh.fInput);
	nslRelabel(input1,bh.fInput);
	nslRelabel(fo.mf,output1);
	nslRelabel(desired,be.desired);
		
	nslConnect(fh.mf,fo.fInput);
	nslConnect(fh.mf,bo.fInput);
	nslConnect(fh.mf,bh.mf);
	nslConnect(fo.mf,bo.mf);
	nslConnect(fo.mf,be.mf);

	nslConnect(bh.dw,fh.dw);
	nslConnect(bh.dh,fh.dh);

	nslConnect(bo.dw,fo.dw);
	nslConnect(bo.dh,fo.dh);
	nslConnect(fo.w,bo.w);

	nslConnect(be.eOutput,bo.bInput);
	nslConnect(bo.bOutput,bh.bInput);
}		
}

nslModule TrainManager () 
{                   
	public NslDoutFloat1 dInput();
	public NslDoutFloat1 dOutput();

	private NslFloat2 pInput();
	private NslFloat2 pOutput();	
	private NslInt0 numPats();
	private NslString0 fname();	
			
public void readFile(int inSize,int outSize)
{
  	int i,nPats,tmp;
  	int pat;
	nslFile file(); // dummy instance

	if (file.open(fname,INPUT) == -1)
  		nslPrintln("Bad file name: ",fname);
  	else {
  		file.read(nPats);
		memAlloc(nPats,inSize,outSize);
		numPats = nPats;
 	
  	   	for(pat = 0; pat < nPats; ++pat) {
      	      for(i = 0; i < inSize; ++i) {
	  			file.read(tmp);
	  			pInput[pat][i] = tmp;
	      	}
      	      for(i = 0; i < outSize; ++i) {
	  			file.read(tmp);
	  			pOutput[pat][i] = tmp;
	      	}
    	   	}
		file.close();
  	}
}
public void memAlloc(int nPats,int inSize,int outSize)
{
	dInput.memAlloc(inSize);	
	dOutput.memAlloc(outSize);
	pInput.memAlloc(nPats, inSize);
	pOutput.memAlloc(nPats, outSize);
}		
public void simTrain()
{
	int timeStep = SYSTEM.getTrainCurrentTimeStep() - 1;
	int endStep = (int) SYSTEM.getTrainEndTime() + 0.5;
	int pat = timeStep%endStep;
	dInput = pInput.row(pat);
      dOutput = pOutput.row(pat); 
}
}

nslModel BackPropModel
{
	public TrainManager train();
	public BackPropAllLayers layers();
        
	private NslInt0 inSize();
	private NslInt0 hidSize();
	private NslInt0 outSize();

public void initModule()
{
	nslConnect(train.dInput, layers.input1);
	nslConnect(train.dOutput, layers.desired);
}
public void initModule()
{
	layers.memAlloc(inSize.value(),hidSize.value(),outSize.value());
	train.readFile(inSize.value(),outSize.value());
}
}

