// Modifications :
//	1) nslModel <name> () -> nslModel <name>
//	2) extern nslModule....
//	3) makeConn -> void makeConn
//	4) line 564 commented out

extern nslModule DlmH1;
extern nslModule DlmAttention;
extern nslModule DlmH2;
extern nslModule DlmAttention;
extern nslModule DlmSimilarity;
extern nslModule DlmCorrelation;
extern nslModule DlmW12;
extern nslModule DlmW21;
extern nslModule DlmRecognition;
extern nslModule DlmDisplay;



nslModel Dlm 
{
	static const int small_layer = 0;
	static const int small_patch = 1;

	public DlmH1 h1();
	public DlmAttention a1();
	public DlmH2 h2();
	public DlmAttention a2();

	public DlmSimilarity similarity();
	public DlmCorrelation correlation();
	public DlmW12 w12();
	public DlmW21 w21();

	public DlmRecognition recognition();

	public DlmDisplay display();

	public NslDinFloat0 shSum()[gallerySizeMax];       // sum over sh2 ;
	public NslDoutInt0 skipModel()[gallerySizeMax]; // indicates which models to skip ;

	private NslInt0 gallerySize();		
	private NslInt0 loops(); // number of cheap simulation steps ;
	private int loop;    // loop counter ;
	private NslFloat0 preferredLayer();  // number of the layer 2 to show ;
                             // a value of -1 indicates no preferrences ;
	private NslFloat0 showLayer(); // actual number of the layer 2 to show ;
                             // depending on preferredLayer and the ;
                             // activity of the layers ;

	private NslInt0 workOnAverage();  // workOnAverage indicates whether to simulate ... ;
                      // .. currently in average instead of the model gallery ;
	private int modelLoIndex;            // range of layers to be simulated ;
	private int modelHiIndex;

	private NslFloat0 avTimeLimit;    // time for attention dynamics on average model only ;

	void memAlloc(int small_layer, int small_patch)
	{
		int frame;      // width of the frame around layer 1
		int i1max; 		// size of the image layer 1, including frame;
		int j1max;
		int i2max;      // size of the model layers 2 ;
		int j2max;
		int i1Rmax;     // size of the projection patches .. ;
		int j1Rmax;     // ... from model layer 2 to image layer 1 ;

	if (small_layer) {
			frame = 0;          // width of the frame around layer 1 ;
			i1max = 10+2*frame; // size of image layer 1, including frame;
			j1max = 10+2*frame;
		}
		else { // small_layer;
			frame = 2;
			i1max = 17+2*frame;
			j1max = 16+2*frame;
		} // small_layer;

		if (small_patch) {
			i1Rmax= 8;     // size of the projection patches .. ;
			j1Rmax= 8;    // ... from model layer 2 to image layer 1 ;
		}
		else { // small_patch;
			i1Rmax= i1max-2*frame;
			j1Rmax= j1max-2*frame;
		} // small_patch;

		i2max = 10;         // size of the model layers 2
		j2max = 10;

// actual size of the simmulated gallery
// NslFloat0 gallerySize("gallerySize")   

		similarity.memAlloc(frame,i1max,j1max,i2max,j2max,i1Rmax,j1Rmax);
		w12.memAlloc(frame,i1max,j1max,i2max,j2max,i1Rmax,j1Rmax);
		w21.memAlloc(frame,i1max,j1max,i2max,j2max,i1Rmax,j1Rmax);
		correlation.memAlloc(frame,i1max,j1max,i2max,j2max,i1Rmax,j1Rmax);
		a1.memAlloc(i1max,j1max);
		a2.memAlloc(i2max,j2max);
		h1.memAlloc(i1max,j1max);
		h2.memAlloc(i2max,j2max);
		display.memAlloc(frame,i1max,j1max,i2max,j2max,i1Rmax,j1Rmax);
	}
	void makeConn() 
	{
		connect(w12.hInput,h1.hInput);
		connect(a1.sa,h1.sa);
		connect(a2.sa,h2.sa);
		connect(h1.sh,w21.sh);
		connect(h1.sh,correlation.sh1);
		connect(h1.sh,a1.sh);
		connect(h2.shMax,a2.sh);
		for (int i=0; i<gallerySizeMax; i++) {
			connect(similarity.sim[i],w12.sim[i]);
			connect(similarity.sim[i],w21.sim[i]);
			connect(w21.hInput[i],h2.hInput[i]);
			connect(h2.shSum[i],recognition.shSum[i]);
			connect(h2.shSum[i],shSum[i]); // DLM
			connect(h2.sh[i],w12.sh[i]);
			connect(h2.sh[i],correlation.sh2[i]);
			connect(correlation.correlSum[i],w12.correlSum[i]);
			connect(correlation.correlSum[i],w21.correlSum[i]);
			connect(recognition.skipModel[i],w12.skipModel[i]);
			connect(recognition.skipModel[i],w21.skipModel[i]);
			connect(recognition.skipModel[i],correlation.skipModel[i]);
			connect(recognition.skipModel[i],h2.skipModel[i]);
		}
	}
}


nslModule DlmCommon ()
{
	protected int i1Index(int i2, int i1R, 
		int frame,int i1max,int i1Rmax,int i2max) 
	{
		return i1R + frame + int((i1max-2*frame-i1Rmax)*i2/
			float(i2max-1)+0.5);
	}
	protected int j1Index(int j2, int j1R, 
		int frame,int j1max,int j1Rmax,int j2max) 
	{
		return j1R + frame + int((j1max-2*frame-j1Rmax)*j2/
			float(j2max-1)+0.5);
	}

	protected int i1RIndex(int i2, int i1, 
		int frame,int i1max,int i1Rmax,int i2max) 
	{
		return i1 - frame - int((i1max-2*frame-i1Rmax)*i2/
			float(i2max-1)+0.5);
	}
	protected int j1RIndex(int j2, int j1, 
		int frame,int j1max,int j1Rmax,int j2max) 
	{
		return j1 - frame - int((j1max-2*frame-j1Rmax)*j2/
			float(j2max-1)+0.5);
	}

// ==== simple minimum and maximum function ============================ ;

	protected float minimum(float val1, float val2)
	{
		if (val1<val2) return val1; 
		else return val2;
	}

	protected float maximum(float val1, float val2)
	{
		if (val1>val2) return val1; 
		else return val2;
	}
}

nslModule DlmAHCommon () extends DlmCommon ()
{
	protected NslFloat0 rho();  // slope radius of squashing function ;
	protected NslInt0 attention(); // Attention!=0 indicates use of  attention dynamics ;
	protected NslFloat0 sigma_g();      // Gauss width of excitatory kernel ;
	protected NslFloat1 g();      	// Gaussian interaction kernel ;
	protected NslFloat2 temp(); 

	protected float initGauss(NslFloat0& sigma_g,float x)
	{
		return exp(-x*x/(2*sigma_g.elem()*sigma_g.elem()));
	}

	protected void initGauss(NslFloat1& g,NslFloat0& sigma_g)
	{
		for (int k=0; k<g.get_imax(); k++) {
			float x = float(k-g.get_imax()/2);
			g[k] = exp(-x*x/(2*sigma_g.elem()*sigma_g.elem()));
		}
	}
// ---- convolution with Gauss kernel  ---------------------------------- ;
		//NslFloat2
	protected int gaussConvolved(NslFloat1& g,NslFloat2& s,NslFloat0& rho)
	{
		return NSLconv_zero(g.expand_row(1),NSLconv_zero(g.expand_col(1),s));
	}

	protected void gaussConvolved(NslFloat2& out,NslFloat1& g,NslFloat2& s)
	{
		NSLconv_zero_col_equal(out,g,s);
		NSLconv_zero_row_equal(out,g,out);
	}

// ---- output function 'sigma' ----------------------------------- ;

	protected float outputFunc(float h,NslFloat0& rho) 
	{
  		float rhoInv = 1./rho.elem();
  		if (h <=0.)              return 0.;
  		else if (h >=rho.elem()) return 1.;
  		else                     return sqrt(h*rhoInv); 
	}
	protected void computeOutputFunc(NslFloat2& sh, NslFloat2& h, NslFloat0& rho)
	{
  		int i,j;
  		for (i=0; i<sh.get_rows(); i++)
    			for (j=0; j<sh.get_cols(); j++)
      			sh[i][j] = outputFunc(h[i][j],rho);
		}
	}

nslModule DlmH () extends DlmAHCommon ()
{
	public NslDinFloat2 sa();

	protected NslFloat2 hTransE();        // excitatoryly transferred signal ;
	protected NslFloat2 d();

	protected NslFloat0 beta_h();	// global inhibition strength 
	protected NslFloat0 beta_ac();// global inhibition compensation
	protected NslFloat0 kappa_hs(); // self-inhibition strength
	protected NslFloat0 kappa_hh(); // mutual interaction strength 
	protected NslFloat0 kappa_ha(); // attention blob on running blob
	protected NslFloat0 lambda_p(); // decay const for h-s>0 ;
	protected NslFloat0 lambda_m(); // decay const for h-s<0 ;
}

nslModule DlmH1 () extends DlmH ()
{
	public NslDinFloat2 hInput();
	public NslDoutFloat2 sh();

	private NslFloat2 h();
	private NslFloat2 s();
	private int i1max;
	private int j1max;

	public void memAlloc(int _i1max,int _j1max)
	{
		i1max = _i1max;
		j1max = _j1max;
	
		hInput.memAlloc(i1max,j1max);
		hTransE.memAlloc(i1max,j1max);
		h.memAlloc(i1max,j1max);
		s.memAlloc(i1max,j1max);
		d.memAlloc(i1max,j1max);
		sh.memAlloc(i1max,j1max);
		sa.memAlloc(i1max,j1max);
		temp.memAlloc(i1max,j1max);
	}
	public void initRun() 
	{
		hInput=0;
  		h=0;
  		hTransE=0;
  		d=0;
  		s=0;
  		sa=0;
  		sh=0;
		initGauss(g,sigma_g);
	}
	public void simRun()
	{
		int i1,j1;

		if (sh.sum()<=0) { // noise
    			h[imax/2][jmax/2]=0.10;
    			h[imax/4][jmax/2]=0.12;
    			h[imax/2][jmax/4]=0.14;
  		}

		// 3 
		hTransE = gaussConvolved(g,sh,rho);

		if (attention.elem()!=0) {
			// 5a 
			// nslDiff(h,1.,- h + hTransE - beta_h*sh.sum() - kappa_hs*s
			//	+ kappa_hh*hInput + kappa_ha*(sa-beta_ac));

			temp = sa;
			NSLadd_minus(temp,beta_ac);
			NSLmult_equal(temp,kappa_ha,temp);
			NSLmult_plus(temp,kappa_hh,hInput);
			NSLmult_minus(temp,kappa_hs,s);
			NSLmult_minus(temp,beta_h,sh.sum());
			NSLadd_plus(temp,hTransE);
			NSLadd_minus(temp,h);
			nslDiff(h,1.,temp);

		} // if (attention.elem()!=0) ;
		else {
			// 5b 
			//nslDiff(h,1., - h + hTransE - beta_h*sh.sum() - kappa_hs*s
			//	+ kappa_hh* hInput);

			NSLmult_equal(temp,kappa_hh,hInput);
			NSLmult_minus(temp,kappa_hs,s);
			NSLmult_minus(temp,beta_h,sh.sum());
			NSLadd_plus(temp,hTransE);
			NSLadd_minus(temp,h);
			nslDiff(h,1.,temp);

		} // if (attention.elem()!=0) else ;

		// ---- compute 'sh' and 'sh' ---------------------------------- ;
		// ---- needs valid 'h' and 'h' ;

		// 7a 
		computeOutputFunc(sh,h,rho);

		// ---- simulate one time step on 's' and 's' ------------------- ;
		// ---- needs valid 'h' and 'h' ;

		// 8a 
		d = h - s;
		for (i1=0; i1<i1max; i1++)
			for (j1=0; j1<j1max; j1++)
				// 8b 
				if (d[i1][j1]>0)
					d[i1][j1] = d[i1][j1]*lambda_p.elem();
				else
					d[i1][j1] = d[i1][j1]*lambda_m.elem();
		// 8c 
		nslDiff(s,1.,d);
	}
}

nslModule DlmH2 () extends DlmH ()
{
	public NslDinFloat2 hInput()[gallerySizeMax];
	public NslDoutFloat0 shSum()[gallerySizeMax];       // sum over sh2 ;
	public NslDoutFloat2 shMax();	// maximum activity in the layers 2 ;
	public NslDoutFloat2 sh()[gallerySizeMax];

	public NslDinInt0 skipModel()[gallerySizeMax]; // indicates which models to skip ;
	public NslDoutFloat2 shShow();
	public NslDinFloat0 showLayer(); 

	private NslFloat2 h()[gallerySizeMax];
	private NslFloat2 s()[gallerySizeMax];

	private NslFloat0 avTimeLimit();    // time for attention dynamics on average model only ;
	private NslInt0 workOnAverage();  // workOnAverage indicates whether to simulate ... ;
                      // .. currently in average instead of the model gallery ;
	private int modelLoIndex;            // range of layers to be simulated ;
	private int modelHiIndex;

	private NslInt0 gallerySize();		
	private int i2max;
	private int j2max; 

	public void memAlloc(int _i2max,int _j2max)
	{
		i2max = _i2max;
		j2max = _j2max;
	
		hTransE.memAlloc(i2max,j2max);
		d.memAlloc(i2max,j2max);
		sa.memAlloc(i2max,j2max);
		shMax.memAlloc(i2max,j2max);
		temp.memAlloc(i2max,j2max);
		shShow.memAlloc(i2max,j2max);

		for (int i=0; i<gallerySizeMax; i++){
			shSum[i].memAlloc();
			hInput[i].memAlloc(i2max,j2max);
			h[i].memAlloc(i2max,j2max);
			s[i].memAlloc(i2max,j2max);
			sh[i].memAlloc(i2max,j2max);
		}
	}

	public void initRun() 
	{
		int model;
	
		if (workOnAverage.elem()) {
			modelLoIndex=0; 
			modelHiIndex=0;
		} else {
			modelLoIndex=1; 
			modelHiIndex=int(gallerySize.elem());
		}

		for (model=0; model<=gallerySize.elem(); model++) {
			hInput[model] = 0;
			h[model] = 0;
			s[model] = 0;
			sh[model] = 0;
		}
		shMax = 0;
		initGauss(g,sigma_g);
	}
	public void simRun()
	{
		int model,i2,j2;

		// 4 
		hTransE = gaussConvolved(g,shMax,rho);

		if (attention.elem()!=0) {
			// 5a
			for (model=modelLoIndex; model<=modelHiIndex; model++) {
				if (skipModel[model].get_data() == 1) continue;
				// 6a 
				//nslDiff(h[model],1.0, - h[model] + hTransE
				//	- beta_h*shSum[model] - kappa_hs* s[model]
				//	+ kappa_hh* hInput[model] + kappa_ha*(sa-beta_ac));

				temp = sa;
				NSLadd_minus(temp,beta_ac);
				NSLmult_equal(temp,kappa_ha,temp);
				NSLmult_plus(temp,kappa_hh,hInput[model]);
				NSLmult_minus(temp,kappa_hs,s[model]);
				NSLmult_minus(temp,beta_h,shSum[model]);
				NSLadd_plus(temp,hTransE);
				NSLadd_minus(temp,h[model]);
				nslDiff(h[model],1.,temp);

			}
		} // if (attention.elem()!=0) ;
		else {
			// 5b
			for (model=modelLoIndex; model<=modelHiIndex; model++) {
				if (skipModel[model].get_data() == 1) continue;
				// 6b 
				//nslDiff(h[model],1.,- h[model] + hTransE
				//	- beta_h*shSum[model] - kappa_hs* s[model]
				//	 + kappa_hh* hInput[model]);

				NSLmult_equal(temp,kappa_hh,hInput[model]);
				NSLmult_minus(temp,kappa_hs,s[model]);
				NSLmult_minus(temp,beta_h,shSum[model]);
				NSLadd_plus(temp,hTransE);
				NSLadd_minus(temp,h[model]);
				nslDiff(h[model],1.,temp);
			}
		}
  
		for (model=modelLoIndex; model<=modelHiIndex; model++) {
			if (skipModel[model].get_data() == 1) continue;
			// 7b 
			computeOutputFunc(sh[model],h[model],rho);
		}

		// ---- compute 'shMax' ----------------------------------------- ;
		// ---- needs valid 'sh' ;

		for (i2=0; i2<i2max; i2++)
			for (j2=0; j2<j2max; j2++) {
				shMax[i2][j2] = 0;
				for (model=modelLoIndex; model<=modelHiIndex; model++) {
					if (skipModel[model].get_data() == 1) continue;
					// 7c 
					shMax[i2][j2] = 
						maximum(shMax[i2][j2],sh[model][i2][j2]);
				}
			}

		//  shSum = 0;
		for (model=modelLoIndex; model<=modelHiIndex; model++) {
			if (skipModel[model].get_data() == 1) 
				shSum[model] = 0; // continue;
			else // 7d 
				shSum[model] = sh[model].sum();
		}

		for (model=modelLoIndex; model<=modelHiIndex; model++) {
			if (skipModel[model].get_data() == 1) continue;
			// 9a 
			d = h[model] - s[model];
			for (i2=0; i2<i2max; i2++)
				for (j2=0; j2<j2max; j2++)
					// 9b 
					if (d[i2][j2]>0)
						d[i2][j2] = d[i2][j2]*lambda_p.elem();
					else
						d[i2][j2] = d[i2][j2]*lambda_m.elem();
			// 9c 
			nslDiff(s[model],1.,d);
		}

		if (nslSystem.getSimTime()>avTimeLimit.elem())
			workOnAverage.elem() = 0;

		if (workOnAverage.elem()) {
			modelLoIndex=0; 
			modelHiIndex=0;
		} else {
			modelLoIndex=1; 
			modelHiIndex=int(gallerySize.elem());
		}
	}
}
nslModule DlmAttention () extends DlmAHCommon ()
{
	public NslDoutFloat2 sa();      
	public NslDinFloat2 sh();	

	private NslFloat2 a();       // attention blob on layers 1 ;
	private NslFloat2 aTransE(); // excitatoryly transfered signal ;
	private NslFloat0 lambda_a();// time constant for the attention dynamics ;
	private NslFloat0 beta_a();  // strength of global inhibition for attention blob ;
	private NslFloat0 kappa_ah();// effect of running blob on attention blob ;
	private NslFloat0 alpha_N();   // attention blob init (only on h1)

	public void memAlloc(int imax,int jmax)
	{
		a.memAlloc(imax,jmax);
		sa.memAlloc(imax,jmax);
		aTransE.memAlloc(imax,jmax);
		sh.memAlloc(imax,jmax);
		temp.memAlloc(imax,jmax);
	}

	public void initRun() 
	{
		if (attention.elem()) {
			a = alpha_N; // a2: alpha_N = 0;
			computeOutputFunc(sa,a,rho); // a2: sa = 0;
			initGauss(g,sigma_g);
		} // if (attention.elem()!=0) ;
	}
	public void simRun()
	{
		if (attention.elem()!=0) {
			// 10, 12a
			// aTransE = gaussConvolved(g,sa,rho);
			gaussConvolved(aTransE,g,sa);

			// 11, 12b
			// nslDiff(a1,1.,lambda_a * (- a + aTransE  - beta_a*sa.sum() + kappa_ah*sh));
			NSLmult_equal(temp,kappa_ah,sh);
			NSLmult_minus(temp,beta_a,sa.sum());
			NSLadd_plus(temp,aTransE);
			NSLadd_minus(temp,a);
			NSLmult_equal(temp,lambda_a,temp);
			nslDiff(a,1.,temp);

			// 13 
			computeOutputFunc(sa,a,rho);
		} // if (attention.elem()!=0)
	}
}

nslModule DlmRecognition () extends DlmCommon ()
{
	public NslDinFloat0 shSum() [gallerySize];  // sum over sh2
	public NslDoutInt0 skipModel() [gallerySize]; // models to skip

	private NslFloat1 rec[gallerySizeMax];    // recognition variable

	private NslFloat0 lambda_r();  // time constant for dynamics
	private NslFloat0 r_theta();   // threshold for model suppression

	private NslFloat0 avTimeLimit();    // time for attention dynamics on average model only ;
	private NslInt0 workOnAverage();  // workOnAverage indicates whether to simulate ... ;
                      // .. currently in average instead of the model gallery ;
	private int modelLoIndex;            // range of layers to be simulated ;
	private int modelHiIndex;

	private NslInt0 gallerySize();		

	public void initRun()
	{
		if (workOnAverage.elem()) {
			modelLoIndex=0; 
			modelHiIndex=0;
		} else {
			modelLoIndex=1; 
			modelHiIndex=int(gallerySize.elem());
		}

		rec[0] = 0.;
		for (int model=1; model<=gallerySize.elem(); model++)
			rec[model] = 1.;
	}
	public void simRun()
	{
		int model;
		float recshSumMax;            // temporary variable ;
	  
		if (!workOnAverage.elem()) {
			recshSumMax = 0;
			for (model=modelLoIndex; model<=modelHiIndex; model++)
				// 22 
				recshSumMax = 
			maximum(recshSumMax,rec[model].elem()*shSum[model].elem());
			// 23 
			for (model=1; model<=gallerySize.elem(); model++)
				nslDiff(rec[model],1.,lambda_r*rec[model]^(shSum[model] - recshSumMax));
		} // if (!workOnAverage.elem()) ;

  // ---- determine which models to skip ------------------------------- ;
  // ---- needs valid 'rec' ;
  
		for (model=modelLoIndex; model<=modelHiIndex; model++) {
			if (skipModel[model].get_data() == 1) continue;
			if (model>0 && rec[model] <= r_theta.elem()) {
				skipModel[model] = 1;
			}
		}
  
    // ---- print the current recognition status: ----------------------- ;
    // ---- time, showed layer, recognition values ;

    		for (model=1; model<=gallerySize.elem(); model++)
			if (rec[model]>r_theta.elem())
				NSLoutput(" ");
			else
				NSLoutput(" *");
		NSLoutput("_");

		if (nslSystem.getSimTime()>avTimeLimit.elem())
			workOnAverage.elem() = 0;

		if (workOnAverage.elem()) {
			modelLoIndex=0; 
			modelHiIndex=0;
		} else {
			modelLoIndex=1; 
			modelHiIndex=int(gallerySize.elem());
		}
	}
}

nslModule DlmW () extends DlmCommon ()
{
	public NslDinFloat4 sim()[gallerySizeMax]; 
	public NslDinFloat4 correlSum()[gallerySizeMax];
	public NslDinInt0 skipModel()[gallerySizeMax]; // indicates which models to skip ;
	public NslDinFloat0 showLayer(); 
	public NslDoutFloat4 wShow();		

	protected NslFloat4 w()[gallerySizeMax];
	protected NslFloat0 lambda_W();             
	protected NslFloat0 wSums()[gallerySizeMax];

	protected NslInt0 workOnAverage();  // workOnAverage indicates whether to simulate ... ;
                      // .. currently in average instead of the model gallery ;
	protected int modelLoIndex;            // range of layers to be simulated ;
	protected int modelHiIndex;

	protected NslFloat0 avTimeLimit();    // time for attention dynamics on average model only ;

	protected NslInt0 gallerySize();		
	protected NslInt0 loops();	// number of cheap simulation steps ;
	protected int loop;		// loop counter ;
	protected int frame;
	protected int i1max;
	protected int j1max;
	protected int i2max;
	protected int j2max;
	protected int i1Rmax;
	protected int j1Rmax; 
}
nslModule DlmW12 () extends DlmW ()
{
	public NslDinFloat2 sh() [gallerySizeMax];
	public NslDoutFloat2 hInput();

	private NslFloat2 normFactor();

	public void memAlloc(int _frame, int _i1max,int _j1max,int _i2max,
		int _j2max,int _i1Rmax,int _j1Rmax)
	{
		frame = _frame;
		i1max = _i1max;
		j1max = _j1max;
		i2max = _i2max;
		j2max = _j2max;
		i1Rmax = _i1Rmax;
		j1Rmax = _j1Rmax;
	
		normFactor.memAlloc(i1max,j1max);
		hInput.memAlloc(i1max,j1max);

		for (int i=0; i<gallerySizeMax;i++) {
			sim[i].memAlloc(i2max,j2max,i1Rmax,j1Rmax);
			w[i].memAlloc(i1Rmax,j1Rmax,i2max,j2max);
			sh[i].memAlloc(i2max,j2max);
			correlSum[i].memAlloc(i2max,j2max,i1Rmax,j1Rmax);
		}
	}

	public void initRun()
	{
		int model,i2,ij2,j2,i1R,i1,j1,j1R,ij1R;
	
		if (workOnAverage.elem()) {
			modelLoIndex=0; 
			modelHiIndex=0;
		} else {
			modelLoIndex=1; 
			modelHiIndex=int(gallerySize.elem());
		}

		if (gallerySize.elem()>0) {
		// ---- copy 's1' to 'W21' and 'W12' ;
			for (model=1; model<=gallerySize.elem(); model++) {
				for (i2=0; i2<i2max; i2++)
					for (j2=0; j2<j2max; j2++)
						for (i1R=0; i1R<i1Rmax; i1R++)
							for (j1R=0; j1R<j1Rmax; j1R++)
								w[model][i1R][j1R][i2][j2] = 
								sim[model][i2][j2][i1R][j1R];
    			}

   		float stmp;
    		if (gallerySize.elem()>0)
			for (i2=0; i2<i2max; i2++)
				for (j2=0; j2<j2max; j2++)
					for (i1R=0; i1R<i1Rmax; i1R++)
						for (j1R=0; j1R<j1Rmax; j1R++) {
	  						stmp = 0;
	  						for (model=1; 
					model<=gallerySize.elem(); model++)
	    						stmp = 		
					maximum(stmp,sim[model][i2][j2][i1R][j1R]);
							w[0][i1R][j1R][i2][j2] = stmp;
						}
    			// ---- initialize W21Sums ;
    		for (model=0; model<=gallerySize.elem(); model++)
			wSums[model] = 
				w[model].sum()/(i1Rmax*j1Rmax*i2max*j2max);
  		}
	}
	public void simRun()
	{
		int model,i2,j2,i1R,i1,j1,j1R;

		hInput=0;
		for (model=modelLoIndex; model<=modelHiIndex; model++) {
			if (skipModel[model].get_data() == 1) continue;
			for (i2=0; i2<i2max; i2++)
				for (j2=0; j2<j2max; j2++) {
					if (sh[model][i2][j2]<=0.) continue;
					for (i1R=0, i1=i1Index(i2, 0,frame,i1max,i1Rmax,i2max); i1R<i1Rmax; i1R++, i1++)
						for (j1R=0, j1=j1Index(j2, 0,frame,j1max,j1Rmax,j2max); j1R<j1Rmax; j1R++, j1++) 
						// 1 
							hInput[i1][j1] = maximum(hInput[i1][j1],
								w[model][i1R][j1R][i2][j2]*sh[model][i2][j2]);
			}
	}

	if (loop<=0) {
		loop=(int)(loops.elem());

		if (!workOnAverage.elem()) {
			if (lambda_W.elem()!=0) {
			// ---- modify weights 'W21' and 'W12' ------------------------- ;
				normFactor = 1.;
      
				for (model=modelLoIndex; model<=modelHiIndex; model++) {
					if (skipModel[model].get_data() == 1) continue;
					for (i2=0; i2<i2max; i2++)
						for (j2=0; j2<j2max; j2++)
							for (i1R=0, i1=i1Index(i2,0,frame,i1max,i1Rmax,i2max); i1R<i1Rmax; i1R++, i1++)
								for (j1R=0, j1=j1Index(j2,0,frame,j1max,j1Rmax,j2max); j1R<j1Rmax; j1R++, j1++) {	
									// --- let weights grow ;
									// 16 
									w[model][i1R][j1R][i2][j2] += nslSystem.getSimDelta()
										*w[model][i1R][j1R][i2][j2]
										*lambda_W.elem()*correlSum[model][i2][j2][i1R][j1R];
		
									// ---- determine normalization factors ;
									if (w[model][i1R][j1R][i2][j2]>sim[model][i2][j2][i1R][j1R]) {
									// 17 
										normFactor[i1][j1] = minimum(normFactor[i1][j1],
											sim[model][i2][j2][i1R][j1R]/w[model][i1R][j1R][i2][j2]);
									}
								} // for (j1R=...) ;
				} // for (model=...) ;
      
				// ---- normalize weights ------------------------------------------ ;
      
				for (i2=0; i2<i2max; i2++)
					for (j2=0; j2<j2max; j2++)
						for (i1R=0, i1=i1Index(i2,0,frame,i1max,i1Rmax,i2max); i1R<i1Rmax; i1R++, i1++)
							for (j1R=0, j1=j1Index(j2,0,frame,j1max,j1Rmax,j2max); j1R<j1Rmax; j1R++, j1++) {
								for (model=modelLoIndex; model<=modelHiIndex; model++) {
									// 20 
									w[model][i1R][j1R][i2][j2] *= normFactor[i1][j1];
								}
							}
      
			} // if (lambda_W.elem()!=0) ;
      
			// ---- compute normed total weights of the connectivities ---------- ;

			// 'W21' does not vary for layers not simulated any more
			//(skipModel==1) while 'W12' does.  The reason is that there is
			//no competition between links going to different models while
			//there is competition between links converging to layer 1 from
			//different models.  That is the reason why the computation of
			//'W21Sums' comes after the continue statement. 

			for (model=modelLoIndex; model<=modelHiIndex; model++) {
				wSums[model] = w[model].sum()/(i1Rmax*j1Rmax*i2max*j2max);
				if (skipModel[model].get_data() == 1) continue;
			}  
		} // if (!workOnAverage.elem()) ;
    
		if (nslSystem.getSimTime()>avTimeLimit.elem())
			workOnAverage.elem() = 0;

		if (workOnAverage.elem()) {
			modelLoIndex=0; 
			modelHiIndex=0;
		} else {
			modelLoIndex=1; 
			modelHiIndex=int(gallerySize.elem());
		}
	} // if (loop<=0)

	loop--;
}
}
nslModule DlmW21 () extends DlmW ()
{
	public NslDinFloat2 sh();
	public NslDoutFloat2 hInput()[gallerySizeMax];

	private NslFloat2 normFactor()[gallerySizeMax]; 	

public void memAlloc(int _frame, int _i1max,int _j1max,int _i2max,int _j2max,int _i1Rmax,int _j1Rmax)
{
	frame = _frame;
	i1max = _i1max;
	j1max = _j1max;
	i2max = _i2max;
	j2max = _j2max;
	i1Rmax = _i1Rmax;
	j1Rmax = _j1Rmax;
	
	sh.memAlloc(i1max,j1max);
	wShow.memAlloc(i2max,j2max,i1Rmax,j1Rmax);

	for (int i=0; i<gallerySizeMax; i++) {
		sim[i].memAlloc(i2max,j2max,i1Rmax,j1Rmax);
		w[i].memAlloc(i2max,j2max,i1Rmax,j1Rmax);
		hInput[i].memAlloc(i2max,j2max);
		correlSum[i].memAlloc(i2max,j2max,i1Rmax,j1Rmax);
		normFactor[i].memAlloc(i2max,j2max);
	}
}

public void initRun()
{
	int model,i2,ij2,j2,i1R,i1,j1,j1R,ij1R;
	
	if (workOnAverage.elem()) {
		modelLoIndex=0; 
		modelHiIndex=0;
	} else {
		modelLoIndex=1; 
		modelHiIndex=int(gallerySize.elem());
	}

	if (gallerySize.elem()>0) {
		for (model=1; model<=gallerySize.elem(); model++) {
			w[model] = sim[model]; 
		}

    // ---- initialize 'W21', for average layer ;
		float stmp;
		if (gallerySize.elem()>0)
			for (i2=0; i2<i2max; i2++)
				for (j2=0; j2<j2max; j2++)
					for (i1R=0; i1R<i1Rmax; i1R++)
						for (j1R=0; j1R<j1Rmax; j1R++) {
	  						stmp = 0;
	  						for (model=1; model<=gallerySize.elem(); model++)
	    						stmp = maximum(stmp,sim[model][i2][j2][i1R][j1R]);
	  						w[0][i2][j2][i1R][j1R] = stmp;
						}
    // ---- initialize W21Sums ;
		for (model=0; model<=gallerySize.elem(); model++) 
			wSums[model] = w[model].sum()/(i1Rmax*j1Rmax*i2max*j2max);
    
	}
}
public void simRun()
{
	int model,i2,j2,i1R,i1,j1,j1R;

  // ---- compute layer input 'hInput' propagated via 'W21' ---------- ;

	for (model=modelLoIndex; model<=modelHiIndex; model++) {
		if (skipModel[model].get_data() == 1) continue;
    		hInput[model]=0;
	}

	for (i1=frame; i1<i1max-frame; i1++)
		for (j1=frame; j1<j1max-frame; j1++) {
			if (sh[i1][j1]<=0.) continue;
			for (model=modelLoIndex; model<=modelHiIndex; model++) {
				if (skipModel[model].get_data() == 1) continue;
				for (i2=0; i2<i2max; i2++) {
					i1R = i1RIndex(i2,i1,frame,i1max,i1Rmax,i2max);
					if ((i1R<0)||(i1R>=i1Rmax)) continue;
					for (j2=0; j2<j2max; j2++) {
						j1R = j1RIndex(j2,j1,frame,j1max,j1Rmax,j2max);
						if ((j1R<0)||(j1R>=j1Rmax)) continue;
						// 2 
						hInput[model][i2][j2] = maximum(hInput[model][i2][j2],
							w[model][i2][j2][i1R][j1R]*sh[i1][j1]);
					}
				}
			}
		}

	if (loop<=0) {
		loop=(int)(loops.elem());

		if (!workOnAverage.elem()) {
			if (lambda_W.elem()!=0) {

				// ---- modify weights 'W21' and 'W12' ------------------------- ;
				for (model=modelLoIndex; model<=modelHiIndex; model++) {
					if (skipModel[model].get_data() == 1) continue;
					normFactor[model] = 1.; // *(normFactor[model]) = 1.;	
					for (i2=0; i2<i2max; i2++)
						for (j2=0; j2<j2max; j2++)
							for (i1R=0, i1=i1Index(i2,0,frame,i1max,i1Rmax,i2max); i1R<i1Rmax; i1R++, i1++)
								for (j1R=0, j1=j1Index(j2,0,frame,j1max,j1Rmax,j2max); j1R<j1Rmax; j1R++, j1++) {
									// --- let weights grow ;
									// 15 
									w[model][i2][j2][i1R][j1R] += nslSystem.getSimDelta()
										*w[model][i2][j2][i1R][j1R]
										*lambda_W.elem()*correlSum[model][i2][j2][i1R][j1R];
		
									// ---- determine normalization factors ;
									if (w[model][i2][j2][i1R][j1R]>sim[model][i2][j2][i1R][j1R]) {
										// 18 
										normFactor[model][i2][j2] = minimum(normFactor[model][i2][j2],
											sim[model][i2][j2][i1R][j1R]/w[model][i2][j2][i1R][j1R]);
									}
								} // for (j1R=...) ;
				} // for (model=...) ;
      
			// ---- normalize weights ------------------------------------------ ;
      
				for (i2=0; i2<i2max; i2++)
					for (j2=0; j2<j2max; j2++)
						for (i1R=0, i1=i1Index(i2,0,frame,i1max,i1Rmax,i2max); i1R<i1Rmax; i1R++, i1++)
							for (j1R=0, j1=j1Index(j2,0,frame,j1max,j1Rmax,j2max); j1R<j1Rmax; j1R++, j1++) {
								for (model=modelLoIndex; model<=modelHiIndex; model++) {
									// 19 
									w[model][i2][j2][i1R][j1R] *= normFactor[model][i2][j2]; 
								}
							}
			} // if (lambda_W.elem()!=0) ;
      
			// ---- compute normed total weights of the connectivities ---------- ;

			// 'W21' does not vary for layers not simulated any more
			//(skipModel==1) while 'W12' does.  The reason is that there is
			//no competition between links going to different models while
			//there is competition between links converging to layer 1 from
			//different models.  That is the reason why the computation of
			//'W21Sums' comes after the continue statement. 

			for (model=modelLoIndex; model<=modelHiIndex; model++) {
				if (skipModel[model].get_data() == 1) continue;
				wSums[model] = w[model].sum()/(i1Rmax*j1Rmax*i2max*j2max);
			}
		} // if (!workOnAverage.elem()) ;

		if (nslSystem.getSimTime()>avTimeLimit.elem())
			workOnAverage.elem() = 0;

		if (workOnAverage.elem()) {
			modelLoIndex=0; 
			modelHiIndex=0;
		} else {
			modelLoIndex=1; 
			modelHiIndex=int(gallerySize.elem());
		}
	} // if (loop<=0) 

	loop--;
}

}

nslModule DlmCorrelation () extends DlmCommon () 
{
	public NslDinFloat2 sh1();
	public NslDinFloat2 sh2()[gallerySizeMax];
	public NslDoutFloat4 correlSum()[gallerySizeMax];
	public NslDoutFloat4 correlLI()[gallerySizeMax];

	public NslDinInt0 skipModel()[gallerySizeMax]; // indicates which models to skip ;
	public NslDoutFloat4 correlLIShow();
	public NslDinFloat0 showLayer(); 

	private NslFloat0 lambda_c();   // rate for the leaky correlation integrater ;
	                        
	private NslInt0 workOnAverage();  // workOnAverage indicates whether to simulate ... ;
                      // .. currently in average instead of the model gallery ;
	private int modelLoIndex;            // range of layers to be simulated ;
	private int modelHiIndex;

	private NslFloat0 avTimeLimit();    // time for attention dynamics on average model only ;

	private NslInt0 gallerySize();		
	private NslInt0 loops();	// number of cheap simulation steps ;
	private int loop;		// loop counter ;
	private int frame;
	private int i1max;
	private int j1max;
	private int i2max;
	private int j2max;
	private int i1Rmax;
	private int j1Rmax; 

	private NslFloat4 temp();

public void memAlloc(int _frame, int _i1max,int _j1max,int _i2max,int _j2max,int _i1Rmax,int _j1Rmax)
{
	frame = _frame;
	i1max = _i1max;
	j1max = _j1max;
	i2max = _i2max;
	j2max = _j2max;
	i1Rmax = _i1Rmax;
	j1Rmax = _j1Rmax;

	sh1.memAlloc(i1max,j1max);
	temp.memAlloc(i2max,j2max,i1Rmax,j1Rmax);
	correlLIShow.memAlloc(i2max,j2max,i1Rmax,j1Rmax);

	for (int i=0; i<gallerySizeMax;i++){
		sh2[i].memAlloc(i2max,j2max);
		correlSum[i].memAlloc(i2max,j2max,i1Rmax,j1Rmax);
		correlLI[i].memAlloc(i2max,j2max,i1Rmax,j1Rmax);
	}
}
public void initRun()
{
	int model;
	
	if (workOnAverage.elem()) {
		modelLoIndex=0; 
		modelHiIndex=0;
	} else {
		modelLoIndex=1; 
		modelHiIndex=int(gallerySize.elem());
	}

     //The correlation matrices 'correl21LI' are initialized with a very
     //small value, so that the net display can work in the
     //beginning. 

	for (model=0; model<=gallerySize.elem(); model++)
		correlLI[model] = 0.00001;

	for (model=0; model<=gallerySize.elem(); model++)
		correlSum[model] = 0;
}
public void simRun()
{
	int model,i2,j2,i1R,i1,j1,j1R;
	
	for (model=modelLoIndex; model<=modelHiIndex; model++) {
		if (skipModel[model].get_data() == 1) continue;
		for (i2=0; i2<i2max; i2++)
			for (j2=0; j2<j2max; j2++) {
				if (sh2[model][i2][j2]<=0.) continue;
				for (i1R=0, i1=i1Index(i2, 0,frame,i1max,i1Rmax,i2max); i1R<i1Rmax; i1R++, i1++)
					for (j1R=0, j1=j1Index(j2, 0,frame,j1max,j1Rmax,j2max); j1R<j1Rmax; j1R++, j1++) {
						// 14 
						correlSum[model][i2][j2][i1R][j1R] += sh1[i1][j1]*sh2[model][i2][j2];
					}
			}
	}

	// ---- simulate one time step on 'correl21LI' ------------------------ ;
	// ---- needs valid 'correl21Sum' ;

	for (model=modelLoIndex; model<=modelHiIndex; model++) {
		if (skipModel[model].get_data() == 1) continue;
//				correlLI[model] = correlLI[model]+nslSystem.getSimDelta()*lambda_c.elem()
//				*(correlSum[model] -loops.elem()* correlLI[model]);

		NSLmult_equal(temp,-loops.elem(),correlLI[model]);
		NSLadd_plus(temp,correlSum[model]);
		NSLmult_equal(temp,lambda_c.elem(),temp);
		NSLmult_plus(correlLI[model],nslSystem.getSimDelta(),temp);
	}

	if (loop<=0) {
		loop=(int)(loops.elem());

		for (model=modelLoIndex; model<=modelHiIndex; model++) {
			if (skipModel[model].get_data() == 1) continue;
			// 21 
			correlSum[model] = 0;
		}   

		if (nslSystem.getSimTime()>avTimeLimit.elem())
			workOnAverage.elem() = 0;

		if (workOnAverage.elem()) {
			modelLoIndex=0; 
			modelHiIndex=0;
		} else {
			modelLoIndex=1; 
			modelHiIndex=int(gallerySize.elem());
		}

	} // if (loop<=0)
	loop--;
}
}

nslModule DlmSimilarity () extends DlmCommon ()
{
	public NslDoutFloat4 sim()[gallerySizeMax]; // similarity matrix from layer 1 to layers 2 ;
	public NslFloat1 DlmImageModelData()[gallerySizeMax];

	private NslFile DlmObjects();
	private NslFile DlmGraph();
	private NslFile DlmModels();

	private NslFloat1 DlmGImage(); 
	private NslFloat2 ImageNormChar();
	private NslFloat1 DlmGModel();
	private NslFloat2 ModelNormFloat();
	private NslFloat2 ModelNormChar();
	private NslInt0 gallerySize();		

	private int frame;      // width of the frame around layer 1
	private int i1max; 	// size of the image layer 1, including frame;
	private int j1max;
	private int i2max;      // size of the model layers 2 ;
	private int j2max;
	private int i1Rmax;     // size of the projection patches .. ;
	private int j1Rmax;     // ... from model layer 2 to image layer 1 ;
	private int Si2offset;
	private int Sj2offset;
	private int Si1offset;
	private int Sj1offset;
	private int Oi1offset;
	private int Oj1offset;
	private int selectedObject;
	private NslString0 objectsFileName;
	private NslString0 modelsFileName;
	private NslString0 objectName;                // name of the image graph file ;

	private int kLevels;
	private int kDirections;
	private int kLevelsTmp;
	private int kDirectionsTmp;
	private float kMaximum;
	private float kFactor;
	private float sigma;
	private float kMaximumTmp;
	private float kFactorTmp;
	private float sigmaTmp;
	private int tablesCreated;

	private NslFloat1 kxArr();
	private NslFloat1 kyArr();
	private NslFloat1 confidenceBak();
	private NslFloat1 deltaPhaseBak();

	private NslFloat0 ObjectSelectionMode();
	private NslFloat0 preferredObject();
	private NslFloat0 alpha_S(); // minimal synaptic weight ;

	private NslString0 modelName()[gallerySizeMax];     // names of model 

public void memAlloc(int _frame,int _i1max,int _j1max,
	int _i2max,int _j2max,int _i1Rmax,int _j1Rmax)
{
	frame = _frame;
	i1max = _i1max;
	j1max = _j1max;
	i2max = _i2max;
	j2max = _j2max;
	i1Rmax = _i1Rmax;
	j1Rmax = _j1Rmax;

	tablesCreated = 0;

	DlmGImage.memAlloc(i1max*j1max*kLevels*kDirections*2);
	ImageNormChar.memAlloc(i1max,j1max);
	DlmGModel.memAlloc(i1max*j1max*kLevels*kDirections*2);
	ModelNormFloat.memAlloc(i2max,j2max);
	ModelNormChar.memAlloc(i2max,j2max);

	kxArr.memAlloc(kLevels*kDirections);
	kyArr.memAlloc(kLevels*kDirections);
	confidenceBak.memAlloc(kLevels*kDirections);
	deltaPhaseBak.memAlloc(kLevels*kDirections);

	for (int i=0; i<gallerySizeMax; i++) {
  		sim[i].memAlloc(i2max,j2max,i1Rmax,j1Rmax);
		DlmImageModelData[i].memAlloc(i1max*j1max*kLevels*kDirections*2);
	}
}

public void initModule()
{
	selectedObject=0;

	strcpy(objectsFileName,"Dlmobjects");
	strcpy(modelsFileName,"Dlmmodels");

	kLevels=5; kDirections=8;
	kMaximum=1.5708; kFactor=0.707107; sigma=6.28318;
	tablesCreated=0;
}

// ---- function to create tables for the similarity function ;

private void createTables()
{
  	float k0, k0Angle;
  	int kLevel, kDirection, kIndex;
  	for (kLevel=0; kLevel<kLevels; kLevel++) {
		k0 = kMaximum * pow(kFactor, kLevel);
    	for (kDirection=0; kDirection<kDirections; kDirection++) {
      		kIndex = (kLevel*kDirections+kDirection);
      		k0Angle = kDirection*M_PI/kDirections;
      		kxArr[kIndex] = k0*cos(k0Angle);
      		kyArr[kIndex] = k0*sin(k0Angle);
    	}

  	}

  	tablesCreated=1;
}

 // The similarity function takes two jets and provides similarity and
 // distance information in fsData. The neural system is not dependent
 // on the concrete form of the features and comparison function. And
 // since this comparison function is a bit complicated, the precise
 // algorithm will be described in a different publication. But you can
 // choose a simpler comparison function that does not use the phase
 // information, see the <#define PHASE> statement above. 

private float similarity(NslFloat1& trafoData1,int i1, int j1, NslFloat1& trafoData2,int i2, int j2)
{
  	int kLevel, kIndex, kIndex2;
  	float k0x, k0y, deltaPhase; 
  	float dispX=0, dispY=0, confidence, meanError=0;
  	float PhiX=0, PhiY=0, GammaXX=0, GammaXY=0, GammaYY=0;

  	for (kLevel=kLevels-1; kLevel>=0; kLevel--) {
		for (kIndex=kLevel*kDirections; kIndex<(kLevel+1)*kDirections; kIndex++) {
      		kIndex2 = kIndex*2;
      		confidence = trafoData1[(i1*j1max+j1)*kLevels*kDirections*2+kIndex2]*
				trafoData2[(i2*j2max+j2)*kLevels*kDirections*2+kIndex2];
      		confidenceBak[kIndex] = confidence;
      		if (confidence!=0) {
				k0x = kxArr[kIndex];
				k0y = kyArr[kIndex];
				deltaPhase = (trafoData1[(i1*j1max+j1)*kLevels*kDirections*2+kIndex2+1]-
					trafoData2[(i2*j2max+j2)*kLevels*kDirections*2+kIndex2+1]);
				while (deltaPhase< dispX*k0x+dispY*k0y-M_PI) 
					{ deltaPhase += 2*M_PI; }
				while (deltaPhase>=dispX*k0x+dispY*k0y+M_PI) 
		   			{ deltaPhase -= 2*M_PI; }
				deltaPhaseBak[kIndex] = deltaPhase;
				PhiX    += confidence*k0x*deltaPhase;
				PhiY    += confidence*k0y*deltaPhase;
				GammaXX += confidence*k0x*k0x;
				GammaXY += confidence*k0x*k0y;
				GammaYY += confidence*k0y*k0y;
      		}
    	}
    	dispX = (+PhiX*GammaYY-PhiY*GammaXY)/(GammaXX*GammaYY-GammaXY*GammaXY);
    	dispY = (-PhiX*GammaXY+PhiY*GammaXX)/(GammaXX*GammaYY-GammaXY*GammaXY);
  	}

  	for (kIndex=0; kIndex<kLevels*kDirections; kIndex++) {
    	deltaPhase = deltaPhaseBak[kIndex]-dispX*kxArr[kIndex]-dispY*kyArr[kIndex];
    	meanError += confidenceBak[kIndex]*(2.-deltaPhase*deltaPhase);
  	}

 	return 0.5*meanError;
}

// ==== function to compute the similarity matrices ======================= ;

  // The objects are stored in files as 16x17 graphs the models as 10x10
  // graphs, hence Si1max=17, Sj1max=16 and Si2max=Sj2max=10. With the
  // variables Si1offset, Sj1offset, i1max, and j1max a subgraph can be
  // selected from the stored object graph and analogously for layers 2.

  // The similarity matrices are only computed at the first
  // initialization or if the relevant object or models as noted in the
  // file Dlmobjects and Dlmmodels has changed. 

public void initSys()
{
  // ---- variables ---------------------------------------------------- ;

  int object, objects, model, models;
  int i1,j1,i2,j2,i1R,j1R;
  int k;
  int Si2, Sj2, Si1, Sj1, Si2max, Sj2max, Si1max, Sj1max;
  char DlmObjectPath[256], DlmObjectName[256];
  char DlmModelPath[256], DlmModelName[256]; // DlmCorrectName[256];
  char tmpStr[256];
  char dummyString[256];
  int dummyInt;
  float floatDummy;
  float Norm, normCharFactor;
  char CoeffChar;
  unsigned char CoeffUChar;

  if (gallerySize.elem()<=0) 
 	return;

  for (model=0; model<=gallerySize.elem(); model++) {
    	strcpy(modelName[model],"nobody");
  }

  // ---- initialize tables if required -------------------------------- ;

  if (!tablesCreated)
    createTables();

  // ---- read in object graph ----------------------------------------- ;

  // ---- open file of object names and read basic parameters ;

//  DlmObjects.init_file(objectsFileName,""); // join with next call
  if (DlmObjects.new_file(objectsFileName,NSL_INPUT) == -1) {
  	NSLoutput("Bad file name: ", objectsFileName);
  	exit(1);

  } 
  DlmObjects.read(DlmObjectPath);
  DlmObjects.read(objects);

  // ---- check whether the preferred object index is valid ;

  if ((preferredObject.elem()<=0)||(preferredObject.elem()>objects)) {
    NSLoutput("\nWARNING : preferredObject (",preferredObject.elem()); 
    NSLoutput(") out of range (1-",objects);
	NSLoutput("), set preferredObject to 1 !!\n");
    preferredObject.elem()=1;
  }

  // ---- select object ;

  switch (int(ObjectSelectionMode.elem())) {
  case 1: // PREFERRED :
    selectedObject = int(preferredObject.elem());
    break;
  case 2: // NEXT :
    selectedObject++;
    break;
  case 3: // RANDOM :
    selectedObject = rand()%objects+1;
    break;
  default:
    NSLoutput("\nERROR: ObjectSelectionMode ",ObjectSelectionMode.elem());
	NSLoutput(" not valid !\n");
    break;
  }

  if ((selectedObject<=0)||(selectedObject>objects)) {
    NSLoutput("\nERROR: selected object (",selectedObject); 
    NSLoutput(") out of range [1,...,",objects);
	NSLoutput("] !\n");
    exit(1);
  }

  // ---- read object name 
  
  for (object=0; object<selectedObject; object++)
  {
    DlmObjects.read(DlmObjectName);
    DlmObjects.read(Oi1offset);
    DlmObjects.read(Oj1offset);
  }
        
  DlmObjects.close_file();   
  
  NSLoutput("\n Object :\n ",DlmObjectName);

  // ---- check whether the object name has changed since the last init ;

  if (strcmp(DlmObjectName,objectName)!=0) {
    for (model=0; model<=gallerySize.elem(); model++)
      	strcpy(modelName[model],"nobody");
    strcpy(objectName,DlmObjectName);
  }

  // ---- open object graph file and read basic parameters ;

  sprintf(tmpStr,"%s%s",DlmObjectPath,objectName);

//  DlmGraph.init_file(tmpStr,""); // join with next call
  if (DlmGraph.new_file(tmpStr,NSL_INPUT,NSL_BINARY) == -1) {
  	cmd_out("Bad file name: ", tmpStr);
  	exit(1);
  }  
  
  DlmGraph.read(floatDummy);
  DlmGraph.read(floatDummy);
  DlmGraph.read(Sj1max);
  DlmGraph.read(floatDummy);
  DlmGraph.read(floatDummy);
  DlmGraph.read(Si1max);
 
  // ---- determine offsets ;

  Si1offset = Oi1offset - (i1max-2*frame-10)/2;
  if (Si1offset<0) Si1offset = 0;
  if (Si1offset>Si1max-i1max+2*frame) Si1offset=Si1max-i1max+2*frame;
  if (Si1offset<0) {
    NSLoutput("\nERROR : object graph has less columns (",Si1max);
    NSLoutput(") than required (",i1max-2*frame);
	NSLoutput(") !!\n");
    exit(1);
  }

  Sj1offset = Oj1offset - (j1max-2*frame-10)/2;
  if (Sj1offset<0) Sj1offset = 0;
  if (Sj1offset>Sj1max-j1max+2*frame) Sj1offset=Sj1max-j1max+2*frame;
  if (Sj1offset<0) {
    NSLoutput("\nERROR : object graph has less rows (",Sj1max);
    NSLoutput(") than required (",j1max-2*frame);
	NSLoutput(") !!\n");
    exit(1);
  }

  NSLoutput("   \t(i = ",Si1offset);
  NSLoutput(",...,",Si1offset+i1max-2*frame-1);
  NSLoutput("; j = ",Sj1offset);
  NSLoutput(",...,",Sj1offset+j1max-2*frame-1);
  NSLoutput(")\n");

  // ---- read in trafo parameter of object graph and check them ;

  DlmGraph.read(kLevelsTmp);
  DlmGraph.read(kDirectionsTmp);
  DlmGraph.read(kMaximumTmp);
  DlmGraph.read(kFactorTmp);
  DlmGraph.read(sigmaTmp);

  if (kLevels!=kLevelsTmp) {
    NSLoutput("\nERROR: kLevels (",kLevelsTmp);
    NSLoutput(") in object graph != ",kLevels);
	NSLoutput(" !!\n");
  }
  if (kDirections!=kDirectionsTmp) {
    NSLoutput("\nERROR: kDirections (",kDirectionsTmp);
    NSLoutput(") in object graph != ",kDirections);
	NSLoutput(" !!\n");
  }
  if (kMaximum!=kMaximumTmp) {
    NSLoutput("\nERROR: kMaximum (",kMaximumTmp);
    NSLoutput(") in object graph != ",kMaximum);
	NSLoutput(" !!\n");
  }
  if (kFactor!=kFactorTmp) {
    NSLoutput("\nERROR: kFactor (",kFactorTmp); 
    NSLoutput(") in object graph != ",kFactor);
	NSLoutput(" !!\n");
  }
  if (sigma!=sigmaTmp){
    NSLoutput("\nERROR: sigma (",sigmaTmp);
    NSLoutput(") in object graph != ",sigma);
	NSLoutput(" !!\n");
  }

  // ---- read in jets of object graph ;

  float ImageNormFloat;

  for (Sj1=0, j1=-Sj1offset+frame; Sj1<Sj1max; Sj1++, j1++)
    for (Si1=Si1max-1, i1=Si1max-1-Si1offset+frame; Si1>=0; Si1--, i1--) {

	// check whether the node belongs to the selected subgraph ;

	char tmpChar=' ';

	if ((i1<frame)||(i1>=i1max-frame)||(j1<frame)||(j1>=j1max-frame)) {

	  DlmGraph.read(Norm);  
	  DlmGraph.read(Norm);  

	  tmpChar=' ';

	  while (tmpChar!='\n')
		DlmGraph.read(tmpChar);

	  for (k=0; k<kLevels*kDirections; k++) {
	    DlmGraph.read(CoeffUChar);  
	    DlmGraph.read(CoeffChar); 
	  }

	} else {

	  DlmGraph.read(ImageNormFloat); 
	  DlmGraph.read(ImageNormChar[i1][j1]); 

	  normCharFactor = 1/ImageNormChar[i1][j1];

	  tmpChar=' ';

	  while (tmpChar!='\n')
	    DlmGraph.read(tmpChar); 

	  for (k=0; k<kLevels*kDirections*2; k+=2) {

	    DlmGraph.read(CoeffUChar);  
	    DlmGraph.read(CoeffChar); 

		DlmGImage[(i1*j1max+j1)*kLevels*kDirections*2+k] 
	      = CoeffUChar*normCharFactor;
	    DlmGImage[(i1*j1max+j1)*kLevels*kDirections*2+k+1] 
	      = float(CoeffChar)/127*M_PI;
	  }
	}
  } // for (Si1=Si1max-1; Si1>=0; Si1--) 

  DlmGraph.close_file();

  DlmImageModelData[0] = DlmGImage;

  // ---- read in models and compute similarity matrices ----------------- ;

  // ---- variables ;

  int modelInGal;

  // ---- open file of model names and read basic parameters ;

//  DlmModels.init_file(modelsFileName,""); // join with next call
  if (DlmModels.new_file(modelsFileName,NSL_INPUT) == -1) {
  	NSLoutput("Bad file name: ", modelsFileName);
  	exit(1);
  }  
  
  DlmModels.read(DlmModelPath);
  DlmModels.read(models);  

  // ---- check whether enough models are in DlmModels ;

  if (models<gallerySize.elem()) {
    NSLoutput("\nWARNING : not enough models in DlmModels, set gallerySize=models (",models);
	NSLoutput(") !!\n");
    gallerySize.elem() = models;
  }

  // ---- search for correct model in gallery ;

  for (modelInGal=1; modelInGal<=models; modelInGal++) {

    DlmModels.read(DlmModelName);  
    if (strncmp(DlmModelName,objectName,strlen(objectName)-1)==0)
      break;
  }
  if (modelInGal>models) {
    NSLoutput("\nERROR : person ",objectName);
	NSLoutput(" not in DlmModels !!\n");
  }

  // ---- model loop ;

  NSLoutput("\n\n Gallery : ");

  for (model=1; model<=gallerySize.elem(); model++) {
    // ---- read name of next model 
    if (model!=1) {
      modelInGal++;
      if (modelInGal>models) {

		DlmModels.close_file();
//		DlmModels.init_file(modelsFileName,""); // join with next call
  		if (DlmModels.new_file(modelsFileName,NSL_INPUT) == -1) {
  			NSLoutput("Bad file name: ", modelsFileName);
  			exit(1);
  		} 
  	
  		DlmModels.read(dummyString);
		DlmModels.read(dummyInt); 

		modelInGal=1;
      }
      DlmModels.read(DlmModelName);
    }
    NSLoutput("\n ",DlmModelName);

    // ---- check whether the model name has changed since the last init ;

    if (strcmp(DlmModelName,modelName[model])==0)
      continue;

    strcpy(modelName[model],DlmModelName);

    // ---- open model graph file and read basic parameters ;

    sprintf(tmpStr,"%s%s",DlmModelPath,DlmModelName);

 //   DlmGraph.init_file(tmpStr,""); // join with next call
    if (DlmGraph.new_file(tmpStr,NSL_INPUT,NSL_BINARY) == -1) {
  		cmd_out("Bad file name: ", tmpStr);
  		exit(1);
    }  
    
	DlmGraph.read(floatDummy);
	DlmGraph.read(floatDummy);
	DlmGraph.read(Sj2max);
	DlmGraph.read(floatDummy);
	DlmGraph.read(floatDummy);
	DlmGraph.read(Si2max);
  
    // ---- determine offsets ;

    Si2offset = (Si2max-i2max)/2;
    if (Si2offset<0) {
		NSLoutput("\nERROR : model graph has less columns (",Si2max);
		NSLoutput(") than required (",i2max);
		NSLoutput(") !!\n");
	}
    Sj2offset = (Sj2max-j2max)/2;
    if (Sj2offset<0){
		NSLoutput("\nERROR : model graph has less rows (",Sj2max);
		NSLoutput(") than required (",j2max);
		NSLoutput(") !!\n");
	}
    
	NSLoutput("   \t(i = ",Si2offset);
	NSLoutput(",...,",Si2offset+i2max-1);
    NSLoutput("; j = ",Sj2offset);
	NSLoutput(",...,",Sj2offset+j2max-1);
	NSLoutput(")");

    // ---- read trafo parameters of model graph and check them 

	DlmGraph.read(kLevelsTmp);
	DlmGraph.read(kDirectionsTmp);
	DlmGraph.read(kMaximumTmp);
	DlmGraph.read(kFactorTmp);
	DlmGraph.read(sigmaTmp);

	if (kLevels!=kLevelsTmp){
		NSLoutput("\nERROR: kLevels (",kLevelsTmp); 
		NSLoutput(") in model graph != ",kLevels);
		NSLoutput(" !!\n");
	}
	if (kDirections!=kDirectionsTmp){
		NSLoutput("\nERROR: kDirections (",kDirectionsTmp); 
		NSLoutput(") in model graph != ",kDirections);
		NSLoutput(" !!\n");
	}
	if (kMaximum!=kMaximumTmp){
		NSLoutput("\nERROR: kMaximum (",kMaximumTmp);
		NSLoutput(") in model graph != ",kMaximum);
		NSLoutput(" !!\n");
	}
	if (kFactor!=kFactorTmp){
		NSLoutput("\nERROR: kFactor (",kFactorTmp);
		NSLoutput(") in model graph != ",kFactor);
		NSLoutput(" !!\n");
	}
	if (sigma!=sigmaTmp){
		NSLoutput("\nERROR: sigma (",sigmaTmp);
		NSLoutput(") in model graph != ",sigma);
		NSLoutput(" !!\n");
	}

    // ---- read jets of model graph ;

    for (Sj2=0, j2=-Sj2offset; Sj2<Sj2max; Sj2++, j2++)
      for (Si2=Si2max-1, i2=Si2max-1-Si2offset; Si2>=0; Si2--, i2--) {

	// check whether the node belongs to the selected subgraph ;

	char tmpChar;

	if ((i2<0)||(i2>=i2max)||(j2<0)||(j2>=j2max)) {

	  DlmGraph.read(Norm);
	  DlmGraph.read(Norm);

	  tmpChar=' ';

	  while (tmpChar!='\n')
	    DlmGraph.read(tmpChar); 

	  for (k=0; k<kLevels*kDirections; k++) {
	    DlmGraph.read(CoeffUChar);  
	    DlmGraph.read(CoeffChar); 
	  }

	} else {

	  DlmGraph.read(ModelNormFloat[i2][j2]);
	  DlmGraph.read(ModelNormChar[i2][j2]);

	  normCharFactor = 1/ModelNormChar[i2][j2];
	  tmpChar=' ';

	  while (tmpChar!='\n')
	    DlmGraph.read(tmpChar); 

	  for (k=0; k<kLevels*kDirections*2; k+=2) {

	    DlmGraph.read(CoeffUChar);  
	    DlmGraph.read(CoeffChar); 

		DlmGModel[(i2*j2max+j2)*kLevels*kDirections*2+k] 
	      = CoeffUChar*normCharFactor;
	    DlmGModel[(i2*j2max+j2)*kLevels*kDirections*2+k+1] 
	      = float(CoeffChar)/127*M_PI;
	  }
	}

   } // for (Si2=Si2max-1; Si2>=0; Si2--) 

	DlmGraph.close_file();
 
	DlmImageModelData[model] = DlmGModel;

    // ---- compute similarity matrix ;

   for (i2=0; i2<i2max; i2++)
		for (j2=0; j2<j2max; j2++)
			for (i1R=0, i1=i1Index(i2,0,frame,i1max,i1Rmax,i2max); i1R<i1Rmax; i1R++, i1++)
				for (j1R=0, j1=j1Index(j2,0,frame,j1max,j1Rmax,j2max); j1R<j1Rmax; j1R++, j1++) {
					sim[model][i2][j2][i1R][j1R] = maximum(alpha_S.elem(),
						similarity(DlmGImage,i1,j1,DlmGModel,i2,j2));
				}
  } // for (model=1; model<=gallerySize.elem(); model++) 

  DlmModels.close_file();

  // ---- close file of model names 
  	float S;
	if (gallerySize.elem()>0)
	   for (int i2=0; i2<i2max; i2++)
		for (int j2=0; j2<j2max; j2++)
		   for (int i1R=0; i1R<i1Rmax; i1R++)
			for (int j1R=0; j1R<j1Rmax; j1R++) {
			   S = 0;
			   for (int model=1; model<=gallerySize.elem(); model++)
					S = maximum(S,sim[model][i2][j2][i1R][j1R]);
			   sim[0][i2][j2][i1R][j1R] = S;
			}
}
}
