
//  HOUSE MODEL
//
//  FILE NAME   : DepthPerception.mod
//  PURPOSE     : Visual depth analisys
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyright 1996
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            1.0 (Agu 13, 1996 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution.
//
//            1.5 (Sept 22, 1997 - Salvador Marmol), 
//            Adecutation for the new library.
//
//            1.6 (Nov 10, 1997 - Oscar Peguero), 
//            Implementation of the stereo schema.
//
//            1.7 (Mar 16, 1998 - Oscar Peguero), 
//            Changes in loops / steps

// added AW
#include "nsl_include.h"

// repeated class names

// needs fix
	static const  double   PI =3.14159;
	static const  double   M_2_PI =2.0/PI;

// need to generate superclass call
 class HouseRetina  : public NslModule
{
	public: HouseRetina(char*, NslModule *, int , int , int ); // "*"
	public: ~HouseRetina();

	public: NslDinFloat2  in; 
	public: NslDoutFloat2  a; 
	public: NslDoutFloat2  d;
	private: NslFloat1  rl; 
	private: NslFloat1  rr;
	private: NslFloat0  w; 
	private: NslFloat0  yf; 
	private: NslFloat0  l; 
	private: NslFloat0  dmax; 
	private: NslFloat0  sigma;
	
 virtual void initRun()
{
	             in= 0;
	in.elem(0,0)   = 1.0;
	in.elem(10,10) = 1.0;
		
	    rr= 0;
	    rl= 0;
	     a= 0;
	     d= 0;
	     w= 3.0;
	    yf= -10;
	     l= 22.0;
	  dmax= 0.25;
	 sigma= 0.25;
	
	view_to_right_retina(rr,in,w,yf,l);
	view_to_left_retina(rl,in,w,yf,l);
	retina_to_accommodation(a,in,rr,w,yf,l,dmax,sigma);
 	retina_to_disparity(d,rr,rl);
}

private: float  xyview_to_left_retina(float  x,float  y,float  w,float  yf,float  l)
{
	float  bl,ql,a0;

         a0= atan(w/(yf+l));
         bl= atan((float )( x+ w)/( y+ l));
         ql= M_2_PI*( bl- a0);
	return ( ql+ 1.0);
}

private: float  xyview_to_right_retina(float  x,float  y,float  w,float  yf,float  l)
{
	float  br,qr,a0; // NslFloat0

    a0= atan(w/(yf+l)); // nslArcTan
    br= atan(( x- w)/( y+ l)); // nslArcTan
    qr= M_2_PI*( br+ a0);

	return ( qr+ 1.0); // qr.getData()
}
private:  void vector_fill(nsl_vector& q,int  i0,int  i1)
{
         for(int  i = i0;  i<= i1; i++)
                q[i] = 1.0;
}
private:  void view_to_right_retina(NslFloat1  &ret, NslFloat2  &in,
	NslFloat0  &w_d, NslFloat0  &yf_d, NslFloat0  &l_d)
{
	int  iq0,iq1;
	float  val,q0,q1;
	float  xc,yc,xc0,xc1;
	float  w,yf,l;

	 w= w_d.get_data(); // getData
	 yf= yf_d.get_data();
	 l= l_d.get_data();

	int  imax = in.getRows();
	int  jmax = in.getCols();

	int  yz = imax/2;	
	int  xz = jmax/2;	

	float  dx = 1.0; 
	float  dy = 1.0; 

	int  rimax = ret.get_size(); // getSize
	int  midq = ( rimax- rimax%2)/2;

	float  k = 0.5;		 
	float  wx2 = 0.5;	 
			
	 for(int  j = 0;  j< jmax; j++)
	     for(int  i = 0;  i< imax; i++)
	    {
		 val= in[i][j];
		 if( val>= k)
		{	
			 xc= (j-xz)*dx;		
			 yc= (i-yz)*dy;		
			 xc0=  xc- wx2;
			 xc1=  xc+ wx2;
	    		 q0= xyview_to_right_retina(xc0,yc,w,yf,l); 
			 iq0= nsl_rint(midq*q0); // nslRint
			 q1= xyview_to_right_retina(xc1,yc,w,yf,l); 
			 iq1= nsl_rint(midq*q1); // nslRint
			vector_fill(ret,iq0,iq1);
		}
	    }
}
private:  void view_to_left_retina(NslFloat1  &ret, NslFloat2  &in,
	NslFloat0  &w_d, NslFloat0  &yf_d, NslFloat0  &l_d)
{
	int  iq0,iq1;
	float  val,q0,q1;
	float  xc,yc,xc0,xc1;
	float  w,yf,l;

	 w= w_d.get_data(); // getData
	 yf= yf_d.get_data();
	 l= l_d.get_data();

	int  imax = in.getRows();
	int  jmax = in.getCols();

	int  xz = jmax/2;	
	int  yz = imax/2;	

	float  dx = 1.0; 
	float  dy = 1.0; 

	int  rimax = ret.get_size(); // getSize
	int  midq = ( rimax- rimax%2)/2;

	float  k = 0.5;		 
	float  wx2 = 0.5;	 
			
	 for(int  j = 0;  j< jmax; j++)
	     for(int  i = 0;  i< imax; i++)
	    {
		 val= in[i][j];
		 if( val>= k)
		{	
			 xc= (j-xz)*dx;		
			 yc= (i-yz)*dy;		
			 xc0=  xc- wx2;		
			 xc1=  xc+ wx2;		
	    		 q0= xyview_to_left_retina(xc0,yc,w,yf,l); 
			 iq0= nsl_rint(midq*q0); // nslRint
			 q1= xyview_to_left_retina(xc1,yc,w,yf,l); 
			 iq1= nsl_rint(midq*q1); // nslRint
			vector_fill(ret,iq0,iq1);
		}
	    }
}
private:  void retina_to_accommodation(NslFloat2  &a, NslFloat2  &in, 
	NslFloat1  &ret, NslFloat0  &w_d, NslFloat0  &yf_d, NslFloat0  &l_d,
	NslFloat0  &dmax_d, NslFloat0  &sigma_d)
{
	float  da,ql,qr;
   float  d,r,s,val;
	int  id,ip;
	float  xc,yc,xc0,xc1,x;
	float  w,yf,l,dmax,sigma;
	
	 w= w_d.get_data(); // getData
	 yf= yf_d.get_data();
	 l= l_d.get_data();
	 dmax= dmax_d.get_data();
	 sigma= sigma_d.get_data();

	int  imax = a.getRows();
	int  jmax = a.getCols();

	int  yz = imax/2;	
	int  xz = jmax/2;	

	float  dx = 1.0; 
	float  dy = 1.0; 

	int  rimax = ret.get_size(); // getSize
	int  midq = ( rimax- rimax%2)/2;

	int  midy = ( imax- imax%2)/2;
	int  midx = ( jmax- jmax%2)/2;

	float  dqdi = 1.0/(jmax-1);
	float  dddi = 1.0/(imax-1);
    float  dpdi = dqdi*midq/midx;
	float  dipdiq = dqdi/dpdi;

	float  k = 0.5;		
	float  xs = 0.5;		
	float  wx2 = 0.5;	

	 for(int  j = 0;  j< jmax; j++)
	     for(int  i = 0;  i< imax; i++)
	    {
		 val= in[i][j];
		 if( val>= k)
		{	
			 xc= (j-xz)*dx;
			 yc= (i-yz)*dy;
			 xc0=  xc- wx2;		
			 xc1=  xc+ wx2;		
			 for( x= xc0;  x<= xc1;  x+= xs)
			{
	    		 ql= xyview_to_left_retina(x,yc,w,yf,l); 
				 qr= xyview_to_right_retina(x,yc,w,yf,l); 
				 da= qr-ql;
				float  ips = midq*(qr-1.0)*dipdiq;
				 ip= (int ) ( ips+ 0.5);
       			 for(id= -midy; id<= midy; id++)
        		{
					 d= id*dddi;
               		 r= pow((d-da)/(sigma*dmax),2.0); // nslPow
                		 s= exp(-r/2.0);
               		a[id+midy][ip+midx] = s;
				}
			}
	    	}
	    }
}
private:  void retina_to_disparity(NslFloat2  &d, NslFloat1  &rr, NslFloat1  &rl)
{
    int  iq,id,iql,ip;
	float  val;

	int  imax = d.getRows();
	int  jmax = d.getCols();

	int  midy = ( imax- imax%2)/2;
	int  midx = ( jmax- jmax%2)/2;
 
	int  rimax = rr.get_size(); // getSize
	int  midq = ( rimax- rimax%2)/2;

	float  dqdi = 1.0/(jmax-1);
	float  dddi = 1.0/(imax-1);
   float  dpdi = dqdi*midq/midx;
	float  dipdiq = dqdi/dpdi;

    for( iq= -midq;  iq<= midq; iq++)
	{
	     ip= nsl_rint(iq*dipdiq); // nslRint
	     for( id= -midy;  id<= midy; id++)
	    {
		 iql=  iq- id;
		 if( iql>= - midq&&  iql<= midq)
		{
                     val= rl[iql+midq]*rr[iq+midq];
		     if(d[id+midy][ip+midx] == 0.0)
			d[id+midy][ip+midx] = val;
		}
	    }
	}
}
};

// take out "*", use "*" instead of "&" in declaration

HouseRetina::HouseRetina(char* _nslName, NslModule *_nslParent, int  sizeX, int  sizeY, int  sizeR) : // ","->":" there is an extra "," in the middle
  NslModule(_nslName, _nslParent),
 in("in",this,sizeX,sizeY), a("a",this,sizeX,sizeY), d("d",this,sizeX,sizeY), 
 rl("rl",this,sizeR), rr("rr",this,sizeR), w("w",this), 
 yf("yf",this), l("l",this), dmax("dmax",this), sigma("sigma",this) 
{

}



HouseRetina::~HouseRetina()
{

}



 class HouseDepth   : public NslModule
{
	public: HouseDepth(char*, NslModule *, int , int ); // NslObject &
	public: ~HouseDepth();

 
	public: NslDinFloat2  s;
	public: NslDinFloat2  t; 
	public: NslDoutFloat2  mf;
	private: NslFloat2  mp; 
	private: NslFloat1  up; 
	private: NslFloat1  uf; 
	private: NslFloat0  tu; // missing 0 in NslFloat
	private: NslFloat0  tm;
	private: NslFloat0 hu;
	private: NslFloat0 hm;
	private: NslFloat1  wm; // missing 1 in NslFloat
	private: NslFloat0  kmu; 
	private: NslFloat0  kam; 
	private: NslFloat0  kum; 
	private: NslFloat0  ksm;
	private: NslFloat0  x1; 
	private: NslFloat0  x2;
	
 virtual void initRun()
{
	 mp= 0;
	 mf= 0;
	 s= 0;
	 t= 0;
	 up= 0;
	 uf= 0;
	 kam= 0.5;
	 kum= 0.6;
	 ksm= 0.8;
	 kmu= 1.0;
	wm[0] = 0.25; wm[1] = 0.68; wm[2] = 0.25;
	 x1= 0.1;
	 x2= 1.1;
	 tu= 0.1;
	 tm= 0.3;
}

 virtual void simRun()
{
	nslDiff(mp,tm,-mp+kam*s-
		kum*nslExpandRows(uf,mp.getRows())+ksm*t+wm*mf+hm);
	 mf= NSLsigma(mp,x1,x2); // nslSigma

	nslDiff(up,tu, - up+ kmu*nslReduceRows(mf) + hu);
	 uf= NSLramp(up); // nslRamp
}
};

 HouseDepth::HouseDepth(char* _nslName, NslModule *_nslParent, int  sizeX, int  sizeY) : // ,
  NslModule(_nslName, _nslParent),
 s("s",this,sizeX,sizeY), t("t",this,sizeX,sizeY), 
	 mf("mf",this,sizeX,sizeY), mp("mp",this,sizeX,sizeY), 
	 up("up",this,sizeY), uf("uf",this,sizeY), tu("tu",this), tm("tm",this), 
	 hu("hu",this), hm("hm",this),wm("wm",this,3), kmu("kmu",this), kam("kam",this), 
	 kum("kum",this), ksm("ksm",this), x1("x1",this), x2("x2",this) 
{

}



HouseDepth::~HouseDepth()
{

}



 class HouseStereo  : public NslModule 	
{
	public: HouseStereo(char*, NslModule *, int , int , int );
	public: ~HouseStereo();

	public:
	NslDinFloat2 in;
	NslDoutFloat2 out;

	 HouseDepth s ; 
	 HouseDepth m ; 
	 HouseRetina r  ; 

 void makeConn()
{
	nslConnect(r.a, s.s);
	nslConnect(r.d, m.s);
	nslConnect(s.mf, m.t);
	nslConnect(m.mf, s.t);

	nslRelabel(in,r.in);
	nslRelabel(m.mf,out);
 }
};

 // missing Depth/Retina calls in constructor
 HouseStereo::HouseStereo(char* _nslName, NslModule *_nslParent, int  sizeX, /*,*/ int  sizeY, int  sizeR) :
 NslModule(_nslName, _nslParent),
 s("s",this,sizeX,sizeY), m("m",this,sizeX,sizeY), r("r",this,sizeX,sizeY,sizeR), 
 in("in",this,sizeX,sizeY),out("out",this,sizeX,sizeY)
{
	makeConn(); // get rid
}



HouseStereo::~HouseStereo()
{

}

// need last instantiations

class HouseVisin : public NslModule
{
public:
	NslDoutFloat2 out;
	
	HouseVisin(char *, NslModule*,int,int);
	~HouseVisin() {}
};

HouseVisin::HouseVisin(char *str, NslModule* parent, 
	int sizeX, int sizeY) : NslModule(str, parent),
	out("out",this,sizeX,sizeY)
{
}

class DepthModel : public NslModel
{
public:
	HouseVisin visin;
	HouseStereo stereo;
	
	DepthModel();
	~DepthModel() {}
		
	void makeConn()
	{
		nslConnect(visin.out, stereo.in);
	}
};

DepthModel::DepthModel() : NslModel("DepthModel"),
	visin("visin",this,11,11), // 11,11), //81,81)
	stereo("stereo",this,11,11,21) // 11,11,21) //81,81,161)
{
	makeConn(); // get rid
}

AslSchemaModel<DepthModel> _DepthModel("DepthModel");