verbose_NSLC
//  HOUSE MODEL
//
//  FILE NAME   : DepthPerception.mod
//  PURPOSE     : Visual depth analisys
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyright 1996
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            1.0 (Agu 13, 1996 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution.
//
//            1.5 (Sept 22, 1997 - Salvador Marmol), 
//            Adecutation for the new library.
//
//            1.6 (Nov 10, 1997 - Oscar Peguero), 
//            Implementation of the stereo schema.
//
//            1.7 (Mar 16, 1998 - Oscar Peguero), 
//            Changes in loops / steps
verbose_off

nslModule HouseRetina(int sizeX,int sizeY,int sizeR) 
{
	static const double PI = 3.14159;
	static const double M_2_PI = 2.0/PI;

	public NslDinFloat2 in(sizeX,sizeY); 
	public NslDoutFloat2 a(sizeX,sizeY); 
	public NslDoutFloat2 d(sizeX,sizeY);
	private NslFloat1 rl(sizeR); 
	private NslFloat1 rr(sizeR);
	private NslFloat0 w(); 
	private NslFloat0 yf(); 
	private NslFloat0 l(); 
	private NslFloat0 dmax(); 
	private NslFloat0 sigma();
	
void initRun()
{
	in             = 0;
	in.elem(0,0)   = 1.0;
	in.elem(10,10) = 1.0;
		
	rr    = 0;
	rl    = 0;
	a     = 0;
	d     = 0;
	w     = 3.0;
	yf    = -10;
	l     = 22.0;
	dmax  = 0.25;
	sigma = 0.25;
	
	view_to_right_retina(rr,in,w,yf,l);
	view_to_left_retina(rl,in,w,yf,l);
	retina_to_accommodation(a,in,rr,w,yf,l,dmax,sigma);
 	retina_to_disparity(d,rr,rl);
}

private float xyview_to_left_retina(float x,float y,float w,float yf,float l)
{
	float bl,ql,a0;

        a0 = atan(w/(yf+l));
        bl = atan((float)(x + w)/(y + l));
        ql = M_2_PI*(bl - a0);
	return(ql + 1.0);
}

private float xyview_to_right_retina(float x,float y,float w,float yf,float l)
{
	NslFloat0 br,qr,a0;

   a0 = nslArcTan(w/(yf+l));
   br = nslArcTan((x - w)/(y + l));
   qr = M_2_PI*(br + a0);

	return(qr.getData() + 1.0);
}
private void vector_fill(nsl_vector& q,int i0,int i1)
{
        for (int i = i0; i <= i1; i++)
                q[i] = 1.0;
}
private void view_to_right_retina(NslFloat1 ret, NslFloat2 in,
	NslFloat0 w_d, NslFloat0 yf_d, NslFloat0 l_d)
{
	int iq0,iq1;
	float val,q0,q1;
	float xc,yc,xc0,xc1;
	float w,yf,l;

	w = w_d.getData();
	yf = yf_d.getData();
	l = l_d.getData();

	int imax = in.getRows();
	int jmax = in.getCols();

	int yz = imax/2;	
	int xz = jmax/2;	

	float dx = 1.0; 
	float dy = 1.0; 

	int rimax = ret.getSize();
	int midq = (rimax - rimax%2)/2;

	float k = 0.5;		 
	float wx2 = 0.5;	 
			
	for (int j = 0; j < jmax; j++)
	    for (int i = 0; i < imax; i++)
	    {
		val = in[i][j];
		if (val >= k)
		{	
			xc = (j-xz)*dx;		
			yc = (i-yz)*dy;		
			xc0 = xc - wx2;
			xc1 = xc + wx2;
	    		q0 = xyview_to_right_retina(xc0,yc,w,yf,l); 
			iq0 = nslRint(midq*q0);
			q1 = xyview_to_right_retina(xc1,yc,w,yf,l); 
			iq1 = nslRint(midq*q1);
			vector_fill(ret,iq0,iq1);
		}
	    }
}
private void view_to_left_retina(NslFloat1 ret, NslFloat2 in,
	NslFloat0 w_d, NslFloat0 yf_d, NslFloat0 l_d)
{
	int iq0,iq1;
	float val,q0,q1;
	float xc,yc,xc0,xc1;
	float w,yf,l;

	w = w_d.getData();
	yf = yf_d.getData();
	l = l_d.getData();

	int imax = in.getRows();
	int jmax = in.getCols();

	int xz = jmax/2;	
	int yz = imax/2;	

	float dx = 1.0; 
	float dy = 1.0; 

	int rimax = ret.getSize();
	int midq = (rimax - rimax%2)/2;

	float k = 0.5;		 
	float wx2 = 0.5;	 
			
	for (int j = 0; j < jmax; j++)
	    for (int i = 0; i < imax; i++)
	    {
		val = in[i][j];
		if (val >= k)
		{	
			xc = (j-xz)*dx;		
			yc = (i-yz)*dy;		
			xc0 = xc - wx2;		
			xc1 = xc + wx2;		
	    		q0 = xyview_to_left_retina(xc0,yc,w,yf,l); 
			iq0 = rint(midq*q0);
			q1 = xyview_to_left_retina(xc1,yc,w,yf,l); 
			iq1 = nslRint(midq*q1);
			vector_fill(ret,iq0,iq1);
		}
	    }
}
private void retina_to_accommodation(NslFloat2 a, NslFloat2 in, 
	NslFloat1 ret, NslFloat0 w_d, NslFloat0 yf_d, NslFloat0 l_d,
	NslFloat0 dmax_d, NslFloat0 sigma_d)
{
	float da,ql,qr;
   float d,r,s,val;
	int id,ip;
	float xc,yc,xc0,xc1,x;
	float w,yf,l,dmax,sigma;
	
	w = w_d.getData();
	yf = yf_d.getData();
	l = l_d.getData();
	dmax = dmax_d.getData();
	sigma = sigma_d.getData();

	int imax = a.getRows();
	int jmax = a.getCols();

	int yz = imax/2;	
	int xz = jmax/2;	

	float dx = 1.0; 
	float dy = 1.0; 

	int rimax = ret.getSize();
	int midq = (rimax - rimax%2)/2;

	int midy = (imax - imax%2)/2;
	int midx = (jmax - jmax%2)/2;

	float dqdi = 1.0/(jmax-1);
	float dddi = 1.0/(imax-1);
    float dpdi = dqdi*midq/midx;
	float dipdiq = dqdi/dpdi;

	float k = 0.5;		
	float xs = 0.5;		
	float wx2 = 0.5;	

	for (int j = 0; j < jmax; j++)
	    for (int i = 0; i < imax; i++)
	    {
		val = in[i][j];
		if (val >= k)
		{	
			xc = (j-xz)*dx;
			yc = (i-yz)*dy;
			xc0 = xc - wx2;		
			xc1 = xc + wx2;		
			for (x = xc0; x <= xc1; x += xs)
			{
	    		ql = xyview_to_left_retina(x,yc,w,yf,l); 
				qr = xyview_to_right_retina(x,yc,w,yf,l); 
				da = qr-ql;
				float ips = midq*(qr-1.0)*dipdiq;
				ip = (int) (ips + 0.5);
       			for (id= -midy; id<= midy; id++)
        		{
					d = id*dddi;
               		r = nslPow((d-da)/(sigma*dmax),2.0);
                		s = exp(-r/2.0);
               		a[id+midy][ip+midx] = s;
				}
			}
	    	}
	    }
}
private void retina_to_disparity(NslFloat2 d, NslFloat1 rr, NslFloat1 rl)
{
    int iq,id,iql,ip;
	float val;

	int imax = d.getRows();
	int jmax = d.getCols();

	int midy = (imax - imax%2)/2;
	int midx = (jmax - jmax%2)/2;
 
	int rimax = rr.getSize();
	int midq = (rimax - rimax%2)/2;

	float dqdi = 1.0/(jmax-1);
	float dddi = 1.0/(imax-1);
   float dpdi = dqdi*midq/midx;
	float dipdiq = dqdi/dpdi;

   for (iq = -midq; iq <= midq; iq++)
	{
	    ip = nslRint(iq*dipdiq);
	    for (id = -midy; id <= midy; id++)
	    {
		iql = iq - id;
		if (iql >= -midq && iql <= midq)
		{
                    val = rl[iql+midq]*rr[iq+midq];
		    if (d[id+midy][ip+midx] == 0.0)
			d[id+midy][ip+midx] = val;
		}
	    }
	}
}
}

nslModule Dev2(int sizeX,int sizeY) 
{ 
	public NslDinFloat2 s(sizeX,sizeY);
	public NslDinFloat2 t(sizeX,sizeY); 
	public NslDoutFloat2 mf(sizeX,sizeY);
	private NslFloat2 mp(sizeX,sizeY); 
	private NslFloat1 up(sizeY); 
	private NslFloat1 uf(sizeY); 
	private NslFloat0 tu();
	private NslFloat0 tm();
	private NslFloat0 hu();
	private NslFloat0 hm();
	private NslFloat0 wm(3); 
	private NslFloat0 kmu(); 
	private NslFloat0 kam(); 
	private NslFloat0 kum(); 
	private NslFloat0 ksm();
	private NslFloat0 x1(); 
	private NslFloat0 x2();
	
void initRun()
{
	mp = 0;
	mf = 0;
	s = 0;
	t = 0;
	up = 0;
	uf = 0;
	kam = 0.5;
	kum = 0.6;
	ksm = 0.8;
	kmu = 1.0;
	wm[0] = 0.25; wm[1] = 0.68; wm[2] = 0.25;
	x1 = 0.1;
	x2 = 1.1;
	tu = 0.1;
	tm = 0.3;
}

void simRun()
{
	nslDiff(mp,tm,-mp+kam*s-
		kum*nslExpandRows(uf,mp.getRows())+ksm*t+wm*mf);
	mf = nslSigma(mp,x1,x2);

	nslDiff(up,tu, -up + kmu*nslReduceRows(mf));
	uf = nslRamp(up);
}
}

nslModule HouseStereo(int sizeX,int sizeY,int sizeR)	
{
	public NslDinFloat2 in(sizeX, sizeY);
	public NslDoutFloat2 out(sizeX, sizeY);

	public Dev2 s(sizeX, sizeY); 
	public Dev2 m(sizeX, sizeY); 
	public HouseRetina r(sizeX, sizeY, sizeR); 

void makeConn()
{
	nslConnect(r.a, s.s);
	nslConnect(r.d, m.s);
	nslConnect(s.mf, m.t);
	nslConnect(m.mf, s.t);

	nslRelabel(in,r.in);
	nslRelabel(m.mf,out);
}
}

nslModule HouseVisin(int sizeX, int sizeY)
{
	public NslDoutFloat2 out(sizeX, sizeY);
}

nslModel DepthModel
{
	public HouseVisin visin(11,11);
	public HouseStereo stereo(11,11,21);
	
	void makeConn()
	{
		nslConnect(visin.out, stereo.in);
	}
}