/*xyz123
//  HOUSE MODEL
//
//  FILE NAME   : DepthPerception.mod
//  PURPOSE     : Visual depth analisys
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyright 1996
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            1.0 (Agu 13, 1996 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution.
//
//            1.5 (Sept 22, 1997 - Salvador Marmol), 
//            Adecutation for the new library.
//
//            1.6 (Nov 10, 1997 - Oscar Peguero), 
//            Implementation of the stereo schema.
//
//            1.7 (Mar 16, 1998 - Oscar Peguero), 
//            Changes in loops / steps
*/

#include "DepthPerception.h"

// Retina schema methods

 class Retina    : public NslModule
{
	public: Retina(charString, NslObject &, int , int , int );
	public: ~Retina();


	public: NslDinFloat2  in; 
	public: NslDinFloat2  a; 
	public: NslDinFloat2  d;
	private: NslFloat1  rl; 
	private: NslFloat1  rr;
	private: NslFloat0  w; 
	private: NslFloat0  yf; 
	private: NslFloat0  l; 
	private: NslFloat0  dmax; 
	private: NslFloat0  sigma;
	
 virtual void initRun()
{
	             in= 0;
	in.elem(0,0)   = 1.0;
	in.elem(10,10) = 1.0;
		
/*	rr    = 0;
	rl    = 0;
	a     = 0;
	d     = 0;
	w     = 3.0;
	yf    = -10;
	l     = 22.0;
	dmax  = 0.25;
	sigma = 0.25;
	
	view_to_right_retina(rr,in,w,yf,l);
	view_to_left_retina(rl,in,w,yf,l);
	retina_to_accommodation(a,in,rr,w,yf,l,dmax,sigma);
//	retina_to_disparity(d,rr,rl);
 */
}
};

Retina::Retina(charString _nslName, NslObject &_nslParent, int  sizeX, , int  sizeY, int  sizeR) : NslModule(_nslName, _nslParent), in("in",*this,sizeX,sizeY), a("a",*this,sizeX,sizeY), d("d",*this,sizeX,sizeY), rl("rl",*this,sizeR), rr("rr",*this,sizeR), w("w",*this), yf("yf",*this), l("l",*this), dmax("dmax",*this), sigma("sigma",*this) 
{

}



Retina::~Retina()
{

}



// Depth schema methods

 class Depth   : public NslModule
{
	public: Depth(charString, NslObject &, int , int );
	public: ~Depth();

 
	public: NslDinFloat2  s;
	public: NslDinFloat2  t; 
	public: NslDinFloat2  mf;
	private: NslDinFloat2  mp; 
	private: NslDinFloat1  up; 
	private: NslDinFloat1  uf; 
	private: NslDinFloat  tu;
	private: NslDinFloat  tm;
	private: NslDinFloat  wm; 
	private: NslDinFloat  kmu; 
	private: NslDinFloat  kam; 
	private: NslDinFloat  kum; 
	private: NslDinFloat  ksm;
	private: NslDinFloat  x1; 
	private: NslDinFloat  x2;
	
 virtual void initRun()
{
	 mp= 0;
	 mf= 0;
	 s= 0;
	 t= 0;
	 up= 0;
	 uf= 0;
	 kam= 0.5;
	 kum= 0.6;
	 ksm= 0.8;
	 kmu= 1.0;
	wm[0] = 0.25; wm[1] = 0.68; wm[2] = 0.25;
	 x1= 0.1;
	 x2= 1.1;
	 tu= 0.1;
	 tm= 0.3;
}

 virtual void simRun()
{
	nslDiff(mp,tm,-mp+kam*s-
		kum*NSLrow_vec_to_mat(u,mp.get_rows())+ksm*t+wm*mf);
	 mf= NSLsigma(mp,x1,x2);

	nslDiff(up,tu, - up+ kmu*NSLmat_to_row_vec(mf));
	 uf= NSLramp(up);
}
};

Depth::Depth(charString _nslName, NslObject &_nslParent, int  sizeX, int  sizeY) : NslModule(_nslName, _nslParent), s("s",*this,sizeX,sizeY), t("t",*this,sizeX,sizeY), mf("mf",*this,sizeX,sizeY), mp("mp",*this,sizeX,sizeY), up("up",*this,sizeY), uf("uf",*this,sizeY), tu("tu",*this), tm("tm",*this), wm("wm",*this,3), kmu("kmu",*this), kam("kam",*this), kum("kum",*this), ksm("ksm",*this), x1("x1",*this), x2("x2",*this) 
{

}



Depth::~Depth()
{

}



// Stereo schema methods

 class Stereo    : public NslModule	
{
	public: Stereo(charString, NslObject &, int , int , int );
	public: ~Stereo();


	 Depth s ; 
	 Depth m ; 
	 Retina r  ; 

 void makeConn()
{
	nslConnect(r.a, s.s);
	nslConnect(r.d, m.s);
	nslConnect(s.mf, m.t);
	nslConnect(m.mf, s.t);
}
};

Stereo::Stereo(charString _nslName, NslObject &_nslParent, int  sizeX, , int  sizeY, int  sizeR) : NslModule(_nslName, _nslParent), s("s",*this,sizeX,sizeY), m("m",*this,sizeX,sizeY), r("r",*this,sizeX,sizeY,sizeR) 
{

}



Stereo::~Stereo()
{

}


