/*xyz123
//  HOUSE MODEL
//
//  FILE NAME   : DepthPerception.mod
//  PURPOSE     : Visual depth analisys
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyright 1996
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            1.0 (Agu 13, 1996 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution.
//
//            1.5 (Sept 22, 1997 - Salvador Marmol), 
//            Adecutation for the new library.
//
//            1.6 (Nov 10, 1997 - Oscar Peguero), 
//            Implementation of the stereo schema.
//
//            1.7 (Mar 16, 1998 - Oscar Peguero), 
//            Changes in loops / steps
*/

import DepthPerception;

// Retina schema methods

nslModule Retina(int sizeX,int sizeY,int sizeR) extends NslModule()
{
	public NslDinFloat2 in(sizeX,sizeY); 
	public NslDinFloat2 a(sizeX,sizeY); 
	public NslDinFloat2 d(sizeX,sizeY);
	private NslFloat1 rl(sizeR); 
	private NslFloat1 rr(sizeR);
	private NslFloat0 w(); 
	private NslFloat0 yf(); 
	private NslFloat0 l(); 
	private NslFloat0 dmax(); 
	private NslFloat0 sigma();
	
void initRun()
{
	in             = 0;
	in.elem(0,0)   = 1.0;
	in.elem(10,10) = 1.0;
		
	rr    = 0;
	rl    = 0;
	a     = 0;
	d     = 0;
	w     = 3.0;
	yf    = -10;
	l     = 22.0;
	dmax  = 0.25;
	sigma = 0.25;
	
	view_to_right_retina(rr,in,w,yf,l);
	view_to_left_retina(rl,in,w,yf,l);
	retina_to_accommodation(a,in,rr,w,yf,l,dmax,sigma);
	retina_to_disparity(d,rr,rl);
}
}

// Depth schema methods

nslModule Depth(int sizeX,int sizeY) extends NslModule()
{ 
	public NslDinFloat2 s(sizeX,sizeY);
	public NslDinFloat2 t(sizeX,sizeY); 
	public NslDinFloat2 mf(sizeX,sizeY);
	private NslDinFloat2 mp(sizeX,sizeY); 
	private NslDinFloat1 up(sizeY); 
	private NslDinFloat1 uf(sizeY); 
	private NslDinFloat0 tu();
	private NslDinFloat0 tm();
	private NslDinFloat0 wm(3); 
	private NslDinFloat0 kmu(); 
	private NslDinFloat0 kam(); 
	private NslDinFloat0 kum(); 
	private NslDinFloat0 ksm();
	private NslDinFloat0 x1(); 
	private NslDinFloat0 x2();
	
void initRun()
{
	mp = 0;
	mf = 0;
	s = 0;
	t = 0;
	up = 0;
	uf = 0;
	kam = 0.5;
	kum = 0.6;
	ksm = 0.8;
	kmu = 1.0;
	wm[0] = 0.25; wm[1] = 0.68; wm[2] = 0.25;
	x1 = 0.1;
	x2 = 1.1;
	tu = 0.1;
	tm = 0.3;
}

void simRun()
{
	nslDiff(mp,tm,-mp+kam*s-
		kum*NSLrow_vec_to_mat(u,mp.get_rows())+ksm*t+wm*mf);
	mf = NSLsigma(mp,x1,x2);

	nslDiff(up,tu, -up + kmu*NSLmat_to_row_vec(mf));
	uf = NSLramp(up);
}
}

// Stereo schema methods

nslModule Stereo(int sizeX,int sizeY,int sizeR) extends NslModule()	
{
	Depth s(sizeX, sizeY); 
	Depth m(sizeX, sizeY); 
	Retina r(sizeX, sizeY, sizeR); 

void makeConn()
{
	nslConnect(r.a, s.s);
	nslConnect(r.d, m.s);
	nslConnect(s.mf, m.t);
	nslConnect(m.mf, s.t);
}
}
