/*****************************************************************/
/** 	Recfield.mod						**/
/*****************************************************************/

// Modifications :
//	
//	1) nslModule Recfield definition at EOF
//	2) LayerB nslModule defined twice
//	3) ConnectQ nslModule not defined
//	4) nslGaussian (..) -> RecLib.nslGaussian (..)


nslClass RecLib () extends NslObject () 
{
 void nslGaussian(NslFloat4 g, NslFloat0 h, NslFloat0 s)
 {
	int             i, j, k, l;	// loops 
	float           dist,dx,dy;

	int x1 = g.getX1();
	int y1 = g.getY1();

	// gaussian values (toroidal layer 1) 

	for (i = 0; i < x1; i++)
	  for (j = 0; j < y1; j++)
	    for (k = 0; k < x1; k++)
	      for (l = 0; l < y1; l++) 
	      {
			dx = fabs(i - k);
			if (dx > (x1 / 2))
		  		dx = x1 - dx; 
			dy = fabs(j - l);
			if (dy > (y1 / 2))
		  		dy = y1 - dy;
			dist = sqrt (dx*dx + dy*dy);
			g[i][j][k][l] = (h.elem() * 
		  		1 / s.elem()) * exp(-pow((dist/s.elem()),2)/2) ; 
	      }
 }
}


nslModule LayerA (int x1, int y1) extends NslModule () 
{
	public NslDoutFloat4 d(x1, y1, x1, y1);  // (da), to w

	private NslFloat0 s(); // gaussian spread (sa)
	private NslFloat0 h(); // gaussian height (ha)

 public void initRun() 
 {
	RecLib.nslGaussian(d,h,s);
 }
}


nslModule ConnectW (int x1, int y1, int x2, int y2) extends NslModule () 
{
	public NslDinFloat4 d(x1, y1, x1, y1);  // from a
	public NslDinFloat4 p(x2, y2, x2, y2);  // from q
	public NslDoutFloat4 e(x1, x2, y1, y2); // to q

	private NslFloat4 w(x1, x2, y1, y2);  

	private NslFloat0 maxinitval();  // max init weight val
	private NslFloat0 seed(); 	// random seed

	private NslFloat0 alpha();  // get weights out from zero
	private NslFloat0 beta();  	// integration parameter for act
	private NslFloat0 gama();  	// cubic decay term for w

	int _x1 = x1;
	int _y1 = y1;
	int _x2 = x2;
	int _y2 = y2;

 private void initWeights(float randa)
 {
	int             i, j, k, l;		// loops 

	// weight initialization 

	for (i = 0; i < _x1; i++)
	  for (j = 0; j < _x2; j++)
	    for (k = 0; k < _y1; k++)
	      for (l = 0; l < _y2; l++)
			w[j][i][l][k] = maxinitval.elem() * randa;
 }
 private int nslNormRand(NslFloat0 seed)
 {
	// calculation of the random maximum value 
	int j, max_rand = 0;
	for (int i = 0; i < 1000; i++) { // max number of iterations ?
	  j = rand();
	  if (j > max_rand)
	    max_rand = j;
	}
	// random seed 
	srand(seed.elem());

	return rand()/max_rand; // normalization to 1
 }
 public void initRun() 
 {
	int randa = nslNormRand(seed);
	initWeights(randa);
 }
 private void convGauss()
 {
	int             i, j, k, l, m, n, o, p;		// loops 
	float           sum, sum2;

	// convolutions gaussian 1 w pp (from L2) 
	for (i = 0; i < _x1; i++)
	  for (j = 0; j < _y1; j++)
	    for (k = 0; k < _x2; k++)
	      for (l = 0; l < _y2; l++) {
		  sum = 0;
		  for (m = 0; m < _x1; m++) 
		  	for (n = 0; n < _y1; n++) {
			   sum2 = 0;
			   for (o = 0; o < _x2; o++) 
		      	for (p = 0; p < _y2; p++) 
				  sum2 = sum2 + w[o][m][p][n] * pp[o][p][k][l];
			   sum = sum + d[i][j][m][n] * sum2;
		      }
		  e[k][i][l][j] = sum;
	      }
 }
 private void modifyWeights()
 {
	int             i, j, k, l, m, n, o, p;		// loops 
	float           sum, sum2;

	// weight modification 

	for (i = 0; i < _x1; i++)
	  for (j = 0; j < _y1; j++)
	    for (k = 0; k < _x2; k++)
	      for (l = 0; l < _y2; l++) {
		   sum = 0;
		   for (m = 0; m < _x1; m++)
		  	for (n = 0; n < _y1; n++)
			   sum = sum + d[m][n][i][j] * e[k][m][l][n];
		   w[k][i][l][j] = alpha + w[k][i][l][j] *
		   (1 + beta * (sum - gama * w[k][i][l][j] * w[k][i][l][j]));
		   if (w[k][i][l][j] < 0)
		  	nslPrint("no");
	      }
 }
 public void simRun()
 {
	// convGauss() 	
	e = d * (w * p);	// eq (9)
	// modifyWeights() 
	nslDiff(w,1.0,alpha + beta*w(d*e - gama*(w^w)); // eq (8)
 }
} 
 
 
nslModule LayerB (int x2, int y2) extends NslModule () 
{

	public NslDoutFloat4 d(x2, y2, x2, y2); // (db) to Q

	private NslFloat0 s();  	// gaussian spread (sb)
	private NslFloat0 h(); 	// gaussian height (hb)

 public void initRun() 
 {
	RecLib.nslGaussian(d,delta,sp);
 }
} 
 
 
nslModule ConnectQ (int x1, int y1, int x2, int y2) extends NslModule () 
{
	public NslDinFloat4 e(x1, x2, y1, y2);  // from w
	public NslDinFloat4 d(x2, y2, x2, y2);  // from b (db)
	public NslDoutFloat4 p(x2, y2, x2, y2); // to w

	private NslFloat4 q(x1, y1, x2, y2);  // x2*y2,x2*y2 inhib (q)
	private NslFloat0 alpha();  // get weights out from zero
	private NslFloat0 beta();  	// integration parameter for act
	private NslFloat0 gama();  	// cubic decay term for w

	int _x1 = x1;
	int _y1 = y1;
	int _x2 = x2;
	int _y2 = y2;

 public void initRun() 
 {
	q = 0;
 }
 private void convGauss()
 {
	int             i, j, k, l, m, n, o, p;		// loops 
	float           sum, sum2;

	// convolutions q gaussian 2 

	for (i = 0; i < _x2; i++)
	  for (j = 0; j < _y2; j++)
	    for (k = 0; k < _x2; k++) 
	      for (l = 0; l < _y2; l++) {
		   sum = 0;
		   for (m = 0; m < _x2; m++)
		  	for (n = 0; n < _y2; n++)
			   sum = sum + d[i][j][m][n] * q[m][n][k][l];
		   p[i][j][k][l] = d[i][j][k][l] - sum;
	      }
 }
 private void modifyWeights()
 {
	int             i, j, k, l, m, n, o, p;		// loops 
	float           sum, sum2;

	// weight modification 

	for (i = 0; i < _x2; i++)
	  for (j = 0; j < _y2; j++)
	    for (k = 0; k < _x2; k++) 
	      for (l = 0; l < _y2; l++) {
		  sum = 0;
		  for (m = 0; m < _x1; m++) 
		  	for (n = 0; n < _y1; n++) 
			   sum = sum + e[i][m][j][n] * e[k][m][l][n];
		  w[i][j][k][l] = alpha + w[i][j][k][l] *
		    (1 + beta * (sum - gama*q[i][j][k][l] * q[i][j][k][l]));
		  if (q[i][j][k][l] < 0)  
		  	nslPrint("no");
	      }
 }
 public void simRun()
 {
	// convGauss() 	... eq (9)
	p = d - (d * q);				
	// modifyWeights() ... eq (8) 
	nslDiff(q,1.0,alpha + beta*q(e*e - ga*(q^q)); 	
 }
}


nslModule Recfield (int x1, int y1, int x2, int y2) extends NslModule () 
{
	public LayerA a(x1, y1);  	
	public LayerB b(x2, y2);  	

	public ConnectW w(x1, y1, x2, y2);  	
	public ConnectQ q(x1, y1, x2, y2);  	

 public void makeConn () 
 {
	nslConnect(a.d,w.d);  	
	nslConnect(b.d,q.d);  	
	nslConnect(w.e,q.e);  	
	nslConnect(q.p,w.p);  	
 }
}


nslModel RecfieldModel extends NslModel()
{
	public Recfield rf(5,5,5,5);  	
}

