//  ASL COMPILER
//
//  ASL (Abstract Schema Language) to c++ compiler
//
//
//
//  FILE NAME   : aslc.c
//  MODULE TYPE : Main module
//  STATUS      : In development
//  PURPOSE     : Main module of the ASL compiler
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyrigth 1995-2000
//
//
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            0.00  (Apr 20, 1992 - Longguang Wu), 
//            Start of development.
//
//            0.50  (May 05, 1993 - Roger Zimmermann),
//            Version 0.5 alpha.     
//
//            1.00  (Dec 23, 1995 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution. 
//
//			  1.10  (Feb 16, 1998 - Sebastian Gutierrez),
//			  Sintax extended according to the NSLC 3.0 specifications.
// 


#include "aslc.h"
// Seguti 2000
#ifdef NSL_PC
	//#include <clex.h>
	//#include <cyacc.h>
	//#include "asldef.h"
	//#include "asldef2.h"
	//#include "aslcyacc.h"
	//#include "aslclex.h"

	// mylexer
	class YYFAR mylexer : public yyflexer 
	{
		public:
			mylexer();

		protected:
			void yytables();
			virtual int yyaction(int action);
		public:
	};


	// myparser
	class YYFAR myparser : public yyfparser 
	{
		public:
			myparser();

		protected:
			void yytables();
			virtual void yyaction(int action);
			#ifdef YYDEBUG
				void YYFAR* yyattribute1(int index) const;
				void yyinitdebug(void YYFAR** p, int count) const;
			#endif

		public:
	};


	myparser mp;
	mylexer ml;
#endif
//Seguti 2000

// Global objects
AslCompiler compiler;
int flagObject    = FALSE;
int flagExtends   = FALSE;
int flagAssign    = FALSE;
int flagSchema	  = FALSE;
int flagMethod 	= FALSE;
char myArgument[STR_LEN_LINE];
char myBuffer[STR_LEN_LINE];
char myStack[STR_LEN_LINE];
char myAssign[STR_LEN_LINE];



/*	Commented out Seguti 2000
// Global Symbol Table.
char symbolTable[MAX_SYMBOL];
char *symTablePos = symbolTable;
int  symPosition = 0;
*/


// External variables and functions
#ifndef NSL_PC
	extern FILE *yyin, *yyout;		// Yacc input and output files.
	extern int yyparse(void);		// Yacc parser.
#endif


// AslSymbol members.
AslSymbol::AslSymbol(void) 
{ 
	// name = symTablePos;
}

void  AslSymbol::setName(char*s) 
{
	name = new char[strlen(s)+1];

	strcpy(name,s);
}

char* AslSymbol::getName(void)
{
	return name; 
}

void AslSymbol::setSymbolType(int t)
{
	symbolType = t;
} 

int  AslSymbol::getSymbolType(void)
{
	return symbolType;
}

int AslSymbol::operator < (AslSymbol s)
{ 
	return (strcmp(name, s.name)<0);  
}

int AslSymbol::operator > (AslSymbol s)
{
	return (strcmp(name, s.name)>0);
}

int AslSymbol::operator == (AslSymbol s)
{
	return (strcmp(name, s.name)==0);
}

void AslSymbol::print(void)
{
	cerr << name << '\n';
}


// AslSchemaSymbol members.
AslSchemaSymbol::AslSchemaSymbol(void) 
{ 
	setSymbolType(SCHEMA_SYMBOL); 
}

void AslSchemaSymbol::setPrototype(int p) 
{ 
	prototype = p;
}

int AslSchemaSymbol::isPrototype(void)
{ 
	return prototype; 
}


// AslStructSymbol members.
AslStructSymbol::AslStructSymbol(void) 
{
	setSymbolType(STRUCT_SYMBOL);
}


// AslFunctionSymbol members.
AslFunctionSymbol::AslFunctionSymbol(void)
{
	setSymbolType(FUNCTION_SYMBOL);
}

// AslIdentifierSymbol members.
AslIdentifierSymbol::AslIdentifierSymbol(void)
{
	setSymbolType(IDENT_SYMBOL);
}

void AslIdentifierSymbol::setScope(int s)
{
	scope = s;
}

int AslIdentifierSymbol::getScope(void)
{
	return scope;
}

void AslIdentifierSymbol::setType(char* t)
{
	type = new char[strlen(t)];

	strcpy(type,t);
}

char* AslIdentifierSymbol::getType(void)
{
	return type;
}

void AslIdentifierSymbol::print(void)
{
	cerr << getName() << " of type " << type << '\n';
}


// Function setLexState is implemented at the end of file aslclex.l


// AslParser members.
AslParser::AslParser(void)
{
	numOfLines    	= 0;
	linePosition  	= 0;
	schemaIndex   	= 0;
	modelIndex		= 0;
	argIndex      	= 0;
	objectIndex		= 0;
	methodIndex		= 0;
	scope	      	= 0;
	arglist		= NULL;
	objectlist	= NULL;
	methodlist	= NULL;
	strcpy(actualExtends,"");
	strcpy(actualModel,"");
	strcpy(actualModelExtends,"");
	// AW2000: ref to mylexer
	#ifdef NSL_PC	
		tml = &ml;
	#endif
	// AW2000: ref to mylexer
}

#ifndef NSL_PC
  int(*AslParser::parse)(void) = (int(*)(void))yyparse;
#endif

void AslParser::saveLine(char* l)
{
	int len = strlen(l);

	strcpy(line, l);

	for (int i=0; i<len; i++)
		if(line[i]=='\t') line[i]=' ';
}

char *AslParser::getLine(void)
{
	return line;
}

void AslParser::incNumOfLines(void)
{
	numOfLines++;
}

void AslParser::setNumOfLines(int s)
{
	numOfLines = s;
}

int AslParser::getNumOfLines(void)
{
	return numOfLines;
}

void AslParser::setActualType(char* t)
{
	strcpy(actualType,t);
}

char* AslParser::getActualType(void)
{
	return actualType;
}

void AslParser::setActualIdentifier(char* i)
{
	strcpy(actualIdentifier,i);
}

char* AslParser::getActualIdentifier(void)
{
	return actualIdentifier;
}

void AslParser::setLinePos(int p)
{
	linePosition = p;
}

int AslParser::getLinePos(void)
{
	return linePosition;
}

void AslParser::addLinePos(int p)
{
	linePosition += p;
}

void AslParser::incNumOfSchemas(void)
{
	schemaIndex++;
}

int AslParser::getNumOfSchemas(void)
{
	return schemaIndex;
}

void AslParser::incNumOfModels(void)
{
	modelIndex++;
}

int AslParser::getNumOfModels(void)
{
	return modelIndex;
}

void AslParser::setAccess(int a)
{
	access = a;
}

void AslParser::incScope(void)
{
	 scope ++;
}

void AslParser::decScope(void)
{
	 scope --;
}

int AslParser::getScope(void)
{
	return scope;
}

void AslParser::pushActualSymbol(AslComposedSymbol *a)
{
	actualSymbol[1] = actualSymbol[0];
	actualSymbol[0] = a;
}

AslComposedSymbol *AslParser::popActualSymbol(void)
{
	AslComposedSymbol *temp=actualSymbol[0];

	actualSymbol[0] = actualSymbol[1];

	return temp;
}

void AslParser::insertSchema(char *n, int p)
{
	int res;
	AslSchemaSymbol *s;

	s = new AslSchemaSymbol;
	s->setName(n);
	s->setPrototype(p);

	res = schemaTree.insert(s);

	if (res)
	{
		char buf[STR_LEN_LINE];

		sprintf(buf,"Schema [%s] already defined", s->getName());
		compiler.semanticError(buf);

		delete s;
	}
	else
		pushActualSymbol(s);
}

AslSchemaSymbol* AslParser::searchSchema(char *n)
{
	AslSchemaSymbol *s,*res;	

	s = new AslSchemaSymbol;
	s->setName(n);

	res = schemaTree.search(s);

	delete s;

	return res;
}

void AslParser::printSchemaList(void)
{
	AslComposedSymbol *res;
	AslPtr    *ptr;

	if ((res=schemaTree.getActual()) != NULL)
	{
		ptr = schemaTree.getActualPtr();

		schemaTree.goLeft();
		printSchemaList();

		res->print();

		pushActualSymbol(res);
		actualSymbol[0]->identifierTree.startOnTop();
		//printIdentifierList();

		schemaTree.setActualPtr(ptr);

		schemaTree.goRight();
		printSchemaList();
	}
}

void AslParser::insertIdentifier(char *n)
{
	int res;
	AslIdentifierSymbol *s;

	s = new AslIdentifierSymbol;
	s->setName(n);
	s->setType(actualType);

	setActualIdentifier(n);

	if (!flagExtends)
	  {
		res = actualSymbol[0]->identifierTree.insert(s);
		if (res)
		  {
			char buf[STR_LEN_LINE];
			sprintf(buf,"Identifier [%s] already defined", s->getName());
			compiler.semanticError(buf);
			delete s;
		  }
	  }
}

AslIdentifierSymbol* AslParser::searchIdentifier(char *n)
{
	if (actualSymbol[0]!= NULL)
	{
		AslIdentifierSymbol *s,*res;

		s = new AslIdentifierSymbol;
		s->setName(n);

		res = actualSymbol[0]->identifierTree.search(s);

		delete s;
		
		return res;
	}

	return NULL;
}

void AslParser::printIdentifierList(void)
{
	AslSymbol *res;
	AslPtr    *ptr;

	if ((res=actualSymbol[0]->identifierTree.getActual()) != NULL)
	{
		ptr = actualSymbol[0]->identifierTree.getActualPtr();

		actualSymbol[0]->identifierTree.goLeft();
		printIdentifierList();

		res->print();

		actualSymbol[0]->identifierTree.setActualPtr(ptr);

		actualSymbol[0]->identifierTree.goRight();
		printIdentifierList();
	}
}

void AslParser::insertFunction(char *f)
{
	int res;
	AslSchemaSymbol *temp;
	AslFunctionSymbol *s;

	s = new AslFunctionSymbol;
	s->setName(f);

	temp = (AslSchemaSymbol*)actualSymbol[0];

	res = temp->functionTree.insert(s);

	if (res)
	{
		char buf[STR_LEN_LINE];

		sprintf(buf,"Identifier [%s] already defined", s->getName());
		compiler.semanticError(buf);

		delete s;
	}
	else
		pushActualSymbol(s);
}

AslFunctionSymbol* AslParser::searchFunction(char *n)
{
	AslSchemaSymbol *temp;
	AslFunctionSymbol *s,*res;

	s = new AslFunctionSymbol;
	s->setName(n);

	temp = (AslSchemaSymbol*)actualSymbol[0];

	res = temp->functionTree.search(s);

	delete s;

	return res;
}

void AslParser::processIncludeLines(char *l)
{
	char includeFileName[STR_LEN_LINE];
	int  ans, lineNumber;

	ans = sscanf(l+1,"%d \"%[^\"]\"", &lineNumber ,includeFileName);

	if (ans<2)
	{
		if( strstr(l, "pragma")==NULL )
			compiler.sintaxError("Unknown compiler directive");
	}
	else
	{
		setNumOfLines(lineNumber);
		compiler.setInputFileName(includeFileName);
	}
}

int AslParser::whichTypeIs(char *n)
{
	AslSymbol *res;

	res = searchSchema(n);

	if (res!=NULL)
		return SCHEMA_SYMBOL;

	res = searchIdentifier(n);

	/* Commented out Seguti 2000
	if (res!=NULL)
		return IDENT_SYMBOL;*/
	return -1;
}

void  AslParser::setActualSchema(char* s)
{
	strcpy(actualSchema,s);
}

char* AslParser::getActualSchema()
{
	return actualSchema;
}
void  AslParser::setActualAccessOption(char* s)
{
	strcpy(actualAccessOption,s);
}

char* AslParser::getActualAccessOption()
{
	return actualAccessOption;
}

void  AslParser::makeConstructor(char* s)
{
	compiler.putInOutFile("\n\n");
	compiler.putInOutFile(s);
	compiler.putInOutFile("::");
	compiler.putInOutFile(s);
	compiler.putInOutFile("(");
	getSchemaArguments();				
	compiler.putInOutFile(") ");
	getActualExtends();
	getExtendsArguments();
	if (strcmp(actualExtends,"") != 0) compiler.putInOutFile(")");
	getObjects();						
	compiler.putInOutFile(" \n{\n");
	compiler.putInOutFile(myAssign);
	compiler.putInOutFile("\n}\n\n");
}

void  AslParser::makeDestructor(char* s)
{
	compiler.putInOutFile("\n\n");
	compiler.putInOutFile(s);
	compiler.putInOutFile("::~");
	compiler.putInOutFile(s);
	compiler.putInOutFile("(");
	compiler.putInOutFile(")\n{\n");
	compiler.putInOutFile("\n}\n\n");
}


void  AslParser::makeConstructorDecl(char* s)
{
	compiler.putInOutFile("\n\t");
	compiler.putInOutFile("public: ");
	compiler.putInOutFile(s);
	compiler.putInOutFile("(");
	getSchemaArgumentsDecl();
	compiler.putInOutFile(");\n");
}

void  AslParser::makeDestructorDecl(char* s)
{
	compiler.putInOutFile("\tpublic: ~");
	compiler.putInOutFile(s);
	compiler.putInOutFile("(");
	compiler.putInOutFile(");\n\n");
}


AslArgument* AslParser::insertArg(char* type, char* accesso, char* ident, int flagArgument)
{
	AslArgument *tmp;
	tmp = new AslArgument;
	tmp->setArgument(type,accesso,ident,flagArgument);
	if (arglist==NULL)
	  {
		arglist=tmp;
	  }
	else
	  {
		mylast->next=tmp;
	  }
	mylast=tmp;
	argIndex++;
	return arglist;
}

AslArgument* AslParser::removeArg(void)
{
	AslArgument *tmp;
	tmp=arglist;
	arglist=arglist->next;
	delete tmp;
	argIndex--;
	return arglist;
	
}

void AslParser::getSchemaArguments(void)
{
	AslArgument *root;

	root=arglist;
	compiler.putInOutFile("charString _nslName, NslObject &_nslParent");
	//if ((root != NULL) && (root->getArgFlag() == FALSE)) compiler.putInOutFile(", ");
	while ((root!=NULL) && (root->getArgFlag() == FALSE))
	  {
		compiler.putInOutFile(", ");
		compiler.putInOutFile(root->getType());
		compiler.putInOutFile(root->getAccess());
		compiler.putInOutFile(" ");
		compiler.putInOutFile(root->getIdent());
		root=removeArg();
		// Seguti 2000
		// added by AW: "root != NULL" 
	 	if (root != NULL && (root->next != NULL) && (root->getArgFlag() == FALSE)) 
			compiler.putInOutFile(", ");
		// Seguti 2000
	  }
}

void AslParser::getSchemaArgumentsDecl(void)
{
	AslArgument *root;

	root=arglist;
	compiler.putInOutFile("charString, NslObject &");
	//if ((root != NULL) && (root->getArgFlag() == FALSE)) compiler.putInOutFile(", ");
	while ((root!=NULL) && (root->getArgFlag() == FALSE))
	  {
		compiler.putInOutFile(", ");
		compiler.putInOutFile(root->getType());
		compiler.putInOutFile(root->getAccess());
		/* Commented out Seguti 2000
		compiler.putInOutFile(" ");
		compiler.putInOutFile(root->getIdent());*/
		root = root->next;
		// Commented out Seguti 2000
	 	//if ((root != NULL) && (root->getArgFlag() == FALSE)) compiler.putInOutFile(", ");\
		// Seguti 2000
	  }
}

void AslParser::getExtendsArguments(void)
{
	AslArgument *root;

	root=arglist;
	if ((root != NULL) && (root->getArgFlag() == TRUE)) 
	  {
	    compiler.putInOutFile(", ");
	  }
	while ((root!=NULL) && (root->getArgFlag() == TRUE))
	  {
		compiler.putInOutFile(root->getIdent());
		root=removeArg();
		if ((root != NULL) && (root->getArgFlag() == TRUE)) compiler.putInOutFile(",");
	  }
}
	
AslObject* AslParser::insertObject(char* typeobj,char* accessobj,char* nameobj)
{
	AslObject *tmp;
	tmp = new AslObject;
	tmp->setParam(typeobj,accessobj,nameobj);
	if (objectlist==NULL)
	  {
		objectlist=tmp;
	  }
	else
	  {
		objlast->nextObject=tmp;
	  }
	objlast=tmp;
	objectIndex++;
	return objectlist;

}

void AslParser::insertObjectArguments(char* argument)
{
	char myArgument[STR_LEN_LINE];
	
	strcpy(myArgument,"\"");
	strcat(myArgument,compiler.parser.getActualIdentifier());
	strcat(myArgument,"\",*this");
	if (strcmp(argument,"") != 0) 
	  {
	  	strcat(myArgument,",");
		strcat(myArgument,argument);
	  }
	objlast->setObjectArguments(myArgument);
}

AslObject* AslParser::removeObject(void)
{
	AslObject *tmp;
	tmp=objectlist;
	objectlist=objectlist->nextObject;
	delete tmp;
	objectIndex--;
	return objectlist;
}

// static object instantiations inside constructor header
void AslParser::getObjects(void) 
{
	AslObject *root;
	root=objectlist;
	while (root!=NULL)
	  {
		compiler.putInOutFile(", ");
		compiler.putInOutFile(root->getObjectName());
		compiler.putInOutFile("(");
		compiler.putInOutFile(root->getObjectArguments());
		compiler.putInOutFile(")");
		root=removeObject();
	  }

}

AslObject* AslParser::insertObjectParam(char* typeparam,char* accessparam,char* param)	
{
	compiler.putInOutFile(typeparam);
	compiler.putInOutFile("accessparam");
	compiler.putInOutFile("param");
	compiler.putInOutFile(" ");
	
	return (AslObject*) 0;
}

AslMethod* AslParser::insertMethod(char* typeMethod,char* accessMethod,char* nameMethod)
{
	AslMethod *tmp;
	tmp = new AslMethod;
	tmp->setParam(typeMethod,accessMethod,nameMethod);
	if (methodlist==NULL)
	  {
		methodlist=tmp;
	  }
	else
	  {
		methodlast->nextMethod=tmp;
	  }
	methodlast=tmp;
	methodIndex++;
	return methodlist;

}

void AslParser::insertMethodArguments(char* argument)
{
	methodlast->setMethodArguments(argument);
}

AslMethod* AslParser::removeMethod(void)
{
	AslMethod *tmp;
	tmp=methodlist;
	methodlist=methodlist->nextMethod;
	delete tmp;
	methodIndex--;
	return methodlist;
}

void AslParser::getMethods(void)
{
	AslMethod *root;
	root=methodlist;
	compiler.putInOutFile("\n ");
	while (root!=NULL)
	  {
		compiler.putInOutFile(root->getMethodName());
		compiler.putInOutFile("(");
		compiler.putInOutFile(root->getMethodArguments());
		compiler.putInOutFile(");\n");
		root=removeMethod();
	  }

}

void AslParser::insertMethodParam(char* typeparam,char* accessparam,char* param)	
{
	compiler.putInOutFile(typeparam);
	compiler.putInOutFile(accessparam);
	compiler.putInOutFile(param);
	compiler.putInOutFile(" ");
}

void AslParser::setActualExtends(char* inherit)
{
	strcpy(actualExtends,inherit);
}

void AslParser::setActualExtendsArgs(char* myargs)
{
	strcat(actualExtends,myargs);
}

// inheritance specification
void AslParser::getActualExtends(void)
{
	char mio[STR_LEN_LINE];

	if (strcmp(actualExtends,"") == 0) 
	  {
		compiler.putInOutFile(actualExtends);		
	  }
	else
	  {
		strcpy(mio,": ");
		strcat(mio,actualExtends);
		compiler.putInOutFile(mio);
		compiler.putInOutFile("(_nslName, _nslParent");
	  }
}

void AslParser::getSuperClass(void)
{
 compiler.putInOutFile(actualExtends);		
}

int AslParser::ExistSuper(void)
{
 if (strcmp(actualExtends,"") == 0) return 0;
 else return 1;
}

void  AslParser::setActualModel(char* s)
{
	strcpy(actualModel,s);
}

char* AslParser::getActualModel()
{
	return actualModel;
}

void AslParser::setActualModelExtends(char* inherit)
{
	strcpy(actualModelExtends,inherit);
}

void AslParser::getActualModelExtends(void)
{
	char mio[STR_LEN_LINE];

	if (strcmp(actualModelExtends,"") == 0) 
	  {
		compiler.putInOutFile(": public NslModel()");		
	  }
	else
	  {
		strcpy(mio,": public ");
		strcat(mio,actualModelExtends);
		strcat(mio,"(\"");
		strcat(mio,compiler.parser.getActualModel());
		strcat(mio,"\")");
		compiler.putInOutFile(mio);
		getExtendsModelArguments();
	        compiler.putInOutFile(")");
	  }
}

void AslParser::getExtendsModelArguments(void)
{
	AslArgument *root;

	root=arglist;
	/*if ((root != NULL) && (root->getArgFlag() == TRUE)) 
	  {
	    compiler.putInOutFile("_nslName, _nslParent");
	    compiler.putInOutFile(", ");
	  }*/
	while ((root!=NULL) && (root->getArgFlag() == TRUE))
	  {
		compiler.putInOutFile(root->getIdent());
		root=removeArg();
		if ((root != NULL) && (root->getArgFlag() == TRUE)) compiler.putInOutFile(",");
	  }
}

void  AslParser::makeModelConstructor(char* s)
{
	compiler.putInOutFile("\n\n");
	compiler.putInOutFile(s);
	compiler.putInOutFile("::");
	compiler.putInOutFile(s);
	compiler.putInOutFile("(");
	compiler.putInOutFile(") ");
	getActualModelExtends();
	//getExtendsModelArguments();
	//if (strcmp(actualExtends,"") != 0) compiler.putInOutFile(")");
	getObjects();	
	compiler.putInOutFile(" \n{\n");
	compiler.putInOutFile(myAssign);
	compiler.putInOutFile("\n}\n\n");
	//printf("myAssign = %s\n", myAssign);
}

void  AslParser::makeModelDestructor(char* s)
{
	compiler.putInOutFile("\n\n");
	compiler.putInOutFile(s);
	compiler.putInOutFile("::~");
	compiler.putInOutFile(s);
	compiler.putInOutFile("(");
	compiler.putInOutFile(")\n{\n");
	compiler.putInOutFile("\n}\n\n");
}

void  AslParser::makeModelConstructorDecl(char* s)
{
	compiler.putInOutFile("\n\t");
	compiler.putInOutFile("public: ");
	compiler.putInOutFile(s);
	compiler.putInOutFile("();\n");
}

void  AslParser::makeModelDestructorDecl(char* s)
{
	compiler.putInOutFile("\tpublic: ~");
	compiler.putInOutFile(s);
	compiler.putInOutFile("();\n\n");
}

void  AslParser::makeModelDecl(char* s)
{
	compiler.putInOutFile("NslApplicationModel <");
	compiler.putInOutFile(s);
	compiler.putInOutFile("> _");
	compiler.putInOutFile(s);
	compiler.putInOutFile("(\"");
	compiler.putInOutFile(s);
	compiler.putInOutFile("\");\n\n");
}


// AslArgument methods
AslArgument::AslArgument(void)
{
	strcpy(type,"");
	strcpy(access,"");
	strcpy(ident,"");
	argFlag = 0;
	next 	= NULL;
}

AslArgument::~AslArgument(void)
{
}

void AslArgument::setArgument(char* type1,char* access1,char* ident1, int flagi)
{
	strcpy(type,type1);
	strcpy(access,access1);
	strcpy(ident,ident1);
	argFlag = flagi;
}

char* AslArgument::getType(void)
{
	return type;
}

char* AslArgument::getAccess(void)
{
	return access;
}

char* AslArgument::getIdent(void)
{
	return ident;
}

int AslArgument::getArgFlag(void)
{
	return argFlag;
}


// AslObject methods
AslObject::AslObject(void)
{
	nextObject = NULL;
}

AslObject::~AslObject(void)
{
}

void AslObject::setParam(char* typeobject,char* accessobject,char* object)
{
	strcpy(type,typeobject);
	strcpy(access,accessobject);
	strcpy(name,object);

}

char* AslObject::getObjectType(void)
{
	return type;
}

char* AslObject::getObjectAccess(void)
{
	return access;
}

char* AslObject::getObjectName(void)
{
	return name;
}


void AslObject::setObjectArguments(char* arg)
{
	strcpy(objectArguments,arg);
}

char* AslObject::getObjectArguments(void)
{
	return objectArguments;
}


// AslMethod methods
AslMethod::AslMethod(void)
{
	nextMethod = NULL;
}

AslMethod::~AslMethod(void)
{
}

void AslMethod::setParam(char* typemethod,char* accessmethod,char* method)
{
	strcpy(type,typemethod);
	strcpy(access,accessmethod);
	strcpy(name,method);

}

char* AslMethod::getMethodType(void)
{
	return type;
}

char* AslMethod::getMethodAccess(void)
{
	return access;
}

char* AslMethod::getMethodName(void)
{
	return name;
}


void AslMethod::setMethodArguments(char* arg)
{
	strcpy(methodArguments,arg);
}

char* AslMethod::getMethodArguments(void)
{
	return methodArguments;
}


// AslCompiler methods
AslCompiler::AslCompiler(void)
{
	numOfDefines  = 0;
	numOfIncludes = 0;
	numOfFiles    = 0;
	numOfErrors   = 0;
	verbose       = FALSE;
	logging       = FALSE;
}

void AslCompiler::copyRight(void)
{
	cerr <<"\nNSLM to C++ Compiler, V1.10\n"
      	    <<"(c) Copyright 1998-2000, Instituto Tecnologico Autonomo de Mexico\n\n";
}

void AslCompiler::help(void)
{
	copyRight();
	cout<<"Usage: "
	    <<"aslc [-v]"
	    <<"[-I path][-D definition][-e logName][-o outName] "
	    <<"fileNames\n\n";
}

void AslCompiler::parseArguments(int argc, char *argv[])
{
	char* s;
	int ans=TRUE, out=FALSE;
		
	while (--argc > 0 && (*++argv)[0] == '-') {
		for (s = argv[0]+1; *s != '\0'; s++)
			switch (*s) {
			case 'D': if (numOfDefines < MAX_OF_DEFINES) {
					--argc; ++argv;
				  	strcpy(defines[numOfDefines++], *argv);
				  }
				  else
					operativeError("Too many defines in command line","AslCompiler::parseArguments");
				  break;

			case 'I': if (numOfIncludes < MAX_OF_DEFINES) {
					--argc; ++argv;
					strcpy(includes[numOfIncludes++], *argv);
				  }
				  else
					operativeError("Too many includes in command line","AslCompiler::parseArguments");
				  break;

			case 'e': if (!logging)
				  {
				  	logging = TRUE;
				  	--argc; ++argv;
				  	strcpy(logName, *argv);

					logFile = fopen(logName, "w");
					if (logFile == NULL)
	    			  	{
						sprintf(buf,"Can't open log file %s \n",logName);
						operativeError(buf,"AslCompiler::parseArguments");
				  	}
	    			  	fclose(logFile);

				  	freopen(logName, "w+", stderr);
				  }
				  break;

			case 'o': if (!out)
				  {
					out = TRUE;
					--argc; ++argv;
				 	setOutputFileName(*argv);						
					#ifndef NSL_PC
						fclose(yyout);
				  		yyout = fopen(outputFileName, "w");
				  		if (yyout == NULL)
						{
							sprintf(buf,"Can't open file %s for ouput\n",outputFileName);
							operativeError(buf,"AslCompiler::parseArguments");
						}
					#else
						if (ml.yyout != NULL) fclose(ml.yyout);
						sprintf(buf,"lexoutput");
						ml.yyout = fopen(buf, "w");
						ml.yyout = fopen(outputFileName, "w");
				  		if (ml.yyout == NULL)
				  		{
							sprintf(buf,"Can't open file %s for ouput\n",outputFileName);
							operativeError(buf,"AslCompiler::parseArguments");
				  		}
					#endif
				  }

				  break;

			case 'v': if (!verbose)
				  {
					copyRight();
				  	verbose = TRUE;
				  }

				  break;

			default:  ans = FALSE;
				  break;
      			}
	}

	numOfFiles = argc;

	if (argc==0 || ans == FALSE)
	{
		help();
		exit(1);
	}

	if (numOfFiles == 0)
		operativeError("No input files \n","AslCompiler::parseArguments");

	fileNames = argv;
}

/*
void AslCompiler::parseArguments(int argc, char *argv[])
{
	int option, ans=TRUE, out=FALSE;
	GetOpt getopt(argc, argv, "D:e:I:o:v");

	while ((option = getopt()) != EOF)
	{
		switch (option)
		{
			case 'D': if (numOfDefines < MAX_OF_DEFINES)
				  	strcpy(defines[numOfDefines++], getopt.optarg);
				  else
					operativeError("Too many defines in command line","AslCompiler::parseArguments");
				  break;

			case 'I': if (numOfIncludes < MAX_OF_DEFINES)
					strcpy(includes[numOfIncludes++], getopt.optarg);
				  else
					operativeError("Too many includes in command line","AslCompiler::parseArguments");
				  break;

			case 'e': if (!logging)
				  {
				  	logging = TRUE;
				  	strcpy(logName, getopt.optarg);

					logFile = fopen(logName, "w");
					if (logFile == NULL)
	    			  	{
						sprintf(buf,"Can't open log file %s \n",logName);
						operativeError(buf,"AslCompiler::parseArguments");
				  	}
	    			  	fclose(logFile);

				  	freopen(logName, "w+", stderr);
				  }
				  break;

			case 'o': if (!out)
				  {
					out = TRUE;
				 	setOutputFileName(getopt.optarg);

					fclose(yyout);

				  	yyout = fopen(outputFileName, "w");
				  	if (yyout == NULL)
				  	{
						sprintf(buf,"Can't open file %s for ouput\n",outputFileName);
						operativeError(buf,"AslCompiler::parseArguments");
				  	}
				  }

				  break;

			case 'v': if (!verbose)
				  {
					copyRight();
				  	verbose = TRUE;
				  }

				  break;

			default:  ans = FALSE;
				  break;
      		}
	}

	numOfFiles = argc - getopt.optind;

	if (argc==1 || ans == FALSE)
	{
		help();
		exit(1);
	}

	if (numOfFiles == 0)
		operativeError("No input files \n","AslCompiler::parseArguments");

	fileNames = &argv[getopt.optind];
}
*/
void AslCompiler::preCompile(void)
{
	char temp[STR_LEN_LINE], tempFileName[STR_LEN_LINE];
	int res=1, i;

	sprintf(buf,"gcc -E ");
	for(i=0; i < numOfIncludes; i++)
	{
		sprintf(temp,"-I%s ",includes[i]);
		strcat(buf, temp);
		if (compiler.isVerboseSet())
			cerr << "Extracting include files " << temp <<"\n";
	}

	for(i=0; i < numOfDefines; i++)
	{
		sprintf(temp,"-D%s ",defines[i]);
		strcat(buf, temp);
		if (compiler.isVerboseSet())
			cerr << "Extracting defines " << temp <<"\n";
	}
	// Seguti 2000 
	#ifndef NSL_PC
		sprintf(tempFileName,"%s_%d_temp.c",inputFileName, getpid());
	#else
		sprintf(tempFileName,"%s_temp.cpp",inputFileName);
	#endif
	if (compiler.isVerboseSet())
			cerr << "Creating temporal file " << tempFileName <<"\n";
	sprintf(temp, "-x c++ %s -o %s", inputFileName, tempFileName);
	strcat(buf, temp);
	if (verbose)
		cerr << "Precompiling file " << getInputFileName() << "\n";  //With command: " << buf << "\n";
	#ifndef NSL_PC
		res = system(buf);
	#endif
	if (res==256)
		operativeError("Can't execute precompiler without errors","AslCompiler::preCompile");
	#ifndef NSL_PC
		fclose(ml.yyin);
		yyin  = fopen(tempFileName,"r");
		if (yyin==NULL)
			operativeError("Can't open temporal input file", "AslCompiler::preCompile");
	#else	
		// added by AW
		ml.yycreate();	
		ml.yyin  = fopen(inputFileName,"r");	// ml.yyin  = fopen(<model.mod>,"r");
		//ml.yyout  = fopen(tempFileName,"w"); // ml.yyout  = fopen(<model.mod>_temp.cpp,"w");
		if (ml.yyin==NULL)
			operativeError("Can't open temporal input file", "AslCompiler::preCompile");
	#endif
	// Seguti 2000

	/* Commented out Seguti	2000
	else  {
		ml.yylex();
		fclose(ml.yyout);
	}
	*/
	
}

int AslCompiler::isVerboseSet(void)
{
	return verbose;
}

int AslCompiler::getNumOfFiles(void)
{
	return numOfFiles;
}

char *AslCompiler::getFileName(int i)
{
	return fileNames[i];
}

void AslCompiler::setInputFileName(char* f)
{
	strcpy(inputFileName, f);
}

char *AslCompiler::getInputFileName(void)
{
	return inputFileName;
}

void AslCompiler::setOutputFileName(char* f)
{
	strcpy(outputFileName, f);
}

char *AslCompiler::getOutputFileName(void)
{
	return outputFileName;
}

void AslCompiler::sintaxError(char* s)
{
	int i;

	cerr << "\nERROR (sintax) in file " << compiler.getInputFileName()
	     <<	" near line " << compiler.parser.getNumOfLines() << "\n\n"
	     << "Description: " << s << "\n       Line: " << compiler.parser.getLine() 
	     << "\n      Error: " ;

	for (i=0; i < compiler.parser.getLinePos(); i++)
		cerr << '-' ;

	cerr << "^\n" ;

	numOfErrors++;

	if (numOfErrors>MAX_NUM_OF_ERRORS)
	{	
		cerr << "\nFATAL ERROR\n\n"
		     << "Description: " << "Too many errors to continue compiling \n"
		     << "\nTerminating compilation with errors \n\n";

		exit(1);
	}
}

void AslCompiler::semanticError(char* s)
{
	int i;

	cerr << "\nERROR (semantic) in file " << compiler.getInputFileName()
	     <<	" near line " << compiler.parser.getNumOfLines() << "\n\n"
	     << "Description: " << s << "\n       Line: " << compiler.parser.getLine() 
	     << "\n      Error: " ;

	for (i=0; i < compiler.parser.getLinePos(); i++)
		cerr << '-' ;

	cerr << "^\n" ;

	numOfErrors++;

	if (numOfErrors>MAX_NUM_OF_ERRORS)
	{	
		cerr << "\nFATAL ERROR\n\n"
		     << "Description: " << "Too many errors to continue compiling \n"
		     << "\nTerminating compilation with errors \n\n";

		exit(1);
	}
}

void AslCompiler::operativeError(char* s, char* f)
{
	cerr << "\nERROR (operative) in function " << f << "\n\n"
	     << "Description: " << s
	     << "\n      Error: " ;

	perror("");
	cerr << "\nTerminating compilation with errors \n\n";

	exit(1);
}

void AslCompiler::putInOutFile(char *s)
{
	fprintf(ml.yyout,"%s",s);
}

void AslCompiler::deleteTempFiles(void)
{
	char tempFileName[STR_LEN_LINE];

	for (int i=0; i < numOfFiles; i++)
	{
		// Seguti 2000
		#ifndef NSL_PC
			sprintf(tempFileName, "%s_%d_temp.c", fileNames[i], getpid());
			unlink(tempFileName);
		#else
			sprintf(tempFileName, "%s_temp.cpp", fileNames[i]);
		#endif
		if (compiler.isVerboseSet())
			cerr << "Deleting temporal file" << tempFileName <<"\n";
		// Seguti 2000
	}
}

int AslCompiler::getNumOfErrors(void)
{
	return numOfErrors;
}

void AslCompiler::printCompilerStatistics(void)
{
	cerr << "\nCompiler Statistics:\n";
	cerr << "Models " << compiler.parser.getNumOfModels() << ":\n";
	cerr << "Schemas " << compiler.parser.getNumOfSchemas() << ":\n\n";

	parser.schemaTree.startOnTop();
	parser.printSchemaList();
}



// Main compiler function
int main(int argc, char *argv[])
{
  #ifndef NSL_PC
	extern int yydebug; // from y.tab.c 
	yydebug = 1;
  #else
	ml.yydebug = 1;
	mp.yydebug = 1;
  #endif

	int files;
	compiler.parseArguments(argc, argv);
	files = compiler.getNumOfFiles();
	for (int i=0; i < files; i++)
	{
		compiler.setInputFileName(compiler.getFileName(i));
		if (compiler.isVerboseSet())
			cerr << "Transforming NSLM file " << compiler.getInputFileName() <<" to C++ \n";
		compiler.preCompile();
		#ifndef NSL_PC
			compiler.parser.parse();
		#else
			int n = YYEXIT_FAILURE;
			if (!mp.yycreate(&ml)) 
			{
				return 0;
			}
			if (!ml.yycreate(&mp)) 
			{
				return 0;
			}
			/* Commented out Seguti 2000
			n = parser.yyparse();
			return n;
			*/		
			mp.yyparse();
		#endif
	}
	compiler.deleteTempFiles();
	if (compiler.isVerboseSet())
		compiler.printCompilerStatistics();
	if (compiler.getNumOfErrors()>0)
		exit(1);	
	return 0;
}


