
//  ASL COMPILER
//
//  ASL (Abstract Schema Language) to c++ compiler
//
//
//
//  FILE NAME   : aslc.h
//  MODULE TYPE : Header of main module
//  STATUS      : In development
//  PURPOSE     : Main module definitions
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyrigth 1995  
//
//
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            0.00  (Apr 20, 1992 - Longguang Wu), 
//            Start of development.
//
//            0.50  (May 05, 1993 - Roger Zimmermann),
//            Version 0.5 alpha.
//
//            1.00  (Dec 17, 1995 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution. 
//
//	          1.10  (Feb 16, 1998 - Sebastian Gutierrez),
//	          Sintax extended according to the NSLC 3.0 specifications.
// 

#ifndef STR_LEN_LINE
#define STR_LEN_LINE 256
#endif

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "aslbtree.h"
#include "aslstack.h"

// Seguti2000
#ifdef NSL_PC
	//#include <GetOpt.h>
	//#include <unistd.h>
	#include <clex.h>
	#include <cyacc.h>
#endif
// Seguti2000



const int MAX_OF_INCLUDES   = 10;	// Maximum number of include paths
const int MAX_OF_DEFINES    = 10;	// Maximum number of defines

// Seguti2000
#ifndef MAX_NUM_OF_ERRORS
	const int MAX_NUM_OF_ERRORS = 100;	// Maximum number of permited errors
#endif
// Seguti2000

// Boolean constants
const int TRUE  = 1;
const int FALSE = 0;

// Symbol type constants.
const int SCHEMA_SYMBOL     = 0;
const int STRUCT_SYMBOL     = 1;
const int FUNCTION_SYMBOL   = 2;
const int IDENT_SYMBOL      = 3;

// Classes prototype.
class AslCompiler;
class AslSymbol;
class AslSchemaSymbol;
class AslIdentifierSymbol;
class AslFunctionSymbol;
class AslStructSymbol;


// Classes for symbol management.
class AslSymbol
{
	char *name;								// Symbol name.
	int  symbolType;						// Symbol type: Schema
 
  public:
	AslSymbol(void);
	void  setName(char*s);					// Set symbol name
	char* getName(void);					// Get symbol name
	void setSymbolType(int t);				// Set symbol type
	int  getSymbolType(void);				// Get symbol type
	virtual int operator < (AslSymbol);		// Overloaded operators 
	virtual int operator > (AslSymbol);		// for searching in the
	virtual int operator == (AslSymbol);	// symbol tree
	virtual void print(void);				// Print symbol information
};


class AslIdentifierSymbol : public AslSymbol
{
	char* type;								// Identifier type
	int  scope;								// Identifier scope

  public:
	AslIdentifierSymbol(void);
	void setScope(int);						// Set identifier scope.
	int  getScope(void);					// Get identifier scope.
	void  setType(char*);					// Set identifier type.
	char* getType(void);					// Get identifier type.
	void print(void);						// Print identifier information.
};


class AslComposedSymbol : public AslSymbol
{
  public:
	AslBTree<AslIdentifierSymbol> identifierTree;	// Identifier symbol tree.
};


class AslStructSymbol : public AslComposedSymbol
{
  public:
	AslStructSymbol(void);
	AslBTree<AslIdentifierSymbol> identifierTree;	// Identifier symbol tree.
};


class AslFunctionSymbol : public AslComposedSymbol
{
  public:
	AslFunctionSymbol(void);
};


class AslSchemaSymbol : public AslComposedSymbol
{
	int  prototype;					// Determine if prototye
									
  public:
	AslSchemaSymbol(void);
	AslBTree<AslStructSymbol>     structTree;	// Struct symbol tree
	AslBTree<AslFunctionSymbol>   functionTree;	// Function symbol tree
	void setPrototype(int p);					// Set if it is prototye or not
	int  isPrototype(void);						// Determine if it is prototype or not
};


// Classes used by the parser
class AslArgument					// AslArgument hold a list of arguments for every Schema
{
  public:
	int argFlag;					// FALSE if SchemaArgument, TRUE if ExtendsArgument
	char type[STR_LEN_LINE];			
	char access[STR_LEN_LINE];
	char ident[STR_LEN_LINE];
	AslArgument *next;

  public:
	AslArgument(void);
	~AslArgument(void);
	void setArgument(char*,char*,char*,int);
	char *getType(void);
	char *getAccess(void);
	char *getIdent(void);
	int  getArgFlag(void);
};


class AslObject						// AslObject hold a list of objects for every Schema
{
  public:
	char type[STR_LEN_LINE];
	char access[STR_LEN_LINE];
	char name[STR_LEN_LINE];
	char objectArguments[STR_LEN_LINE]; 
	AslObject *nextObject;

  public:
	AslObject(void);
	~AslObject(void);
	void setParam(char*,char*,char*);
	void setObjectArguments(char*);
	char *getObjectType(void);
	char *getObjectAccess(void);
	char *getObjectName(void);
	char *getObjectArguments(void);
};


class AslMethod						// AslMethod holds the methods for every Schema
{
  public:
	char type[STR_LEN_LINE];
	char access[STR_LEN_LINE];
	char name[STR_LEN_LINE];
	char methodArguments[STR_LEN_LINE]; 
	AslMethod *nextMethod;

  public:
	AslMethod(void);
	~AslMethod(void);
	void setParam(char*,char*,char*);
	void setMethodArguments(char*);
	char *getMethodType(void);
	char *getMethodAccess(void);
	char *getMethodName(void);
	char *getMethodArguments(void);
};

class AslParser								// : public yyparser
{
	int numOfLines;       					// Line counter (for error flagging)
	int linePosition;     					// Position in the actual line (for error flagging)
	int access;	     						// Determine if analized members are external or internal
	int scope;	      						// Determine the scope of the analized member
	int schemaIndex;      					// Schema counter
	int modelIndex;							// Model counter
	int argIndex;							// Arguments counter
	int objectIndex;						// Objects counter
	int methodIndex;						// Methods counter
	char line[STR_LEN_LINE];				// Line buffer (for error flagging).
	char actualAccessOption[STR_LEN_LINE];	// Actual accessOption
	char actualExtends[STR_LEN_LINE];		// Actual inherit
	char actualIdentifier[STR_LEN_LINE];	// Actual identifier
	char actualModel[STR_LEN_LINE];			// Actual model_name
	char actualModelExtends[STR_LEN_LINE];	// Actual model inherit
	char actualSchema[STR_LEN_LINE];		// Actual schema_name
	char actualType[STR_LEN_LINE];			// Actual type
	AslComposedSymbol* actualSymbol[2];		// Parent symbol.		
	
	// Seguti2000
	#ifdef NSL_PC
		yylexer* tml;
	#endif
	// Seguti2000

  public:
	AslArgument *arglist,*mylast;		// Pointers to Argumentlist 
	AslObject *objectlist,*objlast;		// Pointers to Objectlist
	AslMethod *methodlist,*methodlast;	// Pointers to Methodlist
	AslParser(void);
	#ifndef NSL_PC
		static int (*parse)(void); 		// Yacc parser
	#endif
	void  saveLine(char*);				// Save actual line being analized
	char *getLine(void);				// Get actual line being analized
	void  incNumOfLines(void);			// Increment line counter
	void  setNumOfLines(int);			// Set line counter
	int   getNumOfLines(void);			// Get line counter
	void  setActualType(char*);			// Set actual type being analized
	char *getActualType(void);			// Get actual type being analized
	void  setActualIdentifier(char*);	// Save last identifier being analized
	char *getActualIdentifier(void);	// Get last indetifier being analized
	void  setLinePos(int);				// Set actual line position
	int   getLinePos(void);				// Get actual line position
	void  addLinePos(int);				// Move actual line position
	void  setLexState(int);				// Lexical analizer state setting
	void  incNumOfSchemas(void);		// Increment schema counter
	int   getNumOfSchemas(void);		// Get schema counter
	void  incNumOfModels(void);			// Increment model counter
	int   getNumOfModels(void);			// Get model counter
	void  setAccess(int);				// Set actual schema access
	void  incScope(void);				// Increment member scope
	void  decScope(void);				// Decrement member scope
	int   getScope(void);				// Get actual scope
	void  pushActualSymbol(AslComposedSymbol*); // Save actual analized symbol
	AslComposedSymbol* popActualSymbol(void);	// Get actual analized symbol
	AslBTree<AslSchemaSymbol> schemaTree;   	// Schema symbol tree
	void  insertSchema(char*, int);				// Insert schema symbol in the tree
	AslSchemaSymbol *searchSchema(char*);		// Search schema symbol in the tree
	void  printSchemaList(void);				// Print schema tree (debugging)
	void  insertIdentifier(char*);				// Insert identifier symbol in the tree
	AslIdentifierSymbol *searchIdentifier(char*);	// Search identifier symbol in the tree
	void  printIdentifierList(void);			// Print identifier tree (debugging)
	void  insertFunction(char*);				// Insert identifier symbol in the tree
	AslFunctionSymbol *searchFunction(char*);	// Search identifier symbol in the tree
	void processIncludeLines(char*);			// Include lines to find which file is compiling
	int   whichTypeIs(char*);					// Determine which type the identifier is
	void  setActualSchema(char*);				// Set schema symbol
	char *getActualSchema(void);				// Get schema symbol
	void  setActualAccessOption(char*);			// Set accessOption
	char *getActualAccessOption();				// Get accessOption
	void  makeConstructorDecl(char*);			// Make constructor declaration
	void  makeDestructorDecl(char*);			// Make destructor declaration
	void  makeConstructor(char*);				// Make constructor
	void  makeDestructor(char*);				// Make destructor	
	AslArgument* insertArg(char*,char*,char*,int);	// Insert an argument in the argumentlist
	AslArgument* removeArg(void);				// Remove the first argument of the argumentlist
	void getSchemaArguments(void);				// Get all the Schema arguments and put them in Constructor
	void getSchemaArgumentsDecl(void);			// Get all the Schema arguments and put them in Constructor decl
	void getExtendsArguments(void);				// Get all the Extends arguments and put them in Constructor
	AslObject* insertObject(char*,char*,char*);	// Insert an object in the list of the Schema
	AslObject* insertObjectParam(char*,char*,char*); //Insert the parameters of the object
	AslObject* removeObject(void);				// Remove the object from the list
	void insertObjectArguments(char*);			// Insert the arguments of the object
	void getObjects(void);						// print the list of objects
	AslMethod* insertMethod(char*,char*,char*);	// Insert a method in the list of the Schema
	void insertMethodParam(char*,char*,char*);// Insert the parameters of the	method
	AslMethod* removeMethod(void);				// Remove the method from the list
	void insertMethodArguments(char*);			// Insert the arguments of the method
	void getMethods(void);						// print the list of methods
	void setActualExtends(char*);				// Set inherit definition	
	void setActualExtendsArgs(char*);			// Set inherit argumentList
	void getActualExtends(void);				// Get inherit definition
	void getSuperClass(void);					// Get Superclass name
	int  ExistSuper(void); 						// Determine if exist inherit 
	void  setActualModel(char*);				// Set model_name
	char *getActualModel(void);					// Get model_name
	void  makeModelConstructorDecl(char*);		// Make model constructor declaration
	void  makeModelDestructorDecl(char*);		// Make model destructor declaration
	void  makeModelConstructor(char*);			// Make model constructor
	void  makeModelDestructor(char*);			// Make model destructor	
	void  makeModelDecl(char*);					// Make model declaration
	void setActualModelExtends(char*);			// Set model_inherit definition	
	void getActualModelExtends(void);			// Get model_inherit definition	
	void getExtendsModelArguments(void);		// Get all the Extends arguments and put them in Constructor
	
	/* Commented out Seguti 2000
	#ifdef NSL_PC
		void yyaction(int) {}
		int yywork(void) { return 0; }
		void yydestructclearin(void) { }
	#endif
	*/
};


//  Class for compiler management
class AslCompiler					// : public yylexer
{
	int numOfFiles;					// Number of input files
	int numOfDefines;				// Number of defines
	int numOfIncludes;				// Number of includes
	int logging;					// If true redirect standar error to logName
	int numOfErrors;				// Num of current errors
	int verbose;					// If true write information about compiling process
	char logName[STR_LEN_LINE];		// Log file name
	char defines[MAX_OF_INCLUDES][STR_LEN_LINE];	// Includes vector
	char includes[MAX_OF_DEFINES][STR_LEN_LINE];	// Defines vector
	char **fileNames;						// Asl input files
	char inputFileName[STR_LEN_LINE];
	char outputFileName[STR_LEN_LINE];		// Output c++ file name
	FILE *logFile;							// File that would contain standard error messages

  public:	
	AslCompiler(void);
	AslParser parser;					// Asl parser and it's functions
	char buf[STR_LEN_LINE];				// General propose buffer
	void  copyRight(void);
	void  help(void);
	void  parseArguments(int,char**);	// Parse line argument
	void  preCompile(void);				// Input file precompiling
	int   isVerboseSet(void);			// Know if verbose flag is set
	int   getNumOfFiles(void);			// Get number of input files
	char* getFileName(int);				// Get file name with index i
	int   getNumOfErrors(void);			// Get number of compile errors
	void  setInputFileName(char*);			// Set input file name
	char* getInputFileName(void);			// Get input file name
	void  setOutputFileName(char*);			// Set output file name
	char* getOutputFileName(void);			// Set output file name
	void  sintaxError(char *);			// Sintax error flagging
	void  semanticError(char *);			// Semantic error flagging
	void  operativeError(char *, char *);		// Operative error flagging
	void  deleteTempFiles();			// Remove temporal files
	void  putInOutFile(char*);			// Write in the c++ output file
	void  printCompilerStatistics(void);		// Print compiler statistics
	
	/* Commented out Seguti 2000
	void  makeConstructor(char*);			// Make Constructor
	void  makeDestructor(char*);			// Make Destructor
  #ifdef NSL_PC
	int yylex(void) { return 0; }
	int yyaction(int) { return 0; }
  #endif
	*/
};


// Global variables.
extern AslCompiler compiler;
extern int flagObject;
extern int flagExtends;
extern int flagAssign;
extern int flagSchema;
extern int flagMethod;
extern char myArgument[STR_LEN_LINE];
extern char myBuffer[STR_LEN_LINE];
extern char myStack[STR_LEN_LINE];
extern char myAssign[STR_LEN_LINE];