%{

//  ASL COMPILER
//
//  ASL (Abstract Schema Language) to C++ compiler
//
//
//
//  FILE NAME   : aslclex.l
//  MODULE TYPE : Lex input specification
//  STATUS      : In development
//  PURPOSE     : Lexical analisys
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyrigth 1995-2000
//
//
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            0.00  (Apr 20, 1992 - Longguang Wu), 
//            Start of development.
//
//            0.50  (May 05, 1993 - Roger Zimmermann),
//            Version 0.5 alpha.
//
//            1.00  (Dec 17, 1995 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution.
//
//	      1.05  (Oct 09, 1997 - Oscar Peguero),
//	      Changes for the NSLC 3.0 specifications.
//
//	      1.10  (Feb 16, 1998 - Sebastian Gutierrez),
//	      Compiler reconstruction & sintax extended according to the NSLC 3.0 specifications.
// 

#include <string.h>
#include "aslc.h"

// Seguti2000
//#include "y.tab.h"        // Generated by yacc.   
// Seguti2000

// Seguti2000
#include <clex.h>
#include "aslcyacc.h"		// Generated by yacc.

// Global variables.
extern AslCompiler compiler;
extern int flagObject;
extern int flagAssign;
extern int flagSchema;
extern char myArgument[STR_LEN_LINE];
extern char myStack[STR_LEN_LINE];
extern char myAssign[STR_LEN_LINE];
// Seguti2000

%}

/* Lex definitions. */

COMMENT1	\/\*[^(\*\/)]*\*\/

COMMENT_INIT	\/\*

COMMENT_END		[^(\*\/)]*\*\/

COMMENT2	\/\/[^\n]*\n
	       
	       /* Identifiers: variables, keywords, etc. */
NAME           [a-zA-Z\_][a-zA-Z0-9\_]* 

	       /* Integer constants. */
NUMBER         [0-9]

	       /* String constants. */
STRING         \"[^\"]*\"

	       /* Special characters. */
SPECIAL        [\&\=\{\}\|\[\]\\\;\'\:\?\.\"]

MYSPECIAL	[\(\)\,\*\-\+\%\^\!\>\<\/]

/*seguti2000*/
%name mylexer
/*seguti2000*/

%x ST_SCHEMA ST_VERBOSE ST_SUPER ST_INCLUDE ST_COMMENT ST_SCHEMA_COMMENT

%%

/*seguti2000*/
%{
// extract yylval for use later on in actions
YYSTYPE& yylval = *(YYSTYPE*)yyparserptr->yylvalptr; 
%}
/*seguti2000*/

"#".*\n                        {
					compiler.parser.processIncludeLines(yytext);
					sprintf(compiler.buf,"#line %d \"%s\"\n", compiler.parser.getNumOfLines(), compiler.getInputFileName());
					/*compiler.putInOutFile(compiler.buf);*/
				}

\n.*                            { 
					compiler.putInOutFile("\n");
					compiler.parser.saveLine(yytext+1);
					compiler.parser.incNumOfLines();
					compiler.parser.setLinePos(0);
					yyless(1);
				}
					
verbose_NSLC			{
					yyless(yyleng);
					compiler.parser.setLexState(ST_VERBOSE);
				}

<ST_VERBOSE>verbose_off		{
					yyless(yyleng);
					compiler.parser.setLexState(0);
				}

import				{	
					compiler.parser.addLinePos(yyleng);
					compiler.putInOutFile("#include \"");
					compiler.parser.setLexState(ST_INCLUDE);
				}

<ST_INCLUDE>"."			{
					compiler.parser.addLinePos(yyleng);
					compiler.putInOutFile("/");
				}

<ST_INCLUDE>{NAME}              {
					compiler.parser.addLinePos(yyleng);
					compiler.putInOutFile(yytext);
				}
							
<ST_INCLUDE>[ \t\r]           	{
					/* Ignore blanks. 
					compiler.putInOutFile(yytext);*/
				}

<ST_INCLUDE>";"           	{
					/* Ignore colon 
					compiler.putInOutFile(yytext);*/
				}

<ST_INCLUDE>"\n"		{
					compiler.putInOutFile(".h\"\n");
					compiler.parser.saveLine(yytext+1);
					compiler.parser.incNumOfLines();
					compiler.parser.setLinePos(0);
					yyless(1);
					compiler.parser.setLexState(0);
				}
			
nslModel			{				
					compiler.parser.addLinePos(yyleng);
					compiler.parser.setLexState(ST_SCHEMA);

					return MODEL;
				}			
			
							
nslModule			{				
					compiler.parser.addLinePos(yyleng);
					compiler.parser.setLexState(ST_SCHEMA);

					return SCHEMA;
				}
				
nslClass			{
					compiler.parser.addLinePos(yyleng);
					compiler.parser.setLexState(ST_SCHEMA);
					
					return SCHEMA;
				}
				
external			{
					compiler.parser.addLinePos(yyleng);
				}

internal			{
					compiler.parser.addLinePos(yyleng);
				}
				
static				{}				

{NAME}                          {
					compiler.putInOutFile(yytext);
				}

{STRING}                        {
					/* String constant */
					compiler.putInOutFile(yytext);
				}

{COMMENT_INIT}           	{
					compiler.putInOutFile(yytext);
					compiler.parser.setLexState(ST_COMMENT);
				}
	
<ST_COMMENT>{COMMENT_END}           	{
					compiler.putInOutFile(yytext);
					compiler.parser.setLexState(0);
				}

{COMMENT2}           	{
					compiler.putInOutFile(yytext);
				}

"-"?{NUMBER}+([ULul]*)?         {
					/* Integer constant */
					compiler.putInOutFile(yytext);
				}

"-"?"0x"({NUMBER}|[a-fA-F])+([ULul]*)? {
					/* Integer constant (hex) */
					compiler.putInOutFile(yytext);
				}

"-"?{NUMBER}*"."{NUMBER}+((e|E)"-"?{NUMBER}+)?([fFlL]*)?        {
					/* Float constant */
					compiler.putInOutFile(yytext);
				}

"-"?{NUMBER}+((e|E)"-"?{NUMBER}+)([fFlL]*)?     {
					/* Float constant */
					compiler.putInOutFile(yytext);
				}

"'"."'"                         {
					/* Character constant */
					compiler.putInOutFile(yytext);
				}

[ \t\r]                         {
					/* Ignore blanks. */
					compiler.putInOutFile(yytext);
				}

{SPECIAL}                       {
					compiler.buf[0]=yytext[0];
					compiler.buf[1]='\0';
					compiler.putInOutFile(compiler.buf);
				} 
				
.                               {
					if (isprint(yytext[0]))
						sprintf(compiler.buf,"Illegal character '%c' (%x)\n",yytext[0], yytext[0]);
					else
						sprintf(compiler.buf,"Illegal character (\\%x)\n",yytext[0]);

					compiler.parser.addLinePos(yyleng);
					compiler.sintaxError(compiler.buf);
				}

<ST_SCHEMA>{COMMENT_INIT}           	{
					compiler.putInOutFile(yytext);
					compiler.parser.setLexState(ST_SCHEMA_COMMENT);
				}
	
<ST_SCHEMA_COMMENT>{COMMENT_END}           	{
					compiler.putInOutFile(yytext);
					compiler.parser.setLexState(ST_SCHEMA);
				}

<ST_SCHEMA>{COMMENT2}           	{
					compiler.putInOutFile(yytext);
				}

<ST_SCHEMA>"#".*\n		{
					compiler.parser.processIncludeLines(yytext+1);
					sprintf(compiler.buf,"#line &d \"%s\"",compiler.parser.getNumOfLines(),compiler.getInputFileName());
					compiler.putInOutFile(compiler.buf);
				}

<ST_SCHEMA>\n.*                 {
					compiler.putInOutFile("\n");
					compiler.parser.saveLine(yytext+1);
					compiler.parser.incNumOfLines();
					compiler.parser.setLinePos(0);
					yyless(1);
				}

<ST_SCHEMA>super		{
	
					compiler.parser.getSuperClass();
					compiler.parser.addLinePos(yyleng);
					compiler.parser.setLexState(ST_SUPER);
					
					return SUPERSCHEMA_NAME;
				}
				
<ST_SUPER>"."			{
					
					compiler.parser.addLinePos(yyleng);
					compiler.parser.setLexState(ST_SCHEMA);
					
					return INDIRECTION_1;
				}

<ST_SUPER>[^"."]		{
					sprintf(compiler.buf,"Super Class sintax error\n");
					compiler.sintaxError(compiler.buf);
					compiler.parser.addLinePos(yyleng);
					compiler.parser.setLexState(ST_SCHEMA);
					
				}

<ST_SCHEMA>constructor		{}
				
<ST_SCHEMA>delegate             {
					compiler.parser.addLinePos(yyleng);

					return DELEGATE;
				}

<ST_SCHEMA>destructor           {}

<ST_SCHEMA>external             {	
					compiler.parser.addLinePos(yyleng);

					return EXTERNAL;
				}				
				
<ST_SCHEMA>extends              {
					compiler.parser.addLinePos(yyleng);

					return INHERIT;
				}
				
<ST_SCHEMA>input                {
					compiler.parser.addLinePos(yyleng);

					return INPUT;
				}
				
<ST_SCHEMA>internal             {
					compiler.parser.addLinePos(yyleng);

					return INTERNAL;
				}

<ST_SCHEMA>nslModule		{
					compiler.parser.addLinePos(yyleng);

					return SCHEMA;
				}							
				
<ST_SCHEMA>output               {
					compiler.parser.addLinePos(yyleng);

					return OUTPUT;
				}

<ST_SCHEMA>private              {
					compiler.parser.addLinePos(yyleng);

					return INTERNAL;
				}

<ST_SCHEMA>protected            {
					compiler.parser.addLinePos(yyleng);

					return PROTECTED;
				}
				
<ST_SCHEMA>public               {
					compiler.parser.addLinePos(yyleng);

					return EXTERNAL;
				}
				
<ST_SCHEMA>sin                  {}
				
<ST_SCHEMA>sout                 {}
							
<ST_SCHEMA>stop                 {
					compiler.parser.addLinePos(yyleng);

					return STOP;
				}				
				
<ST_SCHEMA>this                 {
					strcpy(myStack,"*");
					strcat(myStack,yytext);
					compiler.parser.addLinePos(yyleng);
				
					return THIS;
				}

<ST_SCHEMA>self                 {
					strcpy(myStack,"*this");
					compiler.parser.addLinePos(yyleng);
				
					return THIS;
				}																																									
				
<ST_SCHEMA>">=>"                {
					compiler.parser.addLinePos(yyleng);

					return OP_CONNECT_LR;
				}

<ST_SCHEMA>"<=<"                {
					compiler.parser.addLinePos(yyleng);

					return OP_CONNECT_RL;
				}

<ST_SCHEMA>">=<"                {
					compiler.parser.addLinePos(yyleng);

					return OP_DISCONNECT;
				}

<ST_SCHEMA>"==="                {
					compiler.parser.addLinePos(yyleng);

					return OP_RELABEL;
				}

<ST_SCHEMA>"=|="                {
					compiler.parser.addLinePos(yyleng);

					return OP_DELABEL;
				}

<ST_SCHEMA>auto                 {
					compiler.parser.addLinePos(yyleng);

					return AUTO;
				}

<ST_SCHEMA>break                {
					compiler.parser.addLinePos(yyleng);

					return BREAK;
				}

<ST_SCHEMA>case                 {
					compiler.parser.addLinePos(yyleng);

					return CASE;
				}

<ST_SCHEMA>char                 {
					compiler.parser.addLinePos(yyleng);

					return CHAR;
				}

<ST_SCHEMA>const                {
					compiler.parser.addLinePos(yyleng);

					return CONST;
				}

<ST_SCHEMA>continue             {
					compiler.parser.addLinePos(yyleng);

					return CONTINUE;
				}

<ST_SCHEMA>default              {
					compiler.parser.addLinePos(yyleng);

					return DEFAULT;
				}

<ST_SCHEMA>delete               {
					compiler.parser.addLinePos(yyleng);

					return DELETE;
				}
				
<ST_SCHEMA>do                   {
					compiler.parser.addLinePos(yyleng);

					return DO;
				}

<ST_SCHEMA>double               {
					compiler.parser.addLinePos(yyleng);

					return DOUBLE;
				}

<ST_SCHEMA>else                 {
					compiler.parser.addLinePos(yyleng);

					return ELSE;
				}

<ST_SCHEMA>extern               {
					compiler.parser.addLinePos(yyleng);

					return EXTERN;
				}

<ST_SCHEMA>float                {
					compiler.parser.addLinePos(yyleng);

					return FLOAT;
				}

<ST_SCHEMA>for                  {
					compiler.parser.addLinePos(yyleng);

					return FOR;
				}

<ST_SCHEMA>if                   {
					compiler.parser.addLinePos(yyleng);

					return IF;
				}
		    
<ST_SCHEMA>int                  {
					compiler.parser.addLinePos(yyleng);
					
					return INT;
				}

<ST_SCHEMA>long                 {
					compiler.parser.addLinePos(yyleng);

					return LONG;
				}
			
<ST_SCHEMA>new                  {
					compiler.parser.addLinePos(yyleng);

					return NEW;
				}

<ST_SCHEMA>boolean           {
					compiler.parser.addLinePos(yyleng);

					return NSLBOOLEAN;
				}

<ST_SCHEMA>NslData         	{
					compiler.parser.addLinePos(yyleng);
	
					return NSLDATA;
				}

<ST_SCHEMA>NslDinDouble       	{
					compiler.parser.addLinePos(yyleng);

					return NSLDINDOUBLE;
				}

<ST_SCHEMA>NslDinDouble0       	{
					compiler.parser.addLinePos(yyleng);

					return NSLDINDOUBLE;
				}

<ST_SCHEMA>NslDinDouble1       	{
					compiler.parser.addLinePos(yyleng);

					return NSLDINDOUBLE1;
				}

<ST_SCHEMA>NslDinDouble2       	{
					compiler.parser.addLinePos(yyleng);

					return NSLDINDOUBLE2;
				}

<ST_SCHEMA>NslDinDouble3       	{
					compiler.parser.addLinePos(yyleng);

					return NSLDINDOUBLE3;
				}

<ST_SCHEMA>NslDinDouble4       	{
					compiler.parser.addLinePos(yyleng);

					return NSLDINDOUBLE4;
				}

<ST_SCHEMA>NslDinFloat     	{
					compiler.parser.addLinePos(yyleng);

					return NSLDINFLOAT;
				}

<ST_SCHEMA>NslDinFloat0       	{
					compiler.parser.addLinePos(yyleng);

					return NSLDINFLOAT;
				}

<ST_SCHEMA>NslDinFloat1       	{
					compiler.parser.addLinePos(yyleng);

					return NSLDINFLOAT1;
				}

<ST_SCHEMA>NslDinFloat2       	{
					compiler.parser.addLinePos(yyleng);

					return NSLDINFLOAT2;
				}

<ST_SCHEMA>NslDinFloat3    	{
					compiler.parser.addLinePos(yyleng);

					return NSLDINFLOAT3;
				}

<ST_SCHEMA>NslDinFloat4     	{
					compiler.parser.addLinePos(yyleng);

					return NSLDINFLOAT4;
				}

<ST_SCHEMA>NslDinInt     	{
					compiler.parser.addLinePos(yyleng);

					return NSLDININT;
				}

<ST_SCHEMA>NslDinInt0    	{
					compiler.parser.addLinePos(yyleng);

					return NSLDININT;
				}

<ST_SCHEMA>NslDinInt1    	{
					compiler.parser.addLinePos(yyleng);

					return NSLDININT1;
				}

<ST_SCHEMA>NslDinInt2     	{
					compiler.parser.addLinePos(yyleng);

					return NSLDININT2;
				}

<ST_SCHEMA>NslDinInt3     	{
					compiler.parser.addLinePos(yyleng);

					return NSLDININT3;
				}

<ST_SCHEMA>NslDinInt4     	{
					compiler.parser.addLinePos(yyleng);

					return NSLDININT4;
				}

<ST_SCHEMA>NslDoutDouble       	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTDOUBLE;
				}

<ST_SCHEMA>NslDoutDouble0      	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTDOUBLE;
				}

<ST_SCHEMA>NslDoutDouble1      	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTDOUBLE1;
				}

<ST_SCHEMA>NslDoutDouble2      	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTDOUBLE2;
				}

<ST_SCHEMA>NslDoutDouble3      	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTDOUBLE3;
				}

<ST_SCHEMA>NslDoutDouble4      	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTDOUBLE4;
				}

<ST_SCHEMA>NslDoutFloat         {
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTFLOAT;
				}

<ST_SCHEMA>NslDoutFloat0       	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTFLOAT;
				}

<ST_SCHEMA>NslDoutFloat1       	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTFLOAT1;
				}

<ST_SCHEMA>NslDoutFloat2       	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTFLOAT2;
				}

<ST_SCHEMA>NslDoutFloat3    	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTFLOAT3;
				}

<ST_SCHEMA>NslDoutFloat4     	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTFLOAT4;
				}

<ST_SCHEMA>NslDoutInt     	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTINT;
				}

<ST_SCHEMA>NslDoutInt0    	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTINT;
				}

<ST_SCHEMA>NslDoutInt1    	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTINT1;
				}

<ST_SCHEMA>NslDoutInt2     	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTINT2;
				}

<ST_SCHEMA>NslDoutInt3     	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTINT3;
				}

<ST_SCHEMA>NslDoutInt4     	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUTINT4;
				}

<ST_SCHEMA>NslDouble           	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUBLE;
				}

<ST_SCHEMA>NslDouble0          	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUBLE0;
				}

<ST_SCHEMA>NslDouble1          	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUBLE1;
				}

<ST_SCHEMA>NslDouble2          	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUBLE2;
				}

<ST_SCHEMA>NslDouble3          	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUBLE3;
				}

<ST_SCHEMA>NslDouble4          	{
					compiler.parser.addLinePos(yyleng);

					return NSLDOUBLE4;
				}
				
<ST_SCHEMA>NslFile         	{
					compiler.parser.addLinePos(yyleng);

					return NSLFILE;
				}


<ST_SCHEMA>NslFloat            	{
					compiler.parser.addLinePos(yyleng);

					return NSLFLOAT;
				}

<ST_SCHEMA>NslFloat0           	{
					compiler.parser.addLinePos(yyleng);

					return NSLFLOAT0;
				}

<ST_SCHEMA>NslFloat1           	{
					compiler.parser.addLinePos(yyleng);

					return NSLFLOAT1;
				}

<ST_SCHEMA>NslFloat2           	{
					compiler.parser.addLinePos(yyleng);

					return NSLFLOAT2;
				}

<ST_SCHEMA>NslFloat3           	{
					compiler.parser.addLinePos(yyleng);

					return NSLFLOAT3;
				}

<ST_SCHEMA>NslFloat4           	{
					compiler.parser.addLinePos(yyleng);

					return NSLFLOAT4;
				}

<ST_SCHEMA>NslInt           	{
					compiler.parser.addLinePos(yyleng);

					return NSLINT;
				}

<ST_SCHEMA>NslInt0          	{
					compiler.parser.addLinePos(yyleng);

					return NSLINT0;
				}

<ST_SCHEMA>NslInt1         	{
					compiler.parser.addLinePos(yyleng);

					return NSLINT1;
				}

<ST_SCHEMA>NslInt2          	{
					compiler.parser.addLinePos(yyleng);

					return NSLINT2;
				}

<ST_SCHEMA>NslInt3         	{
					compiler.parser.addLinePos(yyleng);

					return NSLINT3;
				}

<ST_SCHEMA>NslInt4         	{
					compiler.parser.addLinePos(yyleng);

					return NSLINT4;
				}

<ST_SCHEMA>NslMatrix         	{
					compiler.parser.addLinePos(yyleng);

					return NSLMATRIX;
				}

<ST_SCHEMA>charString           {
					compiler.parser.addLinePos(yyleng);

					return NSLCHARSTRING;
				}

<ST_SCHEMA>NslString0           {
					compiler.parser.addLinePos(yyleng);

					return NSLSTRING0;
				}

<ST_SCHEMA>NslString1           {
					compiler.parser.addLinePos(yyleng);

					return NSLSTRING1;
				}

<ST_SCHEMA>NslVector         	{
					compiler.parser.addLinePos(yyleng);

					return NSLVECTOR;
				}

<ST_SCHEMA>register             {
					compiler.parser.addLinePos(yyleng);

					return REGISTER;
				}

<ST_SCHEMA>return               {
					compiler.parser.addLinePos(yyleng);

					return RETURN;
				}

<ST_SCHEMA>short                {
					compiler.parser.addLinePos(yyleng);

					return SHORT;
				}

<ST_SCHEMA>signed               {
					compiler.parser.addLinePos(yyleng);

					return SIGNED;
				}

<ST_SCHEMA>sizeof               {
					compiler.parser.addLinePos(yyleng);

					return SIZEOF;
				}

<ST_SCHEMA>static               {
					compiler.parser.addLinePos(yyleng);
					return STATIC;
				}

<ST_SCHEMA>switch               {
					compiler.parser.addLinePos(yyleng);

					return SWITCH;
				}

<ST_SCHEMA>unsigned             {
					compiler.parser.addLinePos(yyleng);

					return UNSIGNED;
				}

<ST_SCHEMA>virtual             {
					compiler.parser.addLinePos(yyleng);

					return VIRTUAL;
				}

<ST_SCHEMA>void                 {
					compiler.parser.addLinePos(yyleng);

					return VOID;
				}

<ST_SCHEMA>volatile             {
					compiler.parser.addLinePos(yyleng);

					return VOLATILE;
				}

<ST_SCHEMA>while                {
					compiler.parser.addLinePos(yyleng);

					return WHILE;
				}

<ST_SCHEMA>initRun		{	
					strcpy(myStack,yytext);
					compiler.parser.addLinePos(yyleng);
					strcpy(yylval.name,yytext);
					
					return INIT_RUN;
				}

<ST_SCHEMA>initSys		{	
					strcpy(myStack,yytext);
					compiler.parser.addLinePos(yyleng);
					strcpy(yylval.name,yytext);
					
					return INIT_SYS;
				}	

<ST_SCHEMA>initTrain		{	
					strcpy(myStack,yytext);
					compiler.parser.addLinePos(yyleng);
					strcpy(yylval.name,yytext);
					
					return INIT_TRAIN;
				}	
<ST_SCHEMA>simRun		{	
					strcpy(myStack,yytext);
					compiler.parser.addLinePos(yyleng);
					strcpy(yylval.name,yytext);
					
					return SIM_RUN;
				}

<ST_SCHEMA>simTrain		{	
					strcpy(myStack,yytext);
					compiler.parser.addLinePos(yyleng);
					strcpy(yylval.name,yytext);
					
					return SIM_TRAIN;
				}	
				
<ST_SCHEMA>endRun		{	
					strcpy(myStack,yytext);
					compiler.parser.addLinePos(yyleng);
					strcpy(yylval.name,yytext);
					
					return END_RUN;
				}

<ST_SCHEMA>endSys		{	
					strcpy(myStack,yytext);
					compiler.parser.addLinePos(yyleng);
					strcpy(yylval.name,yytext);
					
					return END_SYS;
				}	

<ST_SCHEMA>endTrain		{	
					strcpy(myStack,yytext);
					compiler.parser.addLinePos(yyleng);
					strcpy(yylval.name,yytext);
					
					return END_TRAIN;
				}					

<ST_SCHEMA>"*="                 {
					compiler.parser.addLinePos(yyleng);

					return MULT_EQUAL;
				}

<ST_SCHEMA>"/="                 {
					compiler.parser.addLinePos(yyleng);

					return DIV_EQUAL;
				}

<ST_SCHEMA>"%="                 {
					compiler.parser.addLinePos(yyleng);

					return MOD_EQUAL;
				}

<ST_SCHEMA>"+="                 {
					compiler.parser.addLinePos(yyleng);

					return PLUS_EQUAL;
				}

<ST_SCHEMA>"-="                 {
					compiler.parser.addLinePos(yyleng);

					return MINUS_EQUAL;
				}

<ST_SCHEMA>">>="                {
					compiler.parser.addLinePos(yyleng);

					return SHIFT_RIGHT_EQUAL;
				}

<ST_SCHEMA>"<<="                {
					compiler.parser.addLinePos(yyleng);

					return SHIFT_LEFT_EQUAL;
				}

<ST_SCHEMA>"&="                 {
					compiler.parser.addLinePos(yyleng);

					return AND_EQUAL;
				}

<ST_SCHEMA>"^="                 {
					compiler.parser.addLinePos(yyleng);

					return XOR_EQUAL;
				}

<ST_SCHEMA>"|="                 {
					compiler.parser.addLinePos(yyleng);

					return OR_EQUAL;
				}

<ST_SCHEMA>"||"                 {
					compiler.parser.addLinePos(yyleng);

					return OR_LOGIC;
				}

<ST_SCHEMA>"&&"                 {
					compiler.parser.addLinePos(yyleng);

					return AND_LOGIC;
				}

<ST_SCHEMA>"=="                 {
					compiler.parser.addLinePos(yyleng);

					return EQUAL;
				}

<ST_SCHEMA>"!="                 {
					compiler.parser.addLinePos(yyleng);

					return NOT_EQUAL;
				}

<ST_SCHEMA>"<<"                 {
					compiler.parser.addLinePos(yyleng);

					return SHIFT_LEFT;
				}

<ST_SCHEMA>">>"                 {
					compiler.parser.addLinePos(yyleng);

					return SHIFT_RIGHT;
				}

<ST_SCHEMA>"<="                 {
					compiler.parser.addLinePos(yyleng);

					return LOWER_EQUAL;
				}

<ST_SCHEMA>">="                 {
					compiler.parser.addLinePos(yyleng);

					return GREATER_EQUAL;
				}

<ST_SCHEMA>"++"                 {
					compiler.parser.addLinePos(yyleng);

					return PLUS_PLUS;
				}

<ST_SCHEMA>"--"                 {
					compiler.parser.addLinePos(yyleng);

					return MINUS_MINUS;
				}

<ST_SCHEMA>"."	                {
					strcpy(myStack,yytext);
					compiler.parser.addLinePos(yyleng);

					return INDIRECTION_1;
				}

<ST_SCHEMA>"->"	                {
					strcpy(myStack,yytext);
					compiler.parser.addLinePos(yyleng);

					return INDIRECTION_1;
				}


<ST_SCHEMA>".*"                 {
					compiler.parser.addLinePos(yyleng);

					return INDIRECTION_2;
				}

<ST_SCHEMA>"->*"                {
					compiler.parser.addLinePos(yyleng);

					return INDIRECTION_3;
				}

<ST_SCHEMA>"\"*\""              {
					compiler.parser.addLinePos(yyleng);

					return SELF;
				}
				
<ST_SCHEMA>"@"			{
					compiler.parser.addLinePos(yyleng);
					
					return NSLCONV;
				}
								

<ST_SCHEMA>{NAME}               {
				  int type;
				  strcpy(myBuffer,"");
				  /*printf("{NAME} %s\n",yytext);*/
				  if (!flagObject)
				    {
				      if (!flagSchema) strcpy(myBuffer,yytext);
				      /*if ((flagSchema)&&(flagExtends)) strcpy(myBuffer,yytext);*/
				    }
			          if ((flagObject) && (flagAssign))
				    {
					/*compiler.putInOutFile(yytext);*/
				   	if (flagSchema) 
					  {
					   	strcpy(myBuffer,yytext);
						flagSchema = FALSE;
						flagAssign = FALSE;
					  }
				    }
				  compiler.parser.addLinePos(yyleng);
				  strcpy(yylval.name,yytext);
				  type=compiler.parser.whichTypeIs(yytext);
				  switch (type)
				    {
					case SCHEMA_SYMBOL: 	
					  {
					   /*printf("\n SCHEMA %s\n",yylval.name);*/
					   return SCHEMA_NAME;
					  }
					case IDENT_SYMBOL:
					  {
					   /*printf("ID_DECLARED %s\n",yylval.name);*/
					   return IDENTIFIER_DECLARED;
					  }
					default:
					  {
					   /*printf("IDENTIFIER %s\n",yylval.name);*/
					   return IDENTIFIER;
					  }
								
				    }
				}

<ST_SCHEMA>{STRING}             {
				  strcpy(myBuffer,"");					
				  if (flagObject) 
				    {
					strcpy(myStack,yytext) ;
				    }
				  else 
				    {
					strcpy(myStack,"");
					if (!flagSchema) strcpy(myBuffer,yytext);
				    }
				  compiler.parser.addLinePos(yyleng);

				  return CONST_STRING;
				}

<ST_SCHEMA>{NUMBER}+([ULul]*)?  {
				  strcpy(myBuffer,"");
				  if (flagObject)
				    {
					strcpy(myStack,yytext);
				    }
				  else 
				    {
					strcpy(myStack,"");
					if (!flagSchema) strcpy(myBuffer,yytext);
				    }
				  compiler.parser.addLinePos(yyleng);
					
				  return CONST_INT;
				}

<ST_SCHEMA>"0x"({NUMBER}|[a-fA-F])+([ULul]*)?   {
				  	strcpy(myBuffer,"");
					if (flagObject)
					  {
						strcpy(myStack,yytext);
					  }
					else
					  {
						strcpy(myStack,"");
						if (!flagSchema) strcpy(myBuffer,yytext);
					  }
					compiler.parser.addLinePos(yyleng);

					return CONST_INT;
				}

<ST_SCHEMA>{NUMBER}*"."{NUMBER}+((e|E)"-"?{NUMBER}+)?([fFlL]*)? {
					strcpy(myBuffer,"");
					if (flagObject)
					  {
						strcpy(myStack,yytext);
					  }
					else
					  {
						strcpy(myStack,"");
						if (!flagSchema) strcpy(myBuffer,yytext);
					  }
					compiler.parser.addLinePos(yyleng);

					return CONST_FLOAT;
				}

<ST_SCHEMA>{NUMBER}+((e|E)"-"?{NUMBER}+)([fFlL]*)? {
					strcpy(myBuffer,"");
					if (flagObject)
					  {
						strcpy(myStack,yytext);
					  }
					else
					  {
						strcpy(myStack,"");
						if (!flagSchema) strcpy(myBuffer,yytext);
					  }
					compiler.parser.addLinePos(yyleng);

					return CONST_FLOAT;
				}

<ST_SCHEMA>"'"."'"              {
				  strcpy(myBuffer,"");	
				  if (flagObject)
				    {
					strcpy(myStack,yytext);
				    }
				  else
				    {
					strcpy(myStack,"");
					if (!flagSchema) strcpy(myBuffer,yytext);
				    }
				  compiler.parser.addLinePos(yyleng);
					
				  return CONST_CHAR;
				}

<ST_SCHEMA>[ \t\r]              {
					compiler.putInOutFile(yytext);
					compiler.parser.addLinePos(yyleng);

					/* Ignore blanks. */
				}


<ST_SCHEMA>{SPECIAL}            {
				  compiler.buf[0]=yytext[0];
			 	  compiler.buf[1]='\0';
				  if ((strcmp(yytext,"=") == 0) && (!flagAssign) && (flagObject)) 
				    {
					/*printf("= (flagAssign)\n"); strcpy(myAssign,"");*/
					flagAssign = TRUE;
				    }/*
				  if ((strcmp(yytext,";") == 0) && (flagAssign)) 
				    {
					/*printf("; (flagAssign)\n");*
					strcat(myAssign,";\n");
				    }* 
				  if (flagAssign) strcat(myAssign,";\n");*/
				    
				  compiler.parser.addLinePos(yyleng);
					
				  return yytext[0];
				}

<ST_SCHEMA>{MYSPECIAL}        	{
				  if (!flagObject)
				    {
					compiler.buf[0]=yytext[0];
					compiler.buf[1]='\0';	
					if ((strcmp(yytext,"(") == 0) && (flagAssign == 2)) 
					  {
						/*printf("(\n");*/
						flagAssign = FALSE;
					  }
				    }
				  compiler.parser.addLinePos(yyleng);

				  return yytext[0];
				}

<ST_SCHEMA>.                    {
					if (isprint(yytext[0]))
						sprintf(compiler.buf,"Illegal character '%c' (%x)\n",yytext[0],yytext[0]);
					else
						sprintf(compiler.buf,"Illegal character (\\%x)\n",yytext[0]);

					compiler.sintaxError(compiler.buf);
					compiler.parser.addLinePos(yyleng);
				}
%%

int yywrap(void)
{
	return TRUE;
}

void AslParser::setLexState(int p)
{
	// Seguti2000
	//BEGIN(p);
	tml->yybegin(p);
	// Seguti2000
}
