%{

//  ASL COMPILER
//
//  ASL (Abstract Schema Language) to c++ compiler
//
//
//
//  FILE NAME   : aslcyacc.y
//  MODULE TYPE : Yacc input specification
//  STATUS      : In development
//  PURPOSE     : Yacc source to generate ASL parser
//  
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyrigth 1995-2000 
//
// 
//  ASL syntax according to:
//
//	      "ASL: A Hierarchical Computational Model for Distributed Heterogeneous Systems" 
//	      by Alfredo Weitzenfeld.
//	
//  NSL Syntax according to:
//
//            "NSLC 3.0 specifications"
//	      by Alfredo Weitzenfeld.
//		
//
//
//  HISTORY:  Version (Date-Name) Modification, description. 
//
//            0.00  (Apr 20, 1992 - Longguang Wu), 
//            Start of development.
//
//            0.50  (May 05, 1993 - Roger Zimmermann),
//            Version 0.5 alpha.     
//
//            1.00  (Dec 17, 1995 - Claudia Calderas & Salvador Marmol), 
//            Compiler reconstruction, sintax extended.
//
//	      1.05  (Oct 09, 1997 - Oscar Peguero),
//	      Sintax according to the NSLC 3.0 specifications.
//
//	      1.10  (Feb 16, 1998 - Sebastian Gutierrez),
//	      Compiler reconstruction & sintax extended according to the NSLC 3.0 specifications.
//
//
//  Shift/Reduce conflicts:
//
//	      - Pointers' vector and simple pointers.
//	      - Casting an expression and declaring something;


//#include "aslc.h"
// Seguti2000
//#include "asldef.h"
#include "asldef2.h"
#include "aslc.h"

class myparser;
// Seguti2000

void yyerror(char const *s)
{
	//compiler.sintaxError(s); 
}

%}
// Seguti2000
%name myparser
// Seguti2000

%start schema

%union
{
   char  name[STR_LEN_LINE];     // String constants and identifiers.
}

/* commented out Seguti 2000
#ifndef YYSTYPE
#define STR_LEN_LINE 256
union tagYYSTYPE 
{
   char  name[STR_LEN_LINE];     // String constants and identifiers.
};
#define YYSTYPE union tagYYSTYPE
#endif
}*/


/* Constants */

%token  <name>  CONST_CHAR 
%token  <name>  CONST_INT
%token  <name>  CONST_FLOAT
%token  <name>  CONST_STRING


/* Simple types */

%token  <name>  INT 
%token  <name>  CHAR
%token  <name>  DOUBLE 
%token  <name>  FLOAT 
%token  <name>  VOID
%token  <name>  SHORT 
%token  <name>  LONG 
%token  <name>  SIGNED 
%token  <name>  UNSIGNED


/* Nsl data types & operators*/

%token  <name>  NSLBOOLEAN
%token  <name>  NSLCHARSTRING
/*%token  <name>  NSLCONST	/* ???    */	
%token  <name>  NSLCONV		/*  "@"   */
%token  <name>  NSLDATA
%token  <name>  NSLDINDOUBLE
%token  <name>  NSLDINDOUBLE1
%token  <name>  NSLDINDOUBLE2
%token  <name>  NSLDINDOUBLE3
%token  <name>  NSLDINDOUBLE4
%token  <name>  NSLDINFLOAT
%token  <name>  NSLDINFLOAT1
%token  <name>  NSLDINFLOAT2
%token  <name>  NSLDINFLOAT3
%token  <name>  NSLDINFLOAT4
%token  <name>  NSLDININT
%token  <name>  NSLDININT1
%token  <name>  NSLDININT2
%token  <name>  NSLDININT3
%token  <name>  NSLDININT4
%token  <name>  NSLDOUTDOUBLE
%token  <name>  NSLDOUTDOUBLE1
%token  <name>  NSLDOUTDOUBLE2
%token  <name>  NSLDOUTDOUBLE3
%token  <name>  NSLDOUTDOUBLE4
%token  <name>  NSLDOUTFLOAT
%token  <name>  NSLDOUTFLOAT1
%token  <name>  NSLDOUTFLOAT2
%token  <name>  NSLDOUTFLOAT3
%token  <name>  NSLDOUTFLOAT4
%token  <name>  NSLDOUTINT
%token  <name>  NSLDOUTINT1
%token  <name>  NSLDOUTINT2
%token  <name>  NSLDOUTINT3
%token  <name>  NSLDOUTINT4
%token  <name>  NSLDOUBLE
%token  <name>  NSLDOUBLE0
%token  <name>  NSLDOUBLE1
%token  <name>  NSLDOUBLE2
%token  <name>  NSLDOUBLE3
%token  <name>  NSLDOUBLE4
%token  <name>  NSLFILE
%token  <name>  NSLFLOAT
%token  <name>  NSLFLOAT0
%token  <name>  NSLFLOAT1
%token  <name>  NSLFLOAT2
%token  <name>  NSLFLOAT3
%token  <name>  NSLFLOAT4
%token  <name>  NSLINT
%token  <name>  NSLINT0
%token  <name>  NSLINT1
%token  <name>  NSLINT2
%token  <name>  NSLINT3
%token  <name>  NSLINT4
%token  <name>  NSLMATRIX
%token  <name>  NSLSTRING0
%token  <name>  NSLSTRING1
%token  <name>  NSLVECTOR


/* Nsl control methods */

%token  <name>  INIT_RUN
%token  <name>  INIT_SYS
%token  <name>  INIT_TRAIN
%token  <name>  SIM_RUN
%token  <name>  SIM_TRAIN
%token  <name>  END_RUN
%token  <name>  END_SYS
%token  <name>  END_TRAIN


/* Type and variable names */

%token  <name>  SCHEMA_NAME
%token  <name>  SUPERSCHEMA_NAME
%token  <name>  PORT_NAME
%token  <name>  IDENTIFIER_DECLARED
%token  <name>  IDENTIFIER


/* C++ tokens */

%token  <name>  AUTO
%token  <name>  BREAK
%token  <name>  CASE
%token  <name>  CONST
%token  <name>  CONTINUE
%token  <name>  DEFAULT
%token  <name>  DELETE
%token  <name>  DO
%token  <name>  ELSE
%token  <name>  ENUM
%token  <name>  EXTERN
%token  <name>  FOR
%token  <name>  IF
%token  <name>  NEW
%token  <name>  REGISTER
%token  <name>  RETURN
%token  <name>  SIZEOF
%token  <name>  STATIC
%token  <name>  SWITCH
%token  <name>  VIRTUAL
%token  <name>  VOLATILE
%token  <name>  WHILE


/* C++ operators */

%token  <name>  MULT_EQUAL          /*  "*="  */
%token  <name>  DIV_EQUAL           /*  "/="  */
%token  <name>  MOD_EQUAL           /*  "%="  */
%token  <name>  PLUS_EQUAL          /*  "+="  */
%token  <name>  MINUS_EQUAL         /*  "-="  */
%token  <name>  SHIFT_RIGHT_EQUAL   /*  ">>=" */
%token  <name>  SHIFT_LEFT_EQUAL    /*  "<<=" */
%token  <name>  AND_EQUAL           /*  "&="  */
%token  <name>  XOR_EQUAL           /*  "^="  */
%token  <name>  OR_EQUAL            /*  "|="  */
%token  <name>  OR_LOGIC            /*  "||"  */
%token  <name>  AND_LOGIC           /*  "&&"  */
%token  <name>  EQUAL               /*  "=="  */
%token  <name>  NOT_EQUAL           /*  "!="  */
%token  <name>  SHIFT_LEFT          /*  "<<"  */
%token  <name>  SHIFT_RIGHT         /*  ">>"  */
%token  <name>  LOWER_EQUAL         /*  "<="  */
%token  <name>  GREATER_EQUAL       /*  ">="  */
%token  <name>  PLUS_PLUS           /*  "++"  */
%token  <name>  MINUS_MINUS         /*  "--"  */
%token  <name>  INDIRECTION_1       /*  "."   */
%token  <name>  INDIRECTION_2       /*  ".*"  */
%token  <name>  INDIRECTION_3       /*  "->*" */


/* ASL tokens */

%token  <name>  DELEGATE
%token  <name>  EXTERNAL 
%token  <name>  INHERIT
%token  <name>  INTERNAL
%token  <name>  INPUT 
%token  <name>  MODEL
%token  <name>  OUTPUT
%token  <name>  PROTECTED
%token  <name>  SCHEMA
%token  <name>  SELF
%token  <name>  SIN 
%token  <name>  SOUT
%token  <name>  STOP
%token  <name>  THIS


/* ASL operators */

%token  <name>  OP_CONNECT_LR	/*  ">=>"  */ 
%token  <name>  OP_CONNECT_RL	/*  "<=<"  */
%token  <name>  OP_DISCONNECT 	/*  ">=<"  */
%token  <name>  OP_RELABEL 	/*  "==="  */
%token  <name>  OP_DELABEL	/*  "=|="  */


/* Types */

%type <name> accessOption
%type <name> classStorageSpecifier
%type <name> controlMethods
%type <name> declSpecifierList
%type <name> declSpecifier
%type <name> nslCtrlDecl
%type <name> nslType
%type <name> objectTypeName
%type <name> pointerOperator
%type <name> primitiveType
%type <name> simpleTypeName
%type <name> typeSpecifier

%%


/* Module declaration */

schema		: module
			| model
			| accessSpecifier
			  schema
			;

model		: modelSpecifier	{ 
						  compiler.parser.setLexState(0);        
						  compiler.parser.incNumOfModels();
						  flagAssign = 2;
						}

			| MODEL 
			  IDENTIFIER 		{ 
			  			  compiler.parser.insertSchema($2,TRUE); 
						  compiler.putInOutFile("class ");
						  compiler.putInOutFile(myBuffer);
						}
			  ';' 			{ 
			  			  compiler.putInOutFile(";");
			  			  compiler.parser.setLexState(0);
						  flagAssign = 2; 
						}

			| MODEL		
			  SCHEMA_NAME		{
			  			  compiler.putInOutFile("class ");
						  compiler.putInOutFile(myBuffer);
						}
			  ';' 			{ 
			  			  compiler.putInOutFile(";");
			  			  compiler.parser.setLexState(0);
			  			  flagAssign = 2; 
						}
						
			| schema 
			  modelSpecifier 	{
						  compiler.parser.setLexState(0);
						  compiler.parser.incNumOfModels();
						  flagAssign = 2;
						}

			| schema 
			  MODEL 
			  IDENTIFIER 		{ 
			  			  compiler.parser.insertSchema($3,TRUE); 
						  compiler.putInOutFile("class ");
						  compiler.putInOutFile(myBuffer);
						}
			  ';' 			{ 
			  			  compiler.putInOutFile(";");
			  			  compiler.parser.setLexState(0);
			  			  flagAssign = 2; 
						}

			| schema 
			  MODEL		
			  SCHEMA_NAME		{
			  			  compiler.putInOutFile("class ");
						  compiler.putInOutFile(myBuffer);
						}
			  ';' 			{ 
			  			  compiler.putInOutFile(";");
			  			  compiler.parser.setLexState(0);
			  			  flagAssign = 2; 
						}
			;

module		: moduleSpecifier	{ 
						  compiler.parser.setLexState(0);        
						  compiler.parser.incNumOfSchemas();
						  flagAssign = 2;
						}
						
			| SCHEMA 
			  IDENTIFIER 		{ 
			  			  compiler.parser.insertSchema($2,TRUE); 
						  compiler.putInOutFile("class ");
						  compiler.putInOutFile(myBuffer);
						}
			  ';' 			{ 
			  			  if (!flagSchema) compiler.putInOutFile(";");
			  			  compiler.parser.setLexState(0);
						  flagAssign = 2; 
						}

			| SCHEMA 		
			  SCHEMA_NAME		{
			  			  compiler.putInOutFile("class ");
						  compiler.putInOutFile(myBuffer);
						}
			  ';' 			{ 
			  			  if (!flagSchema) compiler.putInOutFile(";");
			  			  compiler.parser.setLexState(0);
			  			  flagAssign = 2; 
						}
						

			| schema 
			  moduleSpecifier 	{
						  compiler.parser.setLexState(0);
						  compiler.parser.incNumOfSchemas();
						  flagAssign = 2;
						}
				
			| schema 
			  SCHEMA 
			  IDENTIFIER 		{ 
			  			  compiler.parser.insertSchema($3,TRUE); 
						  compiler.putInOutFile("class ");
						  compiler.putInOutFile(myBuffer);
						}
			  ';' 			{ 
			  			  if (!flagSchema) compiler.putInOutFile(";");
			  			  compiler.parser.setLexState(0);
			  			  flagAssign = 2; 
						}

			| schema 
			  SCHEMA		
			  SCHEMA_NAME		{
			  			  compiler.putInOutFile("class ");
						  compiler.putInOutFile(myBuffer);
						}
			  ';' 			{ 
			  			  if (!flagSchema) compiler.putInOutFile(";");
			  			  compiler.parser.setLexState(0);
			  			  flagAssign = 2; 
						}
			;


modelSpecifier		: modelHeader 
			    			{ 
			  			  compiler.putInOutFile("\n{");
						  compiler.parser.makeModelConstructorDecl(compiler.parser.getActualModel());
					  	  compiler.parser.makeModelDestructorDecl(compiler.parser.getActualModel());
					  	  strcpy(myAssign,"");
						  flagMethod = FALSE;
						}
			  		  memberList 
			  		  '}' 			{
			  			  compiler.putInOutFile("};");
						  compiler.parser.makeModelConstructor(compiler.parser.getActualModel());
					  	  compiler.parser.makeModelDestructor(compiler.parser.getActualModel());
						  compiler.parser.makeModelDecl(compiler.parser.getActualModel());
					 	}
					;

moduleSpecifier		: moduleHeader 
			  		  '{'			{ 
			  			  if (!flagSchema) compiler.putInOutFile("{");
						  compiler.parser.makeConstructorDecl(compiler.parser.getActualSchema());
					  	  compiler.parser.makeDestructorDecl(compiler.parser.getActualSchema());
					  	  strcpy(myAssign,"");
						  flagMethod = FALSE;
						}
			  		  memberList 
			  		  '}' 			{
			  			  if (!flagSchema) compiler.putInOutFile("}");
			  			  compiler.putInOutFile(";");
						  compiler.parser.makeConstructor(compiler.parser.getActualSchema());
					  	  compiler.parser.makeDestructor(compiler.parser.getActualSchema());
						}
					;

modelHeader			: MODEL			
			  		  IDENTIFIER 		{ 
						  compiler.parser.insertSchema($2,FALSE); 
						  compiler.parser.setActualModel($2);
						  compiler.putInOutFile("class ");
						  compiler.putInOutFile(myBuffer);
						}
			  		  modelBaseSpecifier
			  		  '{'
			 
					| MODEL 		
			  		  SCHEMA_NAME 		{ 
						  AslSchemaSymbol *res;
						  res = compiler.parser.searchSchema($2);
						  if (!res->isPrototype())
							{
							  sprintf(compiler.buf,"Schema [%s] already defined", $2);
							  compiler.semanticError(compiler.buf);
							}
						  else
							{
							  res->setPrototype(FALSE);
							} 
						  compiler.putInOutFile("class ");
						  compiler.putInOutFile(myBuffer);
						}
			  		  modelBaseSpecifier
			  		  '{'		
			   
					| error			{ compiler.sintaxError("Model Header sintax error"); }
					;

modelBaseSpecifier	: /* optional */ 	{compiler.parser.setActualModelExtends("");flagSchema=FALSE;}
					| INHERIT 		{ 
						  flagSchema=FALSE; 
						  flagObject=FALSE;
						  compiler.putInOutFile(" : public ");
						}
			  		  modelBaseList		{flagSchema=TRUE;}
			  		  extendsArgs  		{ flagSchema=FALSE;}
					;

modelBaseList		: SCHEMA_NAME			{ 
							  compiler.parser.setActualModelExtends($1);
							  compiler.putInOutFile(myBuffer);
							}

					| IDENTIFIER 			{ 
							  compiler.putInOutFile(myBuffer);
							  compiler.parser.setActualModelExtends($1);
							}

					| modelBaseList ',' IDENTIFIER 	{ 
							  if((!flagObject)&&(!flagSchema)) compiler.putInOutFile(",");
							  compiler.putInOutFile(myBuffer);
							  sprintf(compiler.buf,"Identifier [%s] is not a schema", $3);
							  compiler.semanticError(compiler.buf);
				     		}
					;



moduleHeader		: SCHEMA		
			  		  IDENTIFIER 		{
						  flagSchema=TRUE;
						  compiler.parser.insertSchema($2,FALSE); 
						  compiler.parser.setActualSchema($2);
						  compiler.putInOutFile("class ");
						  compiler.putInOutFile(myBuffer);
						}
			  		  construct_args	{ flagSchema=FALSE;}	
			  		  baseSpecifier		{ flagSchema=FALSE;}

					| SCHEMA 	
			  		  SCHEMA_NAME 		{
						  flagSchema=TRUE;
						  AslSchemaSymbol *res;
						  res = compiler.parser.searchSchema($2);
						  if (!res->isPrototype())
							{
							  sprintf(compiler.buf,"Schema [%s] already defined", $2);
							  compiler.semanticError(compiler.buf);
							}
						  else
							{
							  res->setPrototype(FALSE);
							} 
						  compiler.putInOutFile("class ");
						  compiler.putInOutFile(myBuffer);
						}

			  		  construct_args	{ flagSchema=FALSE;}
			  		  baseSpecifier		{ flagSchema=FALSE;}
			   
					| error			{ compiler.sintaxError("Module Header sintax error"); }
					;

construct_args		: '(' 			{
						  if (!flagSchema) compiler.putInOutFile("(");
						}
			  		  argumentList 
			  		  ')'			{ 
			  			  if (!flagSchema) compiler.putInOutFile(")");
						  flagSchema=FALSE; 
						}
					;


baseSpecifier		: /* optional */ 		{ compiler.parser.setActualExtends(""); flagSchema=FALSE;}
					| INHERIT 			{ 
							  compiler.putInOutFile(": public");
							  flagSchema=FALSE; 
							  flagObject=FALSE;
							}
			  		  baseList 
			  		  extendsArgs  			{ flagSchema=FALSE;}
					;

baseList			: SCHEMA_NAME			{ 
							  compiler.parser.setActualExtends($1);
							  flagSchema=TRUE;
							  compiler.putInOutFile(myBuffer);
							  /*printf("SCHEMA %s\n",myBuffer);*/
							}

					| IDENTIFIER 			{ 
						  	  compiler.parser.setActualExtends($1);
							  flagSchema=TRUE;
							  compiler.putInOutFile(myBuffer);
							  
							  /*compiler.putInOutFile(myBuffer);
							  sprintf(compiler.buf,"Identifier [%s] is not a schema", $1);
					   	  	  compiler.semanticError(compiler.buf);*/
							}

					| baseList ',' IDENTIFIER 	{ 
							  if((!flagObject)&&(!flagSchema)) compiler.putInOutFile(",");
							  compiler.putInOutFile(myBuffer);
							  sprintf(compiler.buf,"Identifier [%s] is not a schema", $3);
							  compiler.semanticError(compiler.buf);
				     			}
					;


extendsArgs			: '('				{
							  if (!flagSchema) compiler.putInOutFile("(");
							  flagExtends = TRUE;
							}
			  		  argumentList
			  		  ')'				{
			  				  if (!flagSchema) compiler.putInOutFile(")");
							  flagExtends = FALSE;
							}
					;

memberList			: /* optional */
					| memberDeclaration memberList
					| accessSpecifier memberList
					| error			{ compiler.sintaxError("Bad schema member sintax"); }
					;

accessSpecifier		: EXTERNAL	{
					  compiler.putInOutFile("public:");
					  compiler.parser.setAccess(EXT);
						}
					| INTERNAL	{ 
					  compiler.putInOutFile("private:");
					  compiler.parser.setAccess(INT);
						}
					| PROTECTED	{ 
					  compiler.putInOutFile("protected:");
					  compiler.parser.setAccess(INT);
						}
					;


/* Member declarations */


memberDeclaration	: declSpecifierList
			  declMemberList	
			  ';'			{ 
			  			  if (!flagSchema) compiler.putInOutFile(";");
						  if (flagAssign) strcat(myAssign,";\n");
			  			  flagObject = FALSE; 
			  			  flagAssign = FALSE;  
			  			}
			| declSpecifierList 	
			  declMemberList	
			  '{'			{ 
			  			  if (!flagSchema) compiler.putInOutFile("{");
			  			  compiler.parser.insertFunction(compiler.parser.getActualIdentifier()); 
						}
			  statementList		
			  '}'	  		{if (!flagSchema) compiler.putInOutFile("}");}
			| schemaMethodInv
			| superSchemaMethodInv
			| INT			{compiler.putInOutFile("virtual int ");}
			  controlMethods	
			  '{'			{ 
			  			  if (!flagSchema) compiler.putInOutFile("{");
						  compiler.parser.insertFunction(compiler.parser.getActualIdentifier()); 
						}
			  statementList		
			  '}'			{if (!flagSchema) compiler.putInOutFile("}");}
			| VOID			{compiler.putInOutFile("virtual void ");}
			  controlMethods	
			  '{'			{ 
			  			  if (!flagSchema) compiler.putInOutFile("{");
						  compiler.parser.insertFunction(compiler.parser.getActualIdentifier()); 
						}
			  statementList		
			  '}'			{if (!flagSchema) compiler.putInOutFile("}");}
			;

declSpecifierList	: declSpecifier				{ 
								  strcpy($$, $1); 
								  compiler.parser.setActualType($$); 
								}
			| declSpecifierList declSpecifier 	{ 
								  strcat($$, $2); 
								  compiler.parser.setActualType($$);
								}
			;

declSpecifier		: classStorageSpecifier		{ strcpy($$, $1);
							  if (!flagSchema)compiler.putInOutFile($$); 
							}
			| typeSpecifier 	 	{ strcpy($$, $1); }
			;

classStorageSpecifier	: AUTO		{ strcpy($$, "auto");     }
			| EXTERN	{ strcpy($$, "extern");   }
			| REGISTER	{ strcpy($$, "register"); }
			| STATIC	{ strcpy($$, "static");   }
			| VIRTUAL	{ strcpy($$, "virtual");  }
			;

accessOption		: /*optional */			{ 
							  if (flagMethod) {strcpy($$,"&");compiler.putInOutFile($$);}
							  else strcpy($$, ""); 
							  compiler.parser.setActualAccessOption($$); 
							}
			| pointerOperator		{ 
							  strcpy($$,$1);
							  compiler.parser.setActualAccessOption($$);
				    			}
			;

typeSpecifier		: primitiveType	{ 
					  strcpy($$,$1);
					}			
			| nslType  	{ 
					  flagObject=TRUE;
					  strcpy($$,$1);
					}
			| CONST		{ 
					  strcpy($$, "const ");
					  if (!flagSchema)compiler.putInOutFile($$);
					}
			;

primitiveType		: simpleTypeName accessOption 	{strcpy($$,$1);}
			;

nslType			: objectTypeName accessOption	{strcpy($$,$1);}
			;

simpleTypeName		: CHAR 		{ 
					  strcpy($$, "char");
					  if (!flagSchema)compiler.putInOutFile($$);
					  flagMethod=FALSE;
					}
			| DOUBLE 	{ 
					  strcpy($$, "double ");
					  if (!flagSchema)compiler.putInOutFile($$);
					  flagMethod=FALSE;
					}
			| FLOAT		{ 
					  strcpy($$, "float ");
					  if (!flagSchema)compiler.putInOutFile($$);
					  flagMethod=FALSE;
					}
			| INPUT 	{ 
					  strcpy($$, "input ");
					  if (!flagSchema)compiler.putInOutFile($$);
					  flagMethod=FALSE;
					}
			| INT 		{ 
					  strcpy($$, "int ");
					  if (!flagSchema)compiler.putInOutFile($$);
					  flagMethod=FALSE;
					}
			| LONG 		{ 
					  strcpy($$, "long ");
					  if (!flagSchema)compiler.putInOutFile($$);
					  flagMethod=FALSE;
					}
			| OUTPUT 	{ 
					  strcpy($$, "output ");
					  if (!flagSchema)compiler.putInOutFile($$);
					  flagMethod=FALSE;
					}
			| SIGNED 	{ 
					  strcpy($$, "signed ");
					  if (!flagSchema)compiler.putInOutFile($$);
					  flagMethod=FALSE;
					}
			| SHORT 	{ 
					  strcpy($$, "short ");
					  if (!flagSchema)compiler.putInOutFile($$);
					  flagMethod=FALSE;
					}
			| UNSIGNED 	{ 
					  strcpy($$, "unsigned ");
					  if (!flagSchema)compiler.putInOutFile($$);
					  flagMethod=FALSE;
					}
			| VOID		{ 
			 		  strcpy($$, "void ");
					  if (!flagSchema)compiler.putInOutFile($$);
					  flagMethod=FALSE;
					}
			| NSLBOOLEAN	{ 
					  strcpy($$, "boolean ");
					  if (!flagSchema)compiler.putInOutFile($$);
					  flagMethod=FALSE;
					}
			| NSLCHARSTRING	{ 
					  strcpy($$, "charString ");
					  if (!flagSchema)compiler.putInOutFile($$);
					  flagMethod=FALSE;
					}
			| NSLDATA	{ 
					  	  strcpy($$, "NslData ");
					   	  if (!flagSchema)compiler.putInOutFile($$);
						  flagMethod=FALSE;
					}					
			| NSLMATRIX	{ 
						  strcpy($$, "NslMatrix "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						  flagMethod=FALSE;
					}	
			| NSLVECTOR	{ 
						  strcpy($$, "NslVector "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						  flagMethod=FALSE;
					}	
			;

objectTypeName		: NSLDINDOUBLE	 	{ 
						  strcpy($$, "NslDinDouble ");
						  if (!flagSchema)compiler.putInOutFile($$);	
						}
			| NSLDINDOUBLE1	 	{ 
						  strcpy($$, "NslDinDouble1 ");
						  if (!flagSchema)compiler.putInOutFile($$); 
						}
			| NSLDINDOUBLE2	 	{ 
						  strcpy($$, "NslDinDouble2 ");
						  if (!flagSchema)compiler.putInOutFile($$); 
						}
			| NSLDINDOUBLE3	 	{ 
						  strcpy($$, "NslDinDouble3 ");
						  if (!flagSchema)compiler.putInOutFile($$); 
						}	
			| NSLDINDOUBLE4	 	{ 
						  strcpy($$, "NslDinDouble4 ");
						  if (!flagSchema)compiler.putInOutFile($$); 
						}
			| NSLDINFLOAT	 	{ 
						  strcpy($$, "NslDinFloat ");  
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDINFLOAT1	 	{ 
			  			  strcpy($$, "NslDinFloat1 "); 
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDINFLOAT2	 	{ 
						  strcpy($$, "NslDinFloat2 "); 
						  if (!flagSchema)compiler.putInOutFile($$);
						}	
			| NSLDINFLOAT3	 	{ 
						  strcpy($$, "NslDinFloat3 "); 
						  if (!flagSchema)compiler.putInOutFile($$);	
						}
			| NSLDINFLOAT4	 	{ 
						  strcpy($$, "NslDinFloat4 "); 
						  if (!flagSchema)compiler.putInOutFile($$);	
						}
			| NSLDININT	 	{ 
						  strcpy($$, "NslDinInt "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDININT1	 	{ 
						  strcpy($$, "NslDinInt1 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDININT2	  	{ 
						  strcpy($$, "NslDinInt2 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDININT3	 	{ 
						  strcpy($$, "NslDinInt3 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDININT4	 	{ 
						  strcpy($$, "NslDinInt4 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUTDOUBLE	 	{ 
						  strcpy($$, "NslDoutDouble "); 
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUTDOUBLE1 	{ 
						  strcpy($$, "NslDoutDouble1 ");
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUTDOUBLE2 	{ 
						  strcpy($$, "NslDoutDouble2 ");
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUTDOUBLE3 	{ 
						  strcpy($$, "NslDoutDouble3 ");
						  if (!flagSchema)compiler.putInOutFile($$);
						}	
			| NSLDOUTDOUBLE4 	{ 
						  strcpy($$, "NslDoutDouble4 ");
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUTFLOAT	 	{ 
						  strcpy($$, "NslDoutFloat "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUTFLOAT1 	{ 
					 	  strcpy($$, "NslDoutFloat1 "); 
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUTFLOAT2	 	{ 
						  strcpy($$, "NslDoutFloat2 "); 
						  if (!flagSchema)compiler.putInOutFile($$);
						}	
			| NSLDOUTFLOAT3	 	{ 
						  strcpy($$, "NslDoutFloat3 "); 
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUTFLOAT4	 	{ 
						  strcpy($$, "NslDoutFloat4 "); 
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUTINT	 	{ 
						  strcpy($$, "NslDoutInt "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUTINT1	 	{ 
						  strcpy($$, "NslDoutInt1 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUTINT2	 	{ 
						  strcpy($$, "NslDoutInt2 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUTINT3	 	{ 
						  strcpy($$, "NslDoutInt3 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUTINT4	 	{ 
						  strcpy($$, "NslDoutInt4 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUBLE	 	{ 
						  strcpy($$, "NslDouble "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUBLE0	 	{ 
						  strcpy($$, "NslDouble0 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUBLE1	 	{ 
						  strcpy($$, "NslDouble1 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUBLE2	 	{ 
						  strcpy($$, "NslDouble2 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUBLE3	 	{ 
						  strcpy($$, "NslDouble3 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLDOUBLE4	 	{ 
						  strcpy($$, "NslDouble4 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLFILE	 	{ 
						  strcpy($$, "NslFile "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}	
			| NSLFLOAT 		{ 
						  strcpy($$, "NslFloat "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLFLOAT0 		{ 
						  strcpy($$, "NslFloat0 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLFLOAT1 		{ 
						  strcpy($$, "NslFloat1 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLFLOAT2 	 	{ 
						  strcpy($$, "NslFloat2 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLFLOAT3	 	{ 
						  strcpy($$, "NslFloat3 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLFLOAT4	 	{ 
						  strcpy($$, "NslFloat4 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLINT	 	{ 
					 	  strcpy($$, "NslInt "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLINT0	 	{ 
						  strcpy($$, "NslInt0 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLINT1	 	{ 
						  strcpy($$, "NslInt1 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLINT2	 	{ 
						  strcpy($$, "NslInt2 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLINT3	 	{ 
						  strcpy($$, "NslInt3 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLINT4	 	{ 
						  strcpy($$, "NslInt4 "); 	
						  if (!flagSchema)compiler.putInOutFile($$);
						}		
											
			| NSLSTRING0		{ 
					  	  strcpy($$, "NslString0 ");
					   	  if (!flagSchema)compiler.putInOutFile($$);
						}
			| NSLSTRING1		{ 
					  	  strcpy($$, "NslString1 ");
					  	  if (!flagSchema)compiler.putInOutFile($$);
						}
			| SCHEMA_NAME		{ 
						  strcpy($$, $1);
						  strcat($$," ");
						  if (!flagSchema)compiler.putInOutFile($$);
						}			
			;

controlMethods		: nslCtrlDecl '(' ')'	{
			  		 	  compiler.putInOutFile(myStack);
					 	  compiler.putInOutFile("()");
						}
			;

nslCtrlDecl		: INIT_RUN
			| INIT_SYS
			| INIT_TRAIN
			| SIM_RUN
			| SIM_TRAIN
			| END_RUN
			| END_SYS
			| END_TRAIN
			;

declMemberList		: /* optional */			
			| memberDecl		
			| declMemberList 
			  ',' 			{ if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile(","); 
						  flagAssign=TRUE;
						  flagSchema=TRUE;
						}
			  memberDecl		
			;


memberDecl		: declarator  	 {
					  if (flagObject) compiler.parser.insertObject(compiler.parser.getActualType(), compiler.parser.getActualAccessOption(), compiler.parser.getActualIdentifier());
					 }				 	
			  '(' 		 { 
			  		   if (flagObject) 
					     {	
					     	strcpy(myArgument,"");
						/*strcat(myArgument,"\"");
						strcat(myArgument,compiler.parser.getActualIdentifier());
						strcat(myArgument,"\",*this");*/
					     }
					   else if (!flagSchema) compiler.putInOutFile("("); /*else if before seguti99*/
					   if ((!flagSchema)&&(!flagExtends)&&(!flagObject)) flagMethod=TRUE;
			  		 }
			 initializerList {if (flagObject) compiler.parser.insertObjectArguments(myArgument);
			 		  /*if ((!flagSchema)&&(!flagExtends)&&(!flagObject))printf("initalizerList %s\n",compiler.parser.getActualIdentifier());*/
			 		 }
			  ')' 		 {if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile(")");
			  		  if (flagMethod) flagMethod=FALSE;
			  		 }		
			 myInit 
				
			| declarator pureSpecifier				

			| IDENTIFIER 	{compiler.putInOutFile(myBuffer);}
			  ':' 		{if (!flagSchema) compiler.putInOutFile(":");}
			  constantExpression			

			| ':' 		{if (!flagSchema) compiler.putInOutFile(":");}
			  constantExpression

			| declarator declaratorAssign
			;

declaratorAssign	:'=' 	{ 
				  	  	  if (flagAssign)
						    {
							strcat(myAssign,compiler.parser.getActualIdentifier());
							strcat(myAssign," = ");
						    }
						  else if (!flagSchema) compiler.putInOutFile("=");
						}
			  constNeg constantExpression
			;	

pureSpecifier		: /* optional */ 
			| '=' CONST_INT { 
					  if (!strcmp($2, "0")) compiler.sintaxError("Pure specifier need to be '= 0'"); 
					  else 
					    {
					    	compiler.putInOutFile("=");
					    	compiler.putInOutFile(myBuffer);
					    }
					}
			;

myInit			: /* optional */
			| declaratorAssign
			| '[' 
			  IDENTIFIER 
			  ']'			/*{printf("Object Array\n");}*/
			;

declarator		: IDENTIFIER 			{ 
							  compiler.putInOutFile(myBuffer); 	
							  compiler.parser.insertIdentifier($1);
							  /*if (flagExtends) printf("%s\n",myBuffer);*/
							}

			| IDENTIFIER_DECLARED		{ 
							  if (!flagExtends) compiler.putInOutFile(myBuffer);
							  if (flagExtends) compiler.parser.insertIdentifier($1);
	
							  /*  
							  if (!flagExtends)
							    {		
								sprintf(compiler.buf,"MyIdentifier [%s] already defined", $1);
								compiler.semanticError(compiler.buf);
							    }*/
							}

			| pointerOperator declarator

			| declarator '[' 		{if (!flagSchema)compiler.putInOutFile("[");}
			  constantExpression ']'	{if (!flagSchema)compiler.putInOutFile("]");}

			| declarator '[' ']'	{if (!flagSchema) compiler.putInOutFile("[]");}

			| '(' 		{if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("(");}	
			  declarator 
			  ')'		{if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile(")");}
			;

pointerOperator		: '*' 			{if (!flagSchema)compiler.putInOutFile("*");}
			  cvQualifierList  	{strcpy($$,"*");}
			  
			| '&' 			{if (!flagSchema)compiler.putInOutFile("&");}
			  cvQualifierList 	{strcpy($$,"&");}
			;

cvQualifierList		: /* optional */
			| cvQualifier cvQualifierList 
			;

cvQualifier		: CONST		{if (!flagSchema)compiler.putInOutFile("const");}
			| VOLATILE	{if (!flagSchema)compiler.putInOutFile("volatile");}
			;

argument		: /* optional */	
			| argsDef		{
						  compiler.parser.insertArg(compiler.parser.getActualType(), compiler.parser.getActualAccessOption(), compiler.parser.getActualIdentifier(), flagExtends);
						}
			| objectInitList
			| objectMethodsInvList 
			;

argsDef			: declSpecifierList
			| declSpecifierList declarator 
			| declSpecifierList declarator '=' {if (!flagSchema) compiler.putInOutFile("=");}
			  constantExpression
			| declarator		
			;

argumentList		: argument
			| argumentList 
			  ',' 		{if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile(",");}
			  argument
			; 

objectInitList		: objectInit
			| objectInitList objectInit
			;

objectInit		: declarator declaratorAssign
			  ';'			{if (!flagSchema) compiler.putInOutFile(";");
			  			 if (flagAssign) strcat(myAssign,";\n");
			 			 printf("End...ObjectInit\n ");
						}
			;

constNeg		: /* optional*/		
			| '-'			{ 
						  if (flagAssign) strcat(myAssign,"-"); 
						  else if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("-");
						}
			;

objectMethodsInvList	: objectMethodsInv
			| objectMethodsInvList objectMethodsInv
			;

objectMethodsInv	: IDENTIFIER 	{compiler.putInOutFile(myBuffer);}
			  INDIRECTION_1 {compiler.putInOutFile(myStack);}
			  memberDecl 
			  ';'		{
			  		 if (!flagSchema) compiler.putInOutFile(";");
					 if (flagAssign) strcat(myAssign,";\n");
			  		}
			;
			
schemaMethodInv		: SCHEMA_NAME 	{compiler.putInOutFile(myBuffer);}
			  INDIRECTION_1 {compiler.putInOutFile("::");}
			  memberDecl 
			  ';'		{
			  		  if (!flagSchema) compiler.putInOutFile(";");
					  if (flagAssign) strcat(myAssign,";\n");
					}
			;

superSchemaMethodInv	: SUPERSCHEMA_NAME 	{
						 if((compiler.parser.ExistSuper())==0)
						   {
						     sprintf(compiler.buf,"Super Class not defined");
					 	     compiler.semanticError(compiler.buf);
						   }
						}

			  INDIRECTION_1 	{compiler.putInOutFile("::");}
			  memberDecl 	
			  ';'			{
			  			  if (!flagSchema) compiler.putInOutFile(";");
						  if (flagAssign) strcat(myAssign,";\n");
						}	
			;

typeSpecifierList	: /* optional */
			| typeSpecifier typeSpecifierList
			;

/* Declarations */

declaration		: declSpecifierList 
			  declaratorList 
			  ';'			{
			  			  if (!flagSchema) compiler.putInOutFile(";");
						  if (flagAssign) strcat(myAssign,";\n");
						}
			;

declaratorList		: initDeclarator
			| declaratorList 
			  ',' 			{if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile(",");}
			  initDeclarator
			;

initDeclarator		: declarator initializer
			;

initializer		: /* optional */
			| '=' 			{if (!flagSchema) compiler.putInOutFile("=");}
			  assignExpression
			| '=' 			{if (!flagSchema) compiler.putInOutFile("=");}
			  initializerTemplate
			| '(' 			{if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("(");}		
			  expressionList 
			  ')'			{if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile(")");}
			;

initializerList		: /* optional */
			| assignExpression	
			| initializerList 
			  ',' 			{
			  			  if (flagObject) strcat(myArgument,",");
						  else if (!flagSchema) compiler.putInOutFile(",");
			  			}
			  assignExpression	
			| initializerTemplate
			;

initializerTemplate	: '{' 			{if (!flagSchema) compiler.putInOutFile("{");}
			  initializerList 
			  '}'			{if (!flagSchema) compiler.putInOutFile("}");}
			;


/* Expressions */ 

expression		: portExpression
			| expressionIter
			;

portExpression		: STOP '(' 	{compiler.putInOutFile("STOP(");}
			  schemaList 
			  ')'		{compiler.putInOutFile(")");}
			| DELEGATE '(' 	{compiler.putInOutFile("DELEGATE(");}
			  schemaList 
			  ')'		{compiler.putInOutFile(")");}
			| PORT_NAME portOperator PORT_NAME
			| PORT_NAME portOperator '(' {if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("(");}
			  portList ')'	{if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile(")");}
			;

portOperator		: OP_CONNECT_LR	{compiler.putInOutFile("CONNECT_LR");}   
			| OP_CONNECT_RL	{compiler.putInOutFile("CONNECT_RL");} 
			| OP_DISCONNECT {compiler.putInOutFile("DISCONNECT");}    
			| OP_RELABEL    {compiler.putInOutFile("RELABLE");}     
			| OP_DELABEL	{compiler.putInOutFile("DELABEL");}     
			| SHIFT_RIGHT   {compiler.putInOutFile(">>");}     
			| SHIFT_LEFT    {compiler.putInOutFile("<<");}     
			;

portList		: PORT_NAME
			| portList 
			  ',' 		{if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile(",");}
			  PORT_NAME
			;

schemaList		: SCHEMA_NAME	{compiler.putInOutFile(myBuffer);}
			| schemaList 
			  ',' 		{if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile(",");}
			  SCHEMA_NAME	{compiler.putInOutFile(myBuffer);}
			;

expressionIter		: assignExpression			
			| expression 
			  ',' 		{if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile(",");}
			  assignExpression	
			;

assignExpression	: conditionalExpression	
			| unaryExpression assignOperator assignExpression	
			;

assignOperator		: '='		     {if (!flagSchema) compiler.putInOutFile("=");}
			| MULT_EQUAL         {compiler.putInOutFile("*=");}
			| DIV_EQUAL          {compiler.putInOutFile("/=");}
			| MOD_EQUAL          {compiler.putInOutFile("%=");}
			| PLUS_EQUAL         {compiler.putInOutFile("+=");}
			| MINUS_EQUAL        {compiler.putInOutFile("-=");}
			| SHIFT_RIGHT_EQUAL  {compiler.putInOutFile(">>=");}
			| SHIFT_LEFT_EQUAL   {compiler.putInOutFile("<<=");}
			| AND_EQUAL          {compiler.putInOutFile("&=");}
			| XOR_EQUAL          {compiler.putInOutFile("^=");}
			| OR_EQUAL	     {compiler.putInOutFile("|=");}
			;

conditionalExpression	: orLogicalExpression						
			| orLogicalExpression 
			  '?' 			{if (!flagSchema) compiler.putInOutFile("?");}
			  expression 
			  ':' 			{if (!flagSchema) compiler.putInOutFile(":");}
			  conditionalExpression
			;

orLogicalExpression	: andLogicalExpression
			| orLogicalExpression 
			  OR_LOGIC		{compiler.putInOutFile("||");}
			  andLogicalExpression
			;

andLogicalExpression	: inOrExpression
			| andLogicalExpression 
			  AND_LOGIC 		{compiler.putInOutFile("&&");}
			  inOrExpression
			;

inOrExpression		: xorExpression
			| inOrExpression 
			  '|' 			{if (!flagSchema) compiler.putInOutFile("|");}
			  xorExpression
			;

xorExpression		: andExpression
			| xorExpression 
			  '^' 			{if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("^");}
			  andExpression
			;

andExpression		: equalExpression
			| andExpression 
			  '&' 			{if (!flagSchema) compiler.putInOutFile("&");}
			  equalExpression
			;

equalExpression		: relationalExpression
			| equalExpression 
			  EQUAL     		{compiler.putInOutFile("==");}
			  relationalExpression
			| equalExpression 
			  NOT_EQUAL 		{compiler.putInOutFile("!=");}
			  relationalExpression
			;

relationalExpression	: moveExpression
			| relationalExpression 
			  '<'  			{if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("<");}
			  moveExpression
			| relationalExpression 
			  '>'  			{if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile(">");}
			  moveExpression
			| relationalExpression 
			  LOWER_EQUAL   	{compiler.putInOutFile("<=");}
			  moveExpression
			| relationalExpression 
			  GREATER_EQUAL 	{compiler.putInOutFile(">=");}
			  moveExpression
			;

moveExpression		: aditiveExpression
			| moveExpression 
			  SHIFT_LEFT 		{compiler.putInOutFile("<<");} 
			  aditiveExpression
			| moveExpression 
			  SHIFT_RIGHT 		{compiler.putInOutFile(">>");}
			  aditiveExpression
			;

aditiveExpression	: prodExpression
			| aditiveExpression 
			  '+' 			{ 
			  			  if (flagObject) strcat(myArgument,"+");	
			  			  if (flagAssign) strcat(myAssign,"+");
						  else if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("+");
			  			}
  			  prodExpression
			| aditiveExpression
			  '-' 			{ 
			  			  if (flagObject) strcat(myArgument,"-");
			  			  if (flagAssign) strcat(myAssign,"-");
						  else if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("-");
			  			}
			  prodExpression
			;

prodExpression		: apmExpression
			| prodExpression 
			  '*' 			{ 
			  			  if (flagObject) strcat(myArgument,"*");
			  			  if (flagAssign) strcat(myAssign,"*");
						  else if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("*");
			  			}
			  apmExpression	
			| prodExpression
			  '/' 			{ 
			  			  if (flagObject) strcat(myArgument,"/");
			  			  if (flagAssign) strcat(myAssign,"/");
						  else if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("/");
			  			}
			  apmExpression
			| prodExpression
			  '%' 			{ 
			  			  if (flagObject) strcat(myArgument,"%");
			  			  if (flagAssign) strcat(myAssign,"%");
						  else if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("%");
			  			}
			  apmExpression
			;


apmExpression		: mutationExpression
			| apmExpression 
			  INDIRECTION_2 	{compiler.putInOutFile(".*");}
			  mutationExpression
			| apmExpression 
			  INDIRECTION_3 	{compiler.putInOutFile("->*");}
			  mutationExpression
			;

mutationExpression	: unaryExpression						
			/*| '('	{if ((!flagObject)&&(!flagSchema))compiler.putInOutFile("(");}
			 typeSpecifierList 
			 ')'	{if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile(")");}	
			  mutationExpression	*/
			;

unaryExpression		: postfixExpression			
			| PLUS_PLUS   		{compiler.putInOutFile("++");}	
			  unaryExpression
			| MINUS_MINUS 		{compiler.putInOutFile("--");}
			  unaryExpression
			| unaryOperator mutationExpression
			| SIZEOF 		{compiler.putInOutFile("sizeof");}
			  mutationExpression
			| newExpression
			| deleteExpression
			;

unaryOperator		: '*'	{ 
				  if (flagObject) strcat(myArgument,"*");
				  if (flagAssign) strcat(myAssign,"*");
				  else if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("*");
				}
			| '&'	{ 
				  if (flagObject) strcat(myArgument,"&");
				  if (flagAssign) strcat(myAssign,"&");
				  else if (!flagSchema) compiler.putInOutFile("&");
				}
			| '+'	{ 
				  if (flagObject) strcat(myArgument,"+");
				  if (flagAssign) strcat(myAssign,"+");
				  else if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("+");
				}
			| '-'	{ 
				  if (flagObject) strcat(myArgument,"-");
				  if (flagAssign) strcat(myAssign,"-");
				  else if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("-");
				}
			| '!'	{ 
				  if (flagObject) strcat(myArgument,"!");
				  if (flagAssign) strcat(myAssign,"!");
				  else if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("!");
				}
			| '~'	{ 
				  if (flagObject) strcat(myArgument,"~");
				  if (flagAssign) strcat(myAssign,"~");
				  else if (!flagSchema) compiler.putInOutFile("~");
				}
			;

newExpression		: NEW 			{compiler.putInOutFile("new");}		
			  typeSpecifierList
			  newTemplate
			;

newTemplate		: /* optional*/ 
			| '(' 			{if (!flagSchema) compiler.putInOutFile("(");}			
			  initializerList 	
			  ')'			{if (!flagSchema) compiler.putInOutFile(")");}
			;

deleteExpression	: DELETE 		{compiler.putInOutFile("delete");}
			  mutationExpression
			| DELETE '[' ']'	{compiler.putInOutFile("delete[]");}
			  mutationExpression
			;

postfixExpression	: nslExpression
			| postfixExpression '[' {if (!flagSchema) compiler.putInOutFile("[");}
			  expression ']'	{if (!flagSchema) compiler.putInOutFile("]");}		
			| postfixExpression 
			  '(' 			{if (!flagSchema) compiler.putInOutFile("(");}	
			  expressionList 
			  ')'			{if (!flagSchema) compiler.putInOutFile(")");}
			| postfixExpression 
			  INDIRECTION_1 	{compiler.putInOutFile(myStack);}
			  IDENTIFIER		{compiler.putInOutFile(myBuffer);}
			| postfixExpression 
			  PLUS_PLUS		{compiler.putInOutFile("++");}
			| postfixExpression 
			  MINUS_MINUS		{compiler.putInOutFile("--");}
			;

nslExpression		: primaryExpression
			| primitiveType		
			  primaryExpression	{flagMethod=TRUE;}/* AT_SCHEMA en medio */
			| objectTypeName	
			  accessOption	
			  primaryExpression	/* AT_SCHEMA en medio */
			| schemaMethodInv
			| superSchemaMethodInv
			| IDENTIFIER NSLCONV 	{ 
			  		  	  compiler.putInOutFile("nslConv(");
					  	  compiler.putInOutFile(myBuffer);
					  	  compiler.putInOutFile(",");
			  			}
			  IDENTIFIER		{ 
			  		  	  compiler.putInOutFile(myBuffer);
					  	  compiler.putInOutFile(")");
						}
			;

primaryExpression	: literal	
			
			| SELF			{ 
						  if (flagObject) 
						    {
							strcat(myArgument,"\"");
							strcat(myArgument,compiler.parser.getActualIdentifier());
							strcat(myArgument,"\"");
						    }
						  else
						    {
							compiler.putInOutFile("\"");
							compiler.putInOutFile(compiler.parser.getActualIdentifier());
							compiler.putInOutFile("\"");
						    }
						}
			
			| THIS			{ 
						  if (flagObject) strcat(myArgument,myStack);
						  else compiler.putInOutFile(myStack);
						}
		
			| '(' 			{ if (flagObject) strcat(myArgument,"(");
						  if (flagAssign) strcat(myAssign,"(");
						  else if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile("(");
						}
			  myassignExpression
			  ')'			{ if (flagObject) strcat(myArgument,")");
			  			  if (flagAssign) strcat(myAssign,")");
						  else if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile(")");
			  			}
		
			| IDENTIFIER_DECLARED	{ if (flagObject) strcat(myArgument,$1);
						  if (flagAssign) strcat(myAssign,$1);
						  compiler.putInOutFile(myBuffer);
						} /* | AT_SCHEMA principio */	

			| IDENTIFIER		{ if (flagObject) strcat(myArgument,$1);
						  if (flagAssign) strcat(myAssign,$1);
						  compiler.putInOutFile(myBuffer);
						} /* | AT_SCHEMA principio */
			;

literal			: CONST_INT		{ 
						  if (flagObject) strcat(myArgument,myStack);
						  else compiler.putInOutFile(myBuffer);
						}
			| CONST_CHAR		{ 
						  if (flagObject) strcat(myArgument,myStack);
						  else compiler.putInOutFile(myBuffer);
						}
			| CONST_FLOAT		{ 
						  if (flagObject) strcat(myArgument,myStack);
						  else compiler.putInOutFile(myBuffer);
						}
			| CONST_STRING		{ 
						  if (flagObject) strcat(myArgument,myStack);
						  else compiler.putInOutFile(myBuffer);
						}
			;

expressionList		: /* optional */
			| assignExpression
			| expressionList 
			  ',' 			{if ((!flagObject)&&(!flagSchema)) compiler.putInOutFile(",");}
			  assignExpression
			;

constantExpression	: conditionalExpression
			;
			
myassignExpression	: typeSpecifierList
			| assignExpression
			;


/* Statements */

statement		: labelStatement
			| initForStatement
			| complexStatement    
			| selectStatement
			| iterationStatement
			| jumpStatement
			| error			/*{ compiler.sintaxError("Bad statement sintax"); }*/
			;

labelStatement		: CASE 			{compiler.putInOutFile("case");}
			  constantExpression ':'{if (!flagSchema) compiler.putInOutFile(":");}
			  statement
			| DEFAULT ':'		{compiler.putInOutFile("default:");}
			  statement
			;	

expressionStatement	: optExpression 
			  ';'			{
			  			  if (!flagSchema) compiler.putInOutFile(";");
						  if (flagAssign) strcat(myAssign,";\n");
						}
			;

complexStatement	: '{' 			{ 
						  if (!flagSchema) compiler.putInOutFile("{");
						  compiler.parser.incScope(); 
						}
			  statementList 
			  '}' 			{ 
			  			  if (!flagSchema) compiler.putInOutFile("}");
						  compiler.parser.decScope(); 
						}
			;

statementList		: /* optional */
			| statementList statement
			;

selectStatement		: IF '('	{compiler.putInOutFile("if(");}	
			  expression 
			  ')' 		{compiler.putInOutFile(")");}
			  statement
			| ELSE 		{compiler.putInOutFile("else ");}
			  statement
			| SWITCH '('	{compiler.putInOutFile("switch(");}	
			  expression ')'{compiler.putInOutFile(")");}		
			  statement
			; 

iterationStatement	: WHILE '('	{compiler.putInOutFile("while(");}	
			  expression 
			  ')' 
			  statement
			| DO 		{compiler.putInOutFile("do");}
			  statement 
			  WHILE '(' 	{compiler.putInOutFile("while");}
			  expression 
			  ')' ';'	{if (!flagSchema) compiler.putInOutFile(");");}
			| FOR '('	{compiler.putInOutFile("for(");}		
			  initForStatement 
			  optExpression 
			  ';' 			{if (!flagSchema) compiler.putInOutFile(";");}
			  optExpression ')' 	{{if (!flagSchema) compiler.putInOutFile(")");}}
			  statement
			;

optExpression		: /* optional */
			| expression
			;

initForStatement	: expressionStatement
			| declarationStatement 
			;

jumpStatement		: BREAK ';' 	{compiler.putInOutFile("break;");}
			| CONTINUE ';'	{compiler.putInOutFile("continue;");}
			| RETURN 	{compiler.putInOutFile("return ");}
			  expression 
			  ';'		{if (!flagSchema) compiler.putInOutFile(";");}
			| RETURN ';' 	{compiler.putInOutFile("return;");}		
			;

declarationStatement	: declaration 
			;
			
%%
